/*
    This file is part of KDE.

    Copyright (c) 2004 Cornelius Schumacher <schumacher@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#ifndef KODE_STATEMACHINE_H
#define KODE_STATEMACHINE_H

#include "code.h"

#include <qstring.h>
#include <qmap.h>
#include <kdepimmacros.h>

namespace KODE {

class KDE_EXPORT StateMachine
{
  public:
    StateMachine();
  
    void setState( const QString &state, const Code &code );

    void setInitialState( const QString &state );

    Code stateDefinition();
    Code transitionLogic();

  private:
    QMap<QString,Code> mStateMap;

    QString mInitialState;
};

}

#endif
