#ifndef BISON_PARSEHOLIDAY_H
# define BISON_PARSEHOLIDAY_H

#ifndef YYSTYPE
typedef union { int ival; char *sval; } yystype;
# define YYSTYPE yystype
# define YYSTYPE_IS_TRIVIAL 1
#endif
# define	NUMBER	257
# define	MONTH	258
# define	WDAY	259
# define	COLOR	260
# define	STRING	261
# define	IN	262
# define	PLUS	263
# define	MINUS	264
# define	SMALL	265
# define	CYEAR	266
# define	LEAPYEAR	267
# define	LENGTH	268
# define	EASTER	269
# define	EQ	270
# define	NE	271
# define	LE	272
# define	GE	273
# define	LT	274
# define	GT	275
# define	OR	276
# define	AND	277
# define	UMINUS	278


extern YYSTYPE kcallval;

#endif /* not BISON_PARSEHOLIDAY_H */
