/*
    This file is part of KDE Kontact.

    Copyright (c) 2003 Cornelius Schumacher <schumacher@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    As a special exception, permission is given to link this program
    with any edition of Qt, and distribute the resulting executable,
    without including the source code for Qt in the source distribution.
*/
#ifndef KONTACT_ABOUTDIALOG_H
#define KONTACT_ABOUTDIALOG_H

#include <kdialogbase.h>
class KAboutData;
namespace Kontact {

class Core;
class Plugin;

class AboutDialog : public KDialogBase
{
  Q_OBJECT

  public:
    AboutDialog( Kontact::Core *core, const char *name = 0 );

  protected:
    void addAboutPlugin( Kontact::Plugin *plugin );

    void addAboutData( const QString &title, const QString &icon,
                       const KAboutData *about );

    void addLicenseText( const KAboutData *about );

    QString formatPerson( const QString &name, const QString &email );

  private:
    Core *mCore;
};

}

#endif
