/*
    This file is part of KitchenSync.

    Copyright (c) 2004 Holger Freyther <freyther@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include "filter.h"

#include <kconfig.h>

namespace KSync {

Filter::Filter( QObject* parent, const char* name )
  : QObject( parent, name ), mConfig( 0 )
{
}

Filter::~Filter()
{
}

void Filter::load( KConfig* config ) 
{
  mConfig = config;
  doLoad();
}

void Filter::save( KConfig* config ) 
{
  mConfig = config;
  doSave();
}

QString Filter::name() const 
{
  return mName;
}

KConfig* Filter::config() 
{
  return mConfig;
}

void Filter::setName( const QString& name ) 
{
  mName = name;
}

}
