/* soapH.h
   Generated by gSOAP 2.7.0d from /build/kde/cvs/pim/kdepim/kresources/groupwise/soap/groupwise.h
   Copyright (C) 2001-2004 Genivia, Inc. All Rights Reserved.
   This software is released under one of the following three licenses:
   GPL, the gSOAP public license, or Genivia's license for commercial use.
   See README.txt for further details.
*/
#ifndef soapH_H
#define soapH_H
#include "soapStub.h"

SOAP_BEGIN_NAMESPACE(soap)
SOAP_FMAC3 int SOAP_FMAC4 soap_ignore_element(struct soap*);
SOAP_FMAC3 int SOAP_FMAC4 soap_putindependent(struct soap*);
SOAP_FMAC3 int SOAP_FMAC4 soap_getindependent(struct soap*);
SOAP_FMAC3 void SOAP_FMAC4 soap_markelement(struct soap*, const void*, int);
SOAP_FMAC3 int SOAP_FMAC4 soap_putelement(struct soap*, const void*, const char*, int, int);
SOAP_FMAC3 void * SOAP_FMAC4 soap_getelement(struct soap*, int*);
SOAP_FMAC3 int SOAP_FMAC4 soap_resolve_attachments(struct soap*);

SOAP_FMAC3 void * SOAP_FMAC4 soap_instantiate(struct soap*, int, const char*, const char*, size_t*);

SOAP_FMAC3 void SOAP_FMAC4 soap_fdelete(struct soap_clist*);

SOAP_FMAC3 void* SOAP_FMAC4 soap_class_id_enter(struct soap*, const char*, void*, int, size_t, const char*, const char*);

SOAP_FMAC3 void* SOAP_FMAC4 soap_container_id_forward(struct soap*, const char*, void*, int, int, size_t, unsigned int);

SOAP_FMAC3 void SOAP_FMAC4 soap_container_insert(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_byte
#define SOAP_TYPE_byte (2)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_byte(struct soap*, char *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_byte(struct soap*, char const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_byte(struct soap*, const char *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_byte(struct soap*, const char*, int, const char *, const char*);
SOAP_FMAC3 char * SOAP_FMAC4 soap_get_byte(struct soap*, char *, const char*, const char*);
SOAP_FMAC3 char * SOAP_FMAC4 soap_in_byte(struct soap*, const char*, char *, const char*);

#ifndef SOAP_TYPE_ns1__GMTOffset
#define SOAP_TYPE_ns1__GMTOffset (222)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__GMTOffset(struct soap*, int *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__GMTOffset(struct soap*, int const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__GMTOffset(struct soap*, const int *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__GMTOffset(struct soap*, const char*, int, const int *, const char*);
SOAP_FMAC3 int * SOAP_FMAC4 soap_get_ns1__GMTOffset(struct soap*, int *, const char*, const char*);
SOAP_FMAC3 int * SOAP_FMAC4 soap_in_ns1__GMTOffset(struct soap*, const char*, int *, const char*);

#ifndef SOAP_TYPE_int
#define SOAP_TYPE_int (1)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_int(struct soap*, int *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_int(struct soap*, int const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_int(struct soap*, const int *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_int(struct soap*, const char*, int, const int *, const char*);
SOAP_FMAC3 int * SOAP_FMAC4 soap_get_int(struct soap*, int *, const char*, const char*);
SOAP_FMAC3 int * SOAP_FMAC4 soap_in_int(struct soap*, const char*, int *, const char*);

#ifndef SOAP_TYPE_long
#define SOAP_TYPE_long (253)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_long(struct soap*, long *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_long(struct soap*, long const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_long(struct soap*, const long *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_long(struct soap*, const char*, int, const long *, const char*);
SOAP_FMAC3 long * SOAP_FMAC4 soap_get_long(struct soap*, long *, const char*, const char*);
SOAP_FMAC3 long * SOAP_FMAC4 soap_in_long(struct soap*, const char*, long *, const char*);

#ifndef SOAP_TYPE_ns1__Month
#define SOAP_TYPE_ns1__Month (228)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__Month(struct soap*, unsigned char *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__Month(struct soap*, unsigned char const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__Month(struct soap*, const unsigned char *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Month(struct soap*, const char*, int, const unsigned char *, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_get_ns1__Month(struct soap*, unsigned char *, const char*, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_in_ns1__Month(struct soap*, const char*, unsigned char *, const char*);

#ifndef SOAP_TYPE_ns1__Minute
#define SOAP_TYPE_ns1__Minute (227)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__Minute(struct soap*, unsigned char *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__Minute(struct soap*, unsigned char const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__Minute(struct soap*, const unsigned char *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Minute(struct soap*, const char*, int, const unsigned char *, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_get_ns1__Minute(struct soap*, unsigned char *, const char*, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_in_ns1__Minute(struct soap*, const char*, unsigned char *, const char*);

#ifndef SOAP_TYPE_ns1__Hour
#define SOAP_TYPE_ns1__Hour (223)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__Hour(struct soap*, unsigned char *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__Hour(struct soap*, unsigned char const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__Hour(struct soap*, const unsigned char *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Hour(struct soap*, const char*, int, const unsigned char *, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_get_ns1__Hour(struct soap*, unsigned char *, const char*, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_in_ns1__Hour(struct soap*, const char*, unsigned char *, const char*);

#ifndef SOAP_TYPE_ns1__Day
#define SOAP_TYPE_ns1__Day (217)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__Day(struct soap*, unsigned char *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__Day(struct soap*, unsigned char const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__Day(struct soap*, const unsigned char *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Day(struct soap*, const char*, int, const unsigned char *, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_get_ns1__Day(struct soap*, unsigned char *, const char*, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_in_ns1__Day(struct soap*, const char*, unsigned char *, const char*);

#ifndef SOAP_TYPE_unsignedByte
#define SOAP_TYPE_unsignedByte (13)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_unsignedByte(struct soap*, unsigned char *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_unsignedByte(struct soap*, unsigned char const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_unsignedByte(struct soap*, const unsigned char *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_unsignedByte(struct soap*, const char*, int, const unsigned char *, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_get_unsignedByte(struct soap*, unsigned char *, const char*, const char*);
SOAP_FMAC3 unsigned char * SOAP_FMAC4 soap_in_unsignedByte(struct soap*, const char*, unsigned char *, const char*);

#ifndef SOAP_TYPE_unsignedInt
#define SOAP_TYPE_unsignedInt (12)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_unsignedInt(struct soap*, unsigned int *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_unsignedInt(struct soap*, unsigned int const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_unsignedInt(struct soap*, const unsigned int *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_unsignedInt(struct soap*, const char*, int, const unsigned int *, const char*);
SOAP_FMAC3 unsigned int * SOAP_FMAC4 soap_get_unsignedInt(struct soap*, unsigned int *, const char*, const char*);
SOAP_FMAC3 unsigned int * SOAP_FMAC4 soap_in_unsignedInt(struct soap*, const char*, unsigned int *, const char*);

#ifndef SOAP_TYPE_unsignedLong
#define SOAP_TYPE_unsignedLong (254)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_unsignedLong(struct soap*, unsigned long *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_unsignedLong(struct soap*, unsigned long const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_unsignedLong(struct soap*, const unsigned long *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_unsignedLong(struct soap*, const char*, int, const unsigned long *, const char*);
SOAP_FMAC3 unsigned long * SOAP_FMAC4 soap_get_unsignedLong(struct soap*, unsigned long *, const char*, const char*);
SOAP_FMAC3 unsigned long * SOAP_FMAC4 soap_in_unsignedLong(struct soap*, const char*, unsigned long *, const char*);

#ifndef SOAP_TYPE_bool
#define SOAP_TYPE_bool (240)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_bool(struct soap*, bool *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_bool(struct soap*, bool const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_bool(struct soap*, const bool *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_bool(struct soap*, const char*, int, const bool *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_bool2s(struct soap*, bool);
SOAP_FMAC3 bool * SOAP_FMAC4 soap_get_bool(struct soap*, bool *, const char*, const char*);
SOAP_FMAC3 bool * SOAP_FMAC4 soap_in_bool(struct soap*, const char*, bool *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2bool(struct soap*, const char*, bool *);

#ifndef SOAP_TYPE_ns1__WeekDay
#define SOAP_TYPE_ns1__WeekDay (236)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__WeekDay(struct soap*, enum ns1__WeekDay *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__WeekDay(struct soap*, enum ns1__WeekDay const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__WeekDay(struct soap*, const enum ns1__WeekDay *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__WeekDay(struct soap*, const char*, int, const enum ns1__WeekDay *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__WeekDay2s(struct soap*, enum ns1__WeekDay);
SOAP_FMAC3 enum ns1__WeekDay * SOAP_FMAC4 soap_get_ns1__WeekDay(struct soap*, enum ns1__WeekDay *, const char*, const char*);
SOAP_FMAC3 enum ns1__WeekDay * SOAP_FMAC4 soap_in_ns1__WeekDay(struct soap*, const char*, enum ns1__WeekDay *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__WeekDay(struct soap*, const char*, enum ns1__WeekDay *);

#ifndef SOAP_TYPE_ns1__StatusTrackingOptions
#define SOAP_TYPE_ns1__StatusTrackingOptions (233)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__StatusTrackingOptions(struct soap*, enum ns1__StatusTrackingOptions *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__StatusTrackingOptions(struct soap*, enum ns1__StatusTrackingOptions const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__StatusTrackingOptions(struct soap*, const enum ns1__StatusTrackingOptions *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__StatusTrackingOptions(struct soap*, const char*, int, const enum ns1__StatusTrackingOptions *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__StatusTrackingOptions2s(struct soap*, enum ns1__StatusTrackingOptions);
SOAP_FMAC3 enum ns1__StatusTrackingOptions * SOAP_FMAC4 soap_get_ns1__StatusTrackingOptions(struct soap*, enum ns1__StatusTrackingOptions *, const char*, const char*);
SOAP_FMAC3 enum ns1__StatusTrackingOptions * SOAP_FMAC4 soap_in_ns1__StatusTrackingOptions(struct soap*, const char*, enum ns1__StatusTrackingOptions *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__StatusTrackingOptions(struct soap*, const char*, enum ns1__StatusTrackingOptions *);

#ifndef SOAP_TYPE_ns1__RecipientType
#define SOAP_TYPE_ns1__RecipientType (232)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__RecipientType(struct soap*, enum ns1__RecipientType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__RecipientType(struct soap*, enum ns1__RecipientType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__RecipientType(struct soap*, const enum ns1__RecipientType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__RecipientType(struct soap*, const char*, int, const enum ns1__RecipientType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__RecipientType2s(struct soap*, enum ns1__RecipientType);
SOAP_FMAC3 enum ns1__RecipientType * SOAP_FMAC4 soap_get_ns1__RecipientType(struct soap*, enum ns1__RecipientType *, const char*, const char*);
SOAP_FMAC3 enum ns1__RecipientType * SOAP_FMAC4 soap_in_ns1__RecipientType(struct soap*, const char*, enum ns1__RecipientType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__RecipientType(struct soap*, const char*, enum ns1__RecipientType *);

#ifndef SOAP_TYPE_ns1__PostalAddressType
#define SOAP_TYPE_ns1__PostalAddressType (231)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__PostalAddressType(struct soap*, enum ns1__PostalAddressType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__PostalAddressType(struct soap*, enum ns1__PostalAddressType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__PostalAddressType(struct soap*, const enum ns1__PostalAddressType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PostalAddressType(struct soap*, const char*, int, const enum ns1__PostalAddressType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__PostalAddressType2s(struct soap*, enum ns1__PostalAddressType);
SOAP_FMAC3 enum ns1__PostalAddressType * SOAP_FMAC4 soap_get_ns1__PostalAddressType(struct soap*, enum ns1__PostalAddressType *, const char*, const char*);
SOAP_FMAC3 enum ns1__PostalAddressType * SOAP_FMAC4 soap_in_ns1__PostalAddressType(struct soap*, const char*, enum ns1__PostalAddressType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__PostalAddressType(struct soap*, const char*, enum ns1__PostalAddressType *);

#ifndef SOAP_TYPE_ns1__PhoneNumberType
#define SOAP_TYPE_ns1__PhoneNumberType (230)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__PhoneNumberType(struct soap*, enum ns1__PhoneNumberType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__PhoneNumberType(struct soap*, enum ns1__PhoneNumberType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__PhoneNumberType(struct soap*, const enum ns1__PhoneNumberType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PhoneNumberType(struct soap*, const char*, int, const enum ns1__PhoneNumberType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__PhoneNumberType2s(struct soap*, enum ns1__PhoneNumberType);
SOAP_FMAC3 enum ns1__PhoneNumberType * SOAP_FMAC4 soap_get_ns1__PhoneNumberType(struct soap*, enum ns1__PhoneNumberType *, const char*, const char*);
SOAP_FMAC3 enum ns1__PhoneNumberType * SOAP_FMAC4 soap_in_ns1__PhoneNumberType(struct soap*, const char*, enum ns1__PhoneNumberType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__PhoneNumberType(struct soap*, const char*, enum ns1__PhoneNumberType *);

#ifndef SOAP_TYPE_ns1__OccurrenceType
#define SOAP_TYPE_ns1__OccurrenceType (229)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__OccurrenceType(struct soap*, enum ns1__OccurrenceType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__OccurrenceType(struct soap*, enum ns1__OccurrenceType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__OccurrenceType(struct soap*, const enum ns1__OccurrenceType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__OccurrenceType(struct soap*, const char*, int, const enum ns1__OccurrenceType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__OccurrenceType2s(struct soap*, enum ns1__OccurrenceType);
SOAP_FMAC3 enum ns1__OccurrenceType * SOAP_FMAC4 soap_get_ns1__OccurrenceType(struct soap*, enum ns1__OccurrenceType *, const char*, const char*);
SOAP_FMAC3 enum ns1__OccurrenceType * SOAP_FMAC4 soap_in_ns1__OccurrenceType(struct soap*, const char*, enum ns1__OccurrenceType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__OccurrenceType(struct soap*, const char*, enum ns1__OccurrenceType *);

#ifndef SOAP_TYPE_ns1__ItemSource
#define SOAP_TYPE_ns1__ItemSource (226)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__ItemSource(struct soap*, enum ns1__ItemSource *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__ItemSource(struct soap*, enum ns1__ItemSource const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__ItemSource(struct soap*, const enum ns1__ItemSource *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemSource(struct soap*, const char*, int, const enum ns1__ItemSource *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__ItemSource2s(struct soap*, enum ns1__ItemSource);
SOAP_FMAC3 enum ns1__ItemSource * SOAP_FMAC4 soap_get_ns1__ItemSource(struct soap*, enum ns1__ItemSource *, const char*, const char*);
SOAP_FMAC3 enum ns1__ItemSource * SOAP_FMAC4 soap_in_ns1__ItemSource(struct soap*, const char*, enum ns1__ItemSource *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__ItemSource(struct soap*, const char*, enum ns1__ItemSource *);

#ifndef SOAP_TYPE_ns1__ItemOptionsPriority
#define SOAP_TYPE_ns1__ItemOptionsPriority (225)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__ItemOptionsPriority(struct soap*, enum ns1__ItemOptionsPriority *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__ItemOptionsPriority(struct soap*, enum ns1__ItemOptionsPriority const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__ItemOptionsPriority(struct soap*, const enum ns1__ItemOptionsPriority *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemOptionsPriority(struct soap*, const char*, int, const enum ns1__ItemOptionsPriority *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__ItemOptionsPriority2s(struct soap*, enum ns1__ItemOptionsPriority);
SOAP_FMAC3 enum ns1__ItemOptionsPriority * SOAP_FMAC4 soap_get_ns1__ItemOptionsPriority(struct soap*, enum ns1__ItemOptionsPriority *, const char*, const char*);
SOAP_FMAC3 enum ns1__ItemOptionsPriority * SOAP_FMAC4 soap_in_ns1__ItemOptionsPriority(struct soap*, const char*, enum ns1__ItemOptionsPriority *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__ItemOptionsPriority(struct soap*, const char*, enum ns1__ItemOptionsPriority *);

#ifndef SOAP_TYPE_ns1__ItemClass
#define SOAP_TYPE_ns1__ItemClass (224)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__ItemClass(struct soap*, enum ns1__ItemClass *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__ItemClass(struct soap*, enum ns1__ItemClass const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__ItemClass(struct soap*, const enum ns1__ItemClass *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemClass(struct soap*, const char*, int, const enum ns1__ItemClass *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__ItemClass2s(struct soap*, enum ns1__ItemClass);
SOAP_FMAC3 enum ns1__ItemClass * SOAP_FMAC4 soap_get_ns1__ItemClass(struct soap*, enum ns1__ItemClass *, const char*, const char*);
SOAP_FMAC3 enum ns1__ItemClass * SOAP_FMAC4 soap_in_ns1__ItemClass(struct soap*, const char*, enum ns1__ItemClass *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__ItemClass(struct soap*, const char*, enum ns1__ItemClass *);

#ifndef SOAP_TYPE_ns1__FolderType
#define SOAP_TYPE_ns1__FolderType (221)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__FolderType(struct soap*, enum ns1__FolderType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__FolderType(struct soap*, enum ns1__FolderType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__FolderType(struct soap*, const enum ns1__FolderType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FolderType(struct soap*, const char*, int, const enum ns1__FolderType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__FolderType2s(struct soap*, enum ns1__FolderType);
SOAP_FMAC3 enum ns1__FolderType * SOAP_FMAC4 soap_get_ns1__FolderType(struct soap*, enum ns1__FolderType *, const char*, const char*);
SOAP_FMAC3 enum ns1__FolderType * SOAP_FMAC4 soap_in_ns1__FolderType(struct soap*, const char*, enum ns1__FolderType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__FolderType(struct soap*, const char*, enum ns1__FolderType *);

#ifndef SOAP_TYPE_ns1__FolderACLStatus
#define SOAP_TYPE_ns1__FolderACLStatus (220)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__FolderACLStatus(struct soap*, enum ns1__FolderACLStatus *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__FolderACLStatus(struct soap*, enum ns1__FolderACLStatus const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__FolderACLStatus(struct soap*, const enum ns1__FolderACLStatus *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FolderACLStatus(struct soap*, const char*, int, const enum ns1__FolderACLStatus *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__FolderACLStatus2s(struct soap*, enum ns1__FolderACLStatus);
SOAP_FMAC3 enum ns1__FolderACLStatus * SOAP_FMAC4 soap_get_ns1__FolderACLStatus(struct soap*, enum ns1__FolderACLStatus *, const char*, const char*);
SOAP_FMAC3 enum ns1__FolderACLStatus * SOAP_FMAC4 soap_in_ns1__FolderACLStatus(struct soap*, const char*, enum ns1__FolderACLStatus *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__FolderACLStatus(struct soap*, const char*, enum ns1__FolderACLStatus *);

#ifndef SOAP_TYPE_ns1__FilterOp
#define SOAP_TYPE_ns1__FilterOp (219)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__FilterOp(struct soap*, enum ns1__FilterOp *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__FilterOp(struct soap*, enum ns1__FilterOp const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__FilterOp(struct soap*, const enum ns1__FilterOp *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FilterOp(struct soap*, const char*, int, const enum ns1__FilterOp *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__FilterOp2s(struct soap*, enum ns1__FilterOp);
SOAP_FMAC3 enum ns1__FilterOp * SOAP_FMAC4 soap_get_ns1__FilterOp(struct soap*, enum ns1__FilterOp *, const char*, const char*);
SOAP_FMAC3 enum ns1__FilterOp * SOAP_FMAC4 soap_in_ns1__FilterOp(struct soap*, const char*, enum ns1__FilterOp *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__FilterOp(struct soap*, const char*, enum ns1__FilterOp *);

#ifndef SOAP_TYPE_ns1__DistributionType
#define SOAP_TYPE_ns1__DistributionType (218)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__DistributionType(struct soap*, enum ns1__DistributionType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__DistributionType(struct soap*, enum ns1__DistributionType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__DistributionType(struct soap*, const enum ns1__DistributionType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__DistributionType(struct soap*, const char*, int, const enum ns1__DistributionType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__DistributionType2s(struct soap*, enum ns1__DistributionType);
SOAP_FMAC3 enum ns1__DistributionType * SOAP_FMAC4 soap_get_ns1__DistributionType(struct soap*, enum ns1__DistributionType *, const char*, const char*);
SOAP_FMAC3 enum ns1__DistributionType * SOAP_FMAC4 soap_in_ns1__DistributionType(struct soap*, const char*, enum ns1__DistributionType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__DistributionType(struct soap*, const char*, enum ns1__DistributionType *);

#ifndef SOAP_TYPE_ns1__CursorSeek
#define SOAP_TYPE_ns1__CursorSeek (216)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__CursorSeek(struct soap*, enum ns1__CursorSeek *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__CursorSeek(struct soap*, enum ns1__CursorSeek const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__CursorSeek(struct soap*, const enum ns1__CursorSeek *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__CursorSeek(struct soap*, const char*, int, const enum ns1__CursorSeek *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__CursorSeek2s(struct soap*, enum ns1__CursorSeek);
SOAP_FMAC3 enum ns1__CursorSeek * SOAP_FMAC4 soap_get_ns1__CursorSeek(struct soap*, enum ns1__CursorSeek *, const char*, const char*);
SOAP_FMAC3 enum ns1__CursorSeek * SOAP_FMAC4 soap_in_ns1__CursorSeek(struct soap*, const char*, enum ns1__CursorSeek *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__CursorSeek(struct soap*, const char*, enum ns1__CursorSeek *);

#ifndef SOAP_TYPE_ns1__ContactType
#define SOAP_TYPE_ns1__ContactType (215)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__ContactType(struct soap*, enum ns1__ContactType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__ContactType(struct soap*, enum ns1__ContactType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__ContactType(struct soap*, const enum ns1__ContactType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ContactType(struct soap*, const char*, int, const enum ns1__ContactType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__ContactType2s(struct soap*, enum ns1__ContactType);
SOAP_FMAC3 enum ns1__ContactType * SOAP_FMAC4 soap_get_ns1__ContactType(struct soap*, enum ns1__ContactType *, const char*, const char*);
SOAP_FMAC3 enum ns1__ContactType * SOAP_FMAC4 soap_in_ns1__ContactType(struct soap*, const char*, enum ns1__ContactType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__ContactType(struct soap*, const char*, enum ns1__ContactType *);

#ifndef SOAP_TYPE_ns1__CategoryType
#define SOAP_TYPE_ns1__CategoryType (214)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__CategoryType(struct soap*, enum ns1__CategoryType *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__CategoryType(struct soap*, enum ns1__CategoryType const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__CategoryType(struct soap*, const enum ns1__CategoryType *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__CategoryType(struct soap*, const char*, int, const enum ns1__CategoryType *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__CategoryType2s(struct soap*, enum ns1__CategoryType);
SOAP_FMAC3 enum ns1__CategoryType * SOAP_FMAC4 soap_get_ns1__CategoryType(struct soap*, enum ns1__CategoryType *, const char*, const char*);
SOAP_FMAC3 enum ns1__CategoryType * SOAP_FMAC4 soap_in_ns1__CategoryType(struct soap*, const char*, enum ns1__CategoryType *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__CategoryType(struct soap*, const char*, enum ns1__CategoryType *);

#ifndef SOAP_TYPE_ns1__AcceptLevel
#define SOAP_TYPE_ns1__AcceptLevel (213)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__AcceptLevel(struct soap*, enum ns1__AcceptLevel *);
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__AcceptLevel(struct soap*, enum ns1__AcceptLevel const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__AcceptLevel(struct soap*, const enum ns1__AcceptLevel *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AcceptLevel(struct soap*, const char*, int, const enum ns1__AcceptLevel *, const char*);

SOAP_FMAC3 const char* SOAP_FMAC4 soap_ns1__AcceptLevel2s(struct soap*, enum ns1__AcceptLevel);
SOAP_FMAC3 enum ns1__AcceptLevel * SOAP_FMAC4 soap_get_ns1__AcceptLevel(struct soap*, enum ns1__AcceptLevel *, const char*, const char*);
SOAP_FMAC3 enum ns1__AcceptLevel * SOAP_FMAC4 soap_in_ns1__AcceptLevel(struct soap*, const char*, enum ns1__AcceptLevel *, const char*);

SOAP_FMAC3 int SOAP_FMAC4 soap_s2ns1__AcceptLevel(struct soap*, const char*, enum ns1__AcceptLevel *);

#ifndef SOAP_TYPE_ns1__UUID
#define SOAP_TYPE_ns1__UUID (235)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__UUID(struct soap*, std::string const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_ns1__UUID(struct soap*, const std::string *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__UUID(struct soap*, std::string *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__UUID(struct soap*, const std::string *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__UUID(struct soap*, const char*, int, const std::string*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_get_ns1__UUID(struct soap*, std::string *, const char*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_in_ns1__UUID(struct soap*, const char*, std::string*, const char*);

SOAP_FMAC5 std::string * SOAP_FMAC6 soap_new_ns1__UUID(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__UUID(struct soap*, std::string*);
SOAP_FMAC5 std::string * SOAP_FMAC6 soap_instantiate_ns1__UUID(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__UUID(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__uid
#define SOAP_TYPE_ns1__uid (234)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__uid(struct soap*, std::string const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_ns1__uid(struct soap*, const std::string *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__uid(struct soap*, std::string *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__uid(struct soap*, const std::string *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__uid(struct soap*, const char*, int, const std::string*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_get_ns1__uid(struct soap*, std::string *, const char*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_in_ns1__uid(struct soap*, const char*, std::string*, const char*);

SOAP_FMAC5 std::string * SOAP_FMAC6 soap_new_ns1__uid(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__uid(struct soap*, std::string*);
SOAP_FMAC5 std::string * SOAP_FMAC6 soap_instantiate_ns1__uid(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__uid(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__View
#define SOAP_TYPE_ns1__View (212)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_ns1__View(struct soap*, std::string const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_ns1__View(struct soap*, const std::string *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_ns1__View(struct soap*, std::string *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_ns1__View(struct soap*, const std::string *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__View(struct soap*, const char*, int, const std::string*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_get_ns1__View(struct soap*, std::string *, const char*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_in_ns1__View(struct soap*, const char*, std::string*, const char*);

SOAP_FMAC5 std::string * SOAP_FMAC6 soap_new_ns1__View(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__View(struct soap*, std::string*);
SOAP_FMAC5 std::string * SOAP_FMAC6 soap_instantiate_ns1__View(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__View(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__string
#define SOAP_TYPE_std__string (211)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__string(struct soap*, std::string const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__string(struct soap*, const std::string *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__string(struct soap*, std::string *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__string(struct soap*, const std::string *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__string(struct soap*, const char*, int, const std::string*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_get_std__string(struct soap*, std::string *, const char*, const char*);
SOAP_FMAC3 std::string * SOAP_FMAC4 soap_in_std__string(struct soap*, const char*, std::string*, const char*);

SOAP_FMAC5 std::string * SOAP_FMAC6 soap_new_std__string(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__string(struct soap*, std::string*);
SOAP_FMAC5 std::string * SOAP_FMAC6 soap_instantiate_std__string(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__string(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__startFreeBusySessionResponse
#define SOAP_TYPE__ns1__startFreeBusySessionResponse (210)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__startFreeBusySessionResponse(struct soap*, const char*, int, const _ns1__startFreeBusySessionResponse *, const char*);
SOAP_FMAC3 _ns1__startFreeBusySessionResponse * SOAP_FMAC4 soap_get__ns1__startFreeBusySessionResponse(struct soap*, _ns1__startFreeBusySessionResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__startFreeBusySessionResponse * SOAP_FMAC4 soap_in__ns1__startFreeBusySessionResponse(struct soap*, const char*, _ns1__startFreeBusySessionResponse *, const char*);

SOAP_FMAC5 _ns1__startFreeBusySessionResponse * SOAP_FMAC6 soap_new__ns1__startFreeBusySessionResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__startFreeBusySessionResponse(struct soap*, _ns1__startFreeBusySessionResponse*);
SOAP_FMAC5 _ns1__startFreeBusySessionResponse * SOAP_FMAC6 soap_instantiate__ns1__startFreeBusySessionResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__startFreeBusySessionResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__startFreeBusySessionRequest
#define SOAP_TYPE__ns1__startFreeBusySessionRequest (209)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__startFreeBusySessionRequest(struct soap*, const char*, int, const _ns1__startFreeBusySessionRequest *, const char*);
SOAP_FMAC3 _ns1__startFreeBusySessionRequest * SOAP_FMAC4 soap_get__ns1__startFreeBusySessionRequest(struct soap*, _ns1__startFreeBusySessionRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__startFreeBusySessionRequest * SOAP_FMAC4 soap_in__ns1__startFreeBusySessionRequest(struct soap*, const char*, _ns1__startFreeBusySessionRequest *, const char*);

SOAP_FMAC5 _ns1__startFreeBusySessionRequest * SOAP_FMAC6 soap_new__ns1__startFreeBusySessionRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__startFreeBusySessionRequest(struct soap*, _ns1__startFreeBusySessionRequest*);
SOAP_FMAC5 _ns1__startFreeBusySessionRequest * SOAP_FMAC6 soap_instantiate__ns1__startFreeBusySessionRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__startFreeBusySessionRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__sendItemResponse
#define SOAP_TYPE__ns1__sendItemResponse (208)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__sendItemResponse(struct soap*, const char*, int, const _ns1__sendItemResponse *, const char*);
SOAP_FMAC3 _ns1__sendItemResponse * SOAP_FMAC4 soap_get__ns1__sendItemResponse(struct soap*, _ns1__sendItemResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__sendItemResponse * SOAP_FMAC4 soap_in__ns1__sendItemResponse(struct soap*, const char*, _ns1__sendItemResponse *, const char*);

SOAP_FMAC5 _ns1__sendItemResponse * SOAP_FMAC6 soap_new__ns1__sendItemResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__sendItemResponse(struct soap*, _ns1__sendItemResponse*);
SOAP_FMAC5 _ns1__sendItemResponse * SOAP_FMAC6 soap_instantiate__ns1__sendItemResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__sendItemResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__sendItemRequest
#define SOAP_TYPE__ns1__sendItemRequest (207)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__sendItemRequest(struct soap*, const char*, int, const _ns1__sendItemRequest *, const char*);
SOAP_FMAC3 _ns1__sendItemRequest * SOAP_FMAC4 soap_get__ns1__sendItemRequest(struct soap*, _ns1__sendItemRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__sendItemRequest * SOAP_FMAC4 soap_in__ns1__sendItemRequest(struct soap*, const char*, _ns1__sendItemRequest *, const char*);

SOAP_FMAC5 _ns1__sendItemRequest * SOAP_FMAC6 soap_new__ns1__sendItemRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__sendItemRequest(struct soap*, _ns1__sendItemRequest*);
SOAP_FMAC5 _ns1__sendItemRequest * SOAP_FMAC6 soap_instantiate__ns1__sendItemRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__sendItemRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__retractResponse
#define SOAP_TYPE__ns1__retractResponse (206)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__retractResponse(struct soap*, const char*, int, const _ns1__retractResponse *, const char*);
SOAP_FMAC3 _ns1__retractResponse * SOAP_FMAC4 soap_get__ns1__retractResponse(struct soap*, _ns1__retractResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__retractResponse * SOAP_FMAC4 soap_in__ns1__retractResponse(struct soap*, const char*, _ns1__retractResponse *, const char*);

SOAP_FMAC5 _ns1__retractResponse * SOAP_FMAC6 soap_new__ns1__retractResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__retractResponse(struct soap*, _ns1__retractResponse*);
SOAP_FMAC5 _ns1__retractResponse * SOAP_FMAC6 soap_instantiate__ns1__retractResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__retractResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__retractRequest
#define SOAP_TYPE__ns1__retractRequest (205)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__retractRequest(struct soap*, const char*, int, const _ns1__retractRequest *, const char*);
SOAP_FMAC3 _ns1__retractRequest * SOAP_FMAC4 soap_get__ns1__retractRequest(struct soap*, _ns1__retractRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__retractRequest * SOAP_FMAC4 soap_in__ns1__retractRequest(struct soap*, const char*, _ns1__retractRequest *, const char*);

SOAP_FMAC5 _ns1__retractRequest * SOAP_FMAC6 soap_new__ns1__retractRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__retractRequest(struct soap*, _ns1__retractRequest*);
SOAP_FMAC5 _ns1__retractRequest * SOAP_FMAC6 soap_instantiate__ns1__retractRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__retractRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeProxyAccessResponse
#define SOAP_TYPE__ns1__removeProxyAccessResponse (204)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeProxyAccessResponse(struct soap*, const char*, int, const _ns1__removeProxyAccessResponse *, const char*);
SOAP_FMAC3 _ns1__removeProxyAccessResponse * SOAP_FMAC4 soap_get__ns1__removeProxyAccessResponse(struct soap*, _ns1__removeProxyAccessResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__removeProxyAccessResponse * SOAP_FMAC4 soap_in__ns1__removeProxyAccessResponse(struct soap*, const char*, _ns1__removeProxyAccessResponse *, const char*);

SOAP_FMAC5 _ns1__removeProxyAccessResponse * SOAP_FMAC6 soap_new__ns1__removeProxyAccessResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeProxyAccessResponse(struct soap*, _ns1__removeProxyAccessResponse*);
SOAP_FMAC5 _ns1__removeProxyAccessResponse * SOAP_FMAC6 soap_instantiate__ns1__removeProxyAccessResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeProxyAccessResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeProxyAccessRequest
#define SOAP_TYPE__ns1__removeProxyAccessRequest (203)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeProxyAccessRequest(struct soap*, const char*, int, const _ns1__removeProxyAccessRequest *, const char*);
SOAP_FMAC3 _ns1__removeProxyAccessRequest * SOAP_FMAC4 soap_get__ns1__removeProxyAccessRequest(struct soap*, _ns1__removeProxyAccessRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__removeProxyAccessRequest * SOAP_FMAC4 soap_in__ns1__removeProxyAccessRequest(struct soap*, const char*, _ns1__removeProxyAccessRequest *, const char*);

SOAP_FMAC5 _ns1__removeProxyAccessRequest * SOAP_FMAC6 soap_new__ns1__removeProxyAccessRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeProxyAccessRequest(struct soap*, _ns1__removeProxyAccessRequest*);
SOAP_FMAC5 _ns1__removeProxyAccessRequest * SOAP_FMAC6 soap_instantiate__ns1__removeProxyAccessRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeProxyAccessRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeMembersResponse
#define SOAP_TYPE__ns1__removeMembersResponse (202)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeMembersResponse(struct soap*, const char*, int, const _ns1__removeMembersResponse *, const char*);
SOAP_FMAC3 _ns1__removeMembersResponse * SOAP_FMAC4 soap_get__ns1__removeMembersResponse(struct soap*, _ns1__removeMembersResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__removeMembersResponse * SOAP_FMAC4 soap_in__ns1__removeMembersResponse(struct soap*, const char*, _ns1__removeMembersResponse *, const char*);

SOAP_FMAC5 _ns1__removeMembersResponse * SOAP_FMAC6 soap_new__ns1__removeMembersResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeMembersResponse(struct soap*, _ns1__removeMembersResponse*);
SOAP_FMAC5 _ns1__removeMembersResponse * SOAP_FMAC6 soap_instantiate__ns1__removeMembersResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeMembersResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeMembersRequest
#define SOAP_TYPE__ns1__removeMembersRequest (201)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeMembersRequest(struct soap*, const char*, int, const _ns1__removeMembersRequest *, const char*);
SOAP_FMAC3 _ns1__removeMembersRequest * SOAP_FMAC4 soap_get__ns1__removeMembersRequest(struct soap*, _ns1__removeMembersRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__removeMembersRequest * SOAP_FMAC4 soap_in__ns1__removeMembersRequest(struct soap*, const char*, _ns1__removeMembersRequest *, const char*);

SOAP_FMAC5 _ns1__removeMembersRequest * SOAP_FMAC6 soap_new__ns1__removeMembersRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeMembersRequest(struct soap*, _ns1__removeMembersRequest*);
SOAP_FMAC5 _ns1__removeMembersRequest * SOAP_FMAC6 soap_instantiate__ns1__removeMembersRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeMembersRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeItemsResponse
#define SOAP_TYPE__ns1__removeItemsResponse (200)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeItemsResponse(struct soap*, const char*, int, const _ns1__removeItemsResponse *, const char*);
SOAP_FMAC3 _ns1__removeItemsResponse * SOAP_FMAC4 soap_get__ns1__removeItemsResponse(struct soap*, _ns1__removeItemsResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__removeItemsResponse * SOAP_FMAC4 soap_in__ns1__removeItemsResponse(struct soap*, const char*, _ns1__removeItemsResponse *, const char*);

SOAP_FMAC5 _ns1__removeItemsResponse * SOAP_FMAC6 soap_new__ns1__removeItemsResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeItemsResponse(struct soap*, _ns1__removeItemsResponse*);
SOAP_FMAC5 _ns1__removeItemsResponse * SOAP_FMAC6 soap_instantiate__ns1__removeItemsResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeItemsResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeItemsRequest
#define SOAP_TYPE__ns1__removeItemsRequest (199)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeItemsRequest(struct soap*, const char*, int, const _ns1__removeItemsRequest *, const char*);
SOAP_FMAC3 _ns1__removeItemsRequest * SOAP_FMAC4 soap_get__ns1__removeItemsRequest(struct soap*, _ns1__removeItemsRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__removeItemsRequest * SOAP_FMAC4 soap_in__ns1__removeItemsRequest(struct soap*, const char*, _ns1__removeItemsRequest *, const char*);

SOAP_FMAC5 _ns1__removeItemsRequest * SOAP_FMAC6 soap_new__ns1__removeItemsRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeItemsRequest(struct soap*, _ns1__removeItemsRequest*);
SOAP_FMAC5 _ns1__removeItemsRequest * SOAP_FMAC6 soap_instantiate__ns1__removeItemsRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeItemsRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeItemResponse
#define SOAP_TYPE__ns1__removeItemResponse (198)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeItemResponse(struct soap*, const char*, int, const _ns1__removeItemResponse *, const char*);
SOAP_FMAC3 _ns1__removeItemResponse * SOAP_FMAC4 soap_get__ns1__removeItemResponse(struct soap*, _ns1__removeItemResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__removeItemResponse * SOAP_FMAC4 soap_in__ns1__removeItemResponse(struct soap*, const char*, _ns1__removeItemResponse *, const char*);

SOAP_FMAC5 _ns1__removeItemResponse * SOAP_FMAC6 soap_new__ns1__removeItemResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeItemResponse(struct soap*, _ns1__removeItemResponse*);
SOAP_FMAC5 _ns1__removeItemResponse * SOAP_FMAC6 soap_instantiate__ns1__removeItemResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeItemResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__removeItemRequest
#define SOAP_TYPE__ns1__removeItemRequest (197)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__removeItemRequest(struct soap*, const char*, int, const _ns1__removeItemRequest *, const char*);
SOAP_FMAC3 _ns1__removeItemRequest * SOAP_FMAC4 soap_get__ns1__removeItemRequest(struct soap*, _ns1__removeItemRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__removeItemRequest * SOAP_FMAC4 soap_in__ns1__removeItemRequest(struct soap*, const char*, _ns1__removeItemRequest *, const char*);

SOAP_FMAC5 _ns1__removeItemRequest * SOAP_FMAC6 soap_new__ns1__removeItemRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__removeItemRequest(struct soap*, _ns1__removeItemRequest*);
SOAP_FMAC5 _ns1__removeItemRequest * SOAP_FMAC6 soap_instantiate__ns1__removeItemRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__removeItemRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__readCursorResponse
#define SOAP_TYPE__ns1__readCursorResponse (196)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__readCursorResponse(struct soap*, const char*, int, const _ns1__readCursorResponse *, const char*);
SOAP_FMAC3 _ns1__readCursorResponse * SOAP_FMAC4 soap_get__ns1__readCursorResponse(struct soap*, _ns1__readCursorResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__readCursorResponse * SOAP_FMAC4 soap_in__ns1__readCursorResponse(struct soap*, const char*, _ns1__readCursorResponse *, const char*);

SOAP_FMAC5 _ns1__readCursorResponse * SOAP_FMAC6 soap_new__ns1__readCursorResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__readCursorResponse(struct soap*, _ns1__readCursorResponse*);
SOAP_FMAC5 _ns1__readCursorResponse * SOAP_FMAC6 soap_instantiate__ns1__readCursorResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__readCursorResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__readCursorRequest
#define SOAP_TYPE__ns1__readCursorRequest (195)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__readCursorRequest(struct soap*, const char*, int, const _ns1__readCursorRequest *, const char*);
SOAP_FMAC3 _ns1__readCursorRequest * SOAP_FMAC4 soap_get__ns1__readCursorRequest(struct soap*, _ns1__readCursorRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__readCursorRequest * SOAP_FMAC4 soap_in__ns1__readCursorRequest(struct soap*, const char*, _ns1__readCursorRequest *, const char*);

SOAP_FMAC5 _ns1__readCursorRequest * SOAP_FMAC6 soap_new__ns1__readCursorRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__readCursorRequest(struct soap*, _ns1__readCursorRequest*);
SOAP_FMAC5 _ns1__readCursorRequest * SOAP_FMAC6 soap_instantiate__ns1__readCursorRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__readCursorRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__purgeItemResponse
#define SOAP_TYPE__ns1__purgeItemResponse (194)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__purgeItemResponse(struct soap*, const char*, int, const _ns1__purgeItemResponse *, const char*);
SOAP_FMAC3 _ns1__purgeItemResponse * SOAP_FMAC4 soap_get__ns1__purgeItemResponse(struct soap*, _ns1__purgeItemResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__purgeItemResponse * SOAP_FMAC4 soap_in__ns1__purgeItemResponse(struct soap*, const char*, _ns1__purgeItemResponse *, const char*);

SOAP_FMAC5 _ns1__purgeItemResponse * SOAP_FMAC6 soap_new__ns1__purgeItemResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__purgeItemResponse(struct soap*, _ns1__purgeItemResponse*);
SOAP_FMAC5 _ns1__purgeItemResponse * SOAP_FMAC6 soap_instantiate__ns1__purgeItemResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__purgeItemResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__purgeItemRequest
#define SOAP_TYPE__ns1__purgeItemRequest (193)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__purgeItemRequest(struct soap*, const char*, int, const _ns1__purgeItemRequest *, const char*);
SOAP_FMAC3 _ns1__purgeItemRequest * SOAP_FMAC4 soap_get__ns1__purgeItemRequest(struct soap*, _ns1__purgeItemRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__purgeItemRequest * SOAP_FMAC4 soap_in__ns1__purgeItemRequest(struct soap*, const char*, _ns1__purgeItemRequest *, const char*);

SOAP_FMAC5 _ns1__purgeItemRequest * SOAP_FMAC6 soap_new__ns1__purgeItemRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__purgeItemRequest(struct soap*, _ns1__purgeItemRequest*);
SOAP_FMAC5 _ns1__purgeItemRequest * SOAP_FMAC6 soap_instantiate__ns1__purgeItemRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__purgeItemRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__purgeDeletedItemsResponse
#define SOAP_TYPE__ns1__purgeDeletedItemsResponse (192)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__purgeDeletedItemsResponse(struct soap*, const char*, int, const _ns1__purgeDeletedItemsResponse *, const char*);
SOAP_FMAC3 _ns1__purgeDeletedItemsResponse * SOAP_FMAC4 soap_get__ns1__purgeDeletedItemsResponse(struct soap*, _ns1__purgeDeletedItemsResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__purgeDeletedItemsResponse * SOAP_FMAC4 soap_in__ns1__purgeDeletedItemsResponse(struct soap*, const char*, _ns1__purgeDeletedItemsResponse *, const char*);

SOAP_FMAC5 _ns1__purgeDeletedItemsResponse * SOAP_FMAC6 soap_new__ns1__purgeDeletedItemsResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__purgeDeletedItemsResponse(struct soap*, _ns1__purgeDeletedItemsResponse*);
SOAP_FMAC5 _ns1__purgeDeletedItemsResponse * SOAP_FMAC6 soap_instantiate__ns1__purgeDeletedItemsResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__purgeDeletedItemsResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__purgeDeletedItemsRequest
#define SOAP_TYPE__ns1__purgeDeletedItemsRequest (191)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__purgeDeletedItemsRequest(struct soap*, const char*, int, const _ns1__purgeDeletedItemsRequest *, const char*);
SOAP_FMAC3 _ns1__purgeDeletedItemsRequest * SOAP_FMAC4 soap_get__ns1__purgeDeletedItemsRequest(struct soap*, _ns1__purgeDeletedItemsRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__purgeDeletedItemsRequest * SOAP_FMAC4 soap_in__ns1__purgeDeletedItemsRequest(struct soap*, const char*, _ns1__purgeDeletedItemsRequest *, const char*);

SOAP_FMAC5 _ns1__purgeDeletedItemsRequest * SOAP_FMAC6 soap_new__ns1__purgeDeletedItemsRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__purgeDeletedItemsRequest(struct soap*, _ns1__purgeDeletedItemsRequest*);
SOAP_FMAC5 _ns1__purgeDeletedItemsRequest * SOAP_FMAC6 soap_instantiate__ns1__purgeDeletedItemsRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__purgeDeletedItemsRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__positionCursorResponse
#define SOAP_TYPE__ns1__positionCursorResponse (190)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__positionCursorResponse(struct soap*, const char*, int, const _ns1__positionCursorResponse *, const char*);
SOAP_FMAC3 _ns1__positionCursorResponse * SOAP_FMAC4 soap_get__ns1__positionCursorResponse(struct soap*, _ns1__positionCursorResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__positionCursorResponse * SOAP_FMAC4 soap_in__ns1__positionCursorResponse(struct soap*, const char*, _ns1__positionCursorResponse *, const char*);

SOAP_FMAC5 _ns1__positionCursorResponse * SOAP_FMAC6 soap_new__ns1__positionCursorResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__positionCursorResponse(struct soap*, _ns1__positionCursorResponse*);
SOAP_FMAC5 _ns1__positionCursorResponse * SOAP_FMAC6 soap_instantiate__ns1__positionCursorResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__positionCursorResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__positionCursorRequest
#define SOAP_TYPE__ns1__positionCursorRequest (189)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__positionCursorRequest(struct soap*, const char*, int, const _ns1__positionCursorRequest *, const char*);
SOAP_FMAC3 _ns1__positionCursorRequest * SOAP_FMAC4 soap_get__ns1__positionCursorRequest(struct soap*, _ns1__positionCursorRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__positionCursorRequest * SOAP_FMAC4 soap_in__ns1__positionCursorRequest(struct soap*, const char*, _ns1__positionCursorRequest *, const char*);

SOAP_FMAC5 _ns1__positionCursorRequest * SOAP_FMAC6 soap_new__ns1__positionCursorRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__positionCursorRequest(struct soap*, _ns1__positionCursorRequest*);
SOAP_FMAC5 _ns1__positionCursorRequest * SOAP_FMAC6 soap_instantiate__ns1__positionCursorRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__positionCursorRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__modifySettingsResponse
#define SOAP_TYPE__ns1__modifySettingsResponse (188)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__modifySettingsResponse(struct soap*, const char*, int, const _ns1__modifySettingsResponse *, const char*);
SOAP_FMAC3 _ns1__modifySettingsResponse * SOAP_FMAC4 soap_get__ns1__modifySettingsResponse(struct soap*, _ns1__modifySettingsResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__modifySettingsResponse * SOAP_FMAC4 soap_in__ns1__modifySettingsResponse(struct soap*, const char*, _ns1__modifySettingsResponse *, const char*);

SOAP_FMAC5 _ns1__modifySettingsResponse * SOAP_FMAC6 soap_new__ns1__modifySettingsResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__modifySettingsResponse(struct soap*, _ns1__modifySettingsResponse*);
SOAP_FMAC5 _ns1__modifySettingsResponse * SOAP_FMAC6 soap_instantiate__ns1__modifySettingsResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__modifySettingsResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__modifySettingsRequest
#define SOAP_TYPE__ns1__modifySettingsRequest (187)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__modifySettingsRequest(struct soap*, const char*, int, const _ns1__modifySettingsRequest *, const char*);
SOAP_FMAC3 _ns1__modifySettingsRequest * SOAP_FMAC4 soap_get__ns1__modifySettingsRequest(struct soap*, _ns1__modifySettingsRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__modifySettingsRequest * SOAP_FMAC4 soap_in__ns1__modifySettingsRequest(struct soap*, const char*, _ns1__modifySettingsRequest *, const char*);

SOAP_FMAC5 _ns1__modifySettingsRequest * SOAP_FMAC6 soap_new__ns1__modifySettingsRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__modifySettingsRequest(struct soap*, _ns1__modifySettingsRequest*);
SOAP_FMAC5 _ns1__modifySettingsRequest * SOAP_FMAC6 soap_instantiate__ns1__modifySettingsRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__modifySettingsRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__modifyProxyAccessResponse
#define SOAP_TYPE__ns1__modifyProxyAccessResponse (186)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__modifyProxyAccessResponse(struct soap*, const char*, int, const _ns1__modifyProxyAccessResponse *, const char*);
SOAP_FMAC3 _ns1__modifyProxyAccessResponse * SOAP_FMAC4 soap_get__ns1__modifyProxyAccessResponse(struct soap*, _ns1__modifyProxyAccessResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__modifyProxyAccessResponse * SOAP_FMAC4 soap_in__ns1__modifyProxyAccessResponse(struct soap*, const char*, _ns1__modifyProxyAccessResponse *, const char*);

SOAP_FMAC5 _ns1__modifyProxyAccessResponse * SOAP_FMAC6 soap_new__ns1__modifyProxyAccessResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__modifyProxyAccessResponse(struct soap*, _ns1__modifyProxyAccessResponse*);
SOAP_FMAC5 _ns1__modifyProxyAccessResponse * SOAP_FMAC6 soap_instantiate__ns1__modifyProxyAccessResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__modifyProxyAccessResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__modifyProxyAccessRequest
#define SOAP_TYPE__ns1__modifyProxyAccessRequest (185)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__modifyProxyAccessRequest(struct soap*, const char*, int, const _ns1__modifyProxyAccessRequest *, const char*);
SOAP_FMAC3 _ns1__modifyProxyAccessRequest * SOAP_FMAC4 soap_get__ns1__modifyProxyAccessRequest(struct soap*, _ns1__modifyProxyAccessRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__modifyProxyAccessRequest * SOAP_FMAC4 soap_in__ns1__modifyProxyAccessRequest(struct soap*, const char*, _ns1__modifyProxyAccessRequest *, const char*);

SOAP_FMAC5 _ns1__modifyProxyAccessRequest * SOAP_FMAC6 soap_new__ns1__modifyProxyAccessRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__modifyProxyAccessRequest(struct soap*, _ns1__modifyProxyAccessRequest*);
SOAP_FMAC5 _ns1__modifyProxyAccessRequest * SOAP_FMAC6 soap_instantiate__ns1__modifyProxyAccessRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__modifyProxyAccessRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__modifyItemResponse
#define SOAP_TYPE__ns1__modifyItemResponse (184)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__modifyItemResponse(struct soap*, const char*, int, const _ns1__modifyItemResponse *, const char*);
SOAP_FMAC3 _ns1__modifyItemResponse * SOAP_FMAC4 soap_get__ns1__modifyItemResponse(struct soap*, _ns1__modifyItemResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__modifyItemResponse * SOAP_FMAC4 soap_in__ns1__modifyItemResponse(struct soap*, const char*, _ns1__modifyItemResponse *, const char*);

SOAP_FMAC5 _ns1__modifyItemResponse * SOAP_FMAC6 soap_new__ns1__modifyItemResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__modifyItemResponse(struct soap*, _ns1__modifyItemResponse*);
SOAP_FMAC5 _ns1__modifyItemResponse * SOAP_FMAC6 soap_instantiate__ns1__modifyItemResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__modifyItemResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__modifyItemRequest
#define SOAP_TYPE__ns1__modifyItemRequest (183)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__modifyItemRequest(struct soap*, const char*, int, const _ns1__modifyItemRequest *, const char*);
SOAP_FMAC3 _ns1__modifyItemRequest * SOAP_FMAC4 soap_get__ns1__modifyItemRequest(struct soap*, _ns1__modifyItemRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__modifyItemRequest * SOAP_FMAC4 soap_in__ns1__modifyItemRequest(struct soap*, const char*, _ns1__modifyItemRequest *, const char*);

SOAP_FMAC5 _ns1__modifyItemRequest * SOAP_FMAC6 soap_new__ns1__modifyItemRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__modifyItemRequest(struct soap*, _ns1__modifyItemRequest*);
SOAP_FMAC5 _ns1__modifyItemRequest * SOAP_FMAC6 soap_instantiate__ns1__modifyItemRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__modifyItemRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__markUnReadResponse
#define SOAP_TYPE__ns1__markUnReadResponse (182)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__markUnReadResponse(struct soap*, const char*, int, const _ns1__markUnReadResponse *, const char*);
SOAP_FMAC3 _ns1__markUnReadResponse * SOAP_FMAC4 soap_get__ns1__markUnReadResponse(struct soap*, _ns1__markUnReadResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__markUnReadResponse * SOAP_FMAC4 soap_in__ns1__markUnReadResponse(struct soap*, const char*, _ns1__markUnReadResponse *, const char*);

SOAP_FMAC5 _ns1__markUnReadResponse * SOAP_FMAC6 soap_new__ns1__markUnReadResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__markUnReadResponse(struct soap*, _ns1__markUnReadResponse*);
SOAP_FMAC5 _ns1__markUnReadResponse * SOAP_FMAC6 soap_instantiate__ns1__markUnReadResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__markUnReadResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__markUnReadRequest
#define SOAP_TYPE__ns1__markUnReadRequest (181)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__markUnReadRequest(struct soap*, const char*, int, const _ns1__markUnReadRequest *, const char*);
SOAP_FMAC3 _ns1__markUnReadRequest * SOAP_FMAC4 soap_get__ns1__markUnReadRequest(struct soap*, _ns1__markUnReadRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__markUnReadRequest * SOAP_FMAC4 soap_in__ns1__markUnReadRequest(struct soap*, const char*, _ns1__markUnReadRequest *, const char*);

SOAP_FMAC5 _ns1__markUnReadRequest * SOAP_FMAC6 soap_new__ns1__markUnReadRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__markUnReadRequest(struct soap*, _ns1__markUnReadRequest*);
SOAP_FMAC5 _ns1__markUnReadRequest * SOAP_FMAC6 soap_instantiate__ns1__markUnReadRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__markUnReadRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__markReadResponse
#define SOAP_TYPE__ns1__markReadResponse (180)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__markReadResponse(struct soap*, const char*, int, const _ns1__markReadResponse *, const char*);
SOAP_FMAC3 _ns1__markReadResponse * SOAP_FMAC4 soap_get__ns1__markReadResponse(struct soap*, _ns1__markReadResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__markReadResponse * SOAP_FMAC4 soap_in__ns1__markReadResponse(struct soap*, const char*, _ns1__markReadResponse *, const char*);

SOAP_FMAC5 _ns1__markReadResponse * SOAP_FMAC6 soap_new__ns1__markReadResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__markReadResponse(struct soap*, _ns1__markReadResponse*);
SOAP_FMAC5 _ns1__markReadResponse * SOAP_FMAC6 soap_instantiate__ns1__markReadResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__markReadResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__markReadRequest
#define SOAP_TYPE__ns1__markReadRequest (179)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__markReadRequest(struct soap*, const char*, int, const _ns1__markReadRequest *, const char*);
SOAP_FMAC3 _ns1__markReadRequest * SOAP_FMAC4 soap_get__ns1__markReadRequest(struct soap*, _ns1__markReadRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__markReadRequest * SOAP_FMAC4 soap_in__ns1__markReadRequest(struct soap*, const char*, _ns1__markReadRequest *, const char*);

SOAP_FMAC5 _ns1__markReadRequest * SOAP_FMAC6 soap_new__ns1__markReadRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__markReadRequest(struct soap*, _ns1__markReadRequest*);
SOAP_FMAC5 _ns1__markReadRequest * SOAP_FMAC6 soap_instantiate__ns1__markReadRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__markReadRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__logoutResponse
#define SOAP_TYPE__ns1__logoutResponse (178)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__logoutResponse(struct soap*, const char*, int, const _ns1__logoutResponse *, const char*);
SOAP_FMAC3 _ns1__logoutResponse * SOAP_FMAC4 soap_get__ns1__logoutResponse(struct soap*, _ns1__logoutResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__logoutResponse * SOAP_FMAC4 soap_in__ns1__logoutResponse(struct soap*, const char*, _ns1__logoutResponse *, const char*);

SOAP_FMAC5 _ns1__logoutResponse * SOAP_FMAC6 soap_new__ns1__logoutResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__logoutResponse(struct soap*, _ns1__logoutResponse*);
SOAP_FMAC5 _ns1__logoutResponse * SOAP_FMAC6 soap_instantiate__ns1__logoutResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__logoutResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__loginResponse
#define SOAP_TYPE__ns1__loginResponse (177)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__loginResponse(struct soap*, const char*, int, const _ns1__loginResponse *, const char*);
SOAP_FMAC3 _ns1__loginResponse * SOAP_FMAC4 soap_get__ns1__loginResponse(struct soap*, _ns1__loginResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__loginResponse * SOAP_FMAC4 soap_in__ns1__loginResponse(struct soap*, const char*, _ns1__loginResponse *, const char*);

SOAP_FMAC5 _ns1__loginResponse * SOAP_FMAC6 soap_new__ns1__loginResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__loginResponse(struct soap*, _ns1__loginResponse*);
SOAP_FMAC5 _ns1__loginResponse * SOAP_FMAC6 soap_instantiate__ns1__loginResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__loginResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__loginRequest
#define SOAP_TYPE__ns1__loginRequest (176)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__loginRequest(struct soap*, const char*, int, const _ns1__loginRequest *, const char*);
SOAP_FMAC3 _ns1__loginRequest * SOAP_FMAC4 soap_get__ns1__loginRequest(struct soap*, _ns1__loginRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__loginRequest * SOAP_FMAC4 soap_in__ns1__loginRequest(struct soap*, const char*, _ns1__loginRequest *, const char*);

SOAP_FMAC5 _ns1__loginRequest * SOAP_FMAC6 soap_new__ns1__loginRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__loginRequest(struct soap*, _ns1__loginRequest*);
SOAP_FMAC5 _ns1__loginRequest * SOAP_FMAC6 soap_instantiate__ns1__loginRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__loginRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getTimezoneListResponse
#define SOAP_TYPE__ns1__getTimezoneListResponse (175)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getTimezoneListResponse(struct soap*, const char*, int, const _ns1__getTimezoneListResponse *, const char*);
SOAP_FMAC3 _ns1__getTimezoneListResponse * SOAP_FMAC4 soap_get__ns1__getTimezoneListResponse(struct soap*, _ns1__getTimezoneListResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getTimezoneListResponse * SOAP_FMAC4 soap_in__ns1__getTimezoneListResponse(struct soap*, const char*, _ns1__getTimezoneListResponse *, const char*);

SOAP_FMAC5 _ns1__getTimezoneListResponse * SOAP_FMAC6 soap_new__ns1__getTimezoneListResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getTimezoneListResponse(struct soap*, _ns1__getTimezoneListResponse*);
SOAP_FMAC5 _ns1__getTimezoneListResponse * SOAP_FMAC6 soap_instantiate__ns1__getTimezoneListResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getTimezoneListResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getSettingsResponse
#define SOAP_TYPE__ns1__getSettingsResponse (174)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getSettingsResponse(struct soap*, const char*, int, const _ns1__getSettingsResponse *, const char*);
SOAP_FMAC3 _ns1__getSettingsResponse * SOAP_FMAC4 soap_get__ns1__getSettingsResponse(struct soap*, _ns1__getSettingsResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getSettingsResponse * SOAP_FMAC4 soap_in__ns1__getSettingsResponse(struct soap*, const char*, _ns1__getSettingsResponse *, const char*);

SOAP_FMAC5 _ns1__getSettingsResponse * SOAP_FMAC6 soap_new__ns1__getSettingsResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getSettingsResponse(struct soap*, _ns1__getSettingsResponse*);
SOAP_FMAC5 _ns1__getSettingsResponse * SOAP_FMAC6 soap_instantiate__ns1__getSettingsResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getSettingsResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getSettingsRequest
#define SOAP_TYPE__ns1__getSettingsRequest (173)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getSettingsRequest(struct soap*, const char*, int, const _ns1__getSettingsRequest *, const char*);
SOAP_FMAC3 _ns1__getSettingsRequest * SOAP_FMAC4 soap_get__ns1__getSettingsRequest(struct soap*, _ns1__getSettingsRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__getSettingsRequest * SOAP_FMAC4 soap_in__ns1__getSettingsRequest(struct soap*, const char*, _ns1__getSettingsRequest *, const char*);

SOAP_FMAC5 _ns1__getSettingsRequest * SOAP_FMAC6 soap_new__ns1__getSettingsRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getSettingsRequest(struct soap*, _ns1__getSettingsRequest*);
SOAP_FMAC5 _ns1__getSettingsRequest * SOAP_FMAC6 soap_instantiate__ns1__getSettingsRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getSettingsRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getProxyListResponse
#define SOAP_TYPE__ns1__getProxyListResponse (172)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getProxyListResponse(struct soap*, const char*, int, const _ns1__getProxyListResponse *, const char*);
SOAP_FMAC3 _ns1__getProxyListResponse * SOAP_FMAC4 soap_get__ns1__getProxyListResponse(struct soap*, _ns1__getProxyListResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getProxyListResponse * SOAP_FMAC4 soap_in__ns1__getProxyListResponse(struct soap*, const char*, _ns1__getProxyListResponse *, const char*);

SOAP_FMAC5 _ns1__getProxyListResponse * SOAP_FMAC6 soap_new__ns1__getProxyListResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getProxyListResponse(struct soap*, _ns1__getProxyListResponse*);
SOAP_FMAC5 _ns1__getProxyListResponse * SOAP_FMAC6 soap_instantiate__ns1__getProxyListResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getProxyListResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getProxyAccessListResponse
#define SOAP_TYPE__ns1__getProxyAccessListResponse (171)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getProxyAccessListResponse(struct soap*, const char*, int, const _ns1__getProxyAccessListResponse *, const char*);
SOAP_FMAC3 _ns1__getProxyAccessListResponse * SOAP_FMAC4 soap_get__ns1__getProxyAccessListResponse(struct soap*, _ns1__getProxyAccessListResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getProxyAccessListResponse * SOAP_FMAC4 soap_in__ns1__getProxyAccessListResponse(struct soap*, const char*, _ns1__getProxyAccessListResponse *, const char*);

SOAP_FMAC5 _ns1__getProxyAccessListResponse * SOAP_FMAC6 soap_new__ns1__getProxyAccessListResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getProxyAccessListResponse(struct soap*, _ns1__getProxyAccessListResponse*);
SOAP_FMAC5 _ns1__getProxyAccessListResponse * SOAP_FMAC6 soap_instantiate__ns1__getProxyAccessListResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getProxyAccessListResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getItemsResponse
#define SOAP_TYPE__ns1__getItemsResponse (170)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getItemsResponse(struct soap*, const char*, int, const _ns1__getItemsResponse *, const char*);
SOAP_FMAC3 _ns1__getItemsResponse * SOAP_FMAC4 soap_get__ns1__getItemsResponse(struct soap*, _ns1__getItemsResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getItemsResponse * SOAP_FMAC4 soap_in__ns1__getItemsResponse(struct soap*, const char*, _ns1__getItemsResponse *, const char*);

SOAP_FMAC5 _ns1__getItemsResponse * SOAP_FMAC6 soap_new__ns1__getItemsResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getItemsResponse(struct soap*, _ns1__getItemsResponse*);
SOAP_FMAC5 _ns1__getItemsResponse * SOAP_FMAC6 soap_instantiate__ns1__getItemsResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getItemsResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getItemsRequest
#define SOAP_TYPE__ns1__getItemsRequest (169)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getItemsRequest(struct soap*, const char*, int, const _ns1__getItemsRequest *, const char*);
SOAP_FMAC3 _ns1__getItemsRequest * SOAP_FMAC4 soap_get__ns1__getItemsRequest(struct soap*, _ns1__getItemsRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__getItemsRequest * SOAP_FMAC4 soap_in__ns1__getItemsRequest(struct soap*, const char*, _ns1__getItemsRequest *, const char*);

SOAP_FMAC5 _ns1__getItemsRequest * SOAP_FMAC6 soap_new__ns1__getItemsRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getItemsRequest(struct soap*, _ns1__getItemsRequest*);
SOAP_FMAC5 _ns1__getItemsRequest * SOAP_FMAC6 soap_instantiate__ns1__getItemsRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getItemsRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getItemResponse
#define SOAP_TYPE__ns1__getItemResponse (168)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getItemResponse(struct soap*, const char*, int, const _ns1__getItemResponse *, const char*);
SOAP_FMAC3 _ns1__getItemResponse * SOAP_FMAC4 soap_get__ns1__getItemResponse(struct soap*, _ns1__getItemResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getItemResponse * SOAP_FMAC4 soap_in__ns1__getItemResponse(struct soap*, const char*, _ns1__getItemResponse *, const char*);

SOAP_FMAC5 _ns1__getItemResponse * SOAP_FMAC6 soap_new__ns1__getItemResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getItemResponse(struct soap*, _ns1__getItemResponse*);
SOAP_FMAC5 _ns1__getItemResponse * SOAP_FMAC6 soap_instantiate__ns1__getItemResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getItemResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getItemRequest
#define SOAP_TYPE__ns1__getItemRequest (167)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getItemRequest(struct soap*, const char*, int, const _ns1__getItemRequest *, const char*);
SOAP_FMAC3 _ns1__getItemRequest * SOAP_FMAC4 soap_get__ns1__getItemRequest(struct soap*, _ns1__getItemRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__getItemRequest * SOAP_FMAC4 soap_in__ns1__getItemRequest(struct soap*, const char*, _ns1__getItemRequest *, const char*);

SOAP_FMAC5 _ns1__getItemRequest * SOAP_FMAC6 soap_new__ns1__getItemRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getItemRequest(struct soap*, _ns1__getItemRequest*);
SOAP_FMAC5 _ns1__getItemRequest * SOAP_FMAC6 soap_instantiate__ns1__getItemRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getItemRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getFreeBusyResponse
#define SOAP_TYPE__ns1__getFreeBusyResponse (166)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getFreeBusyResponse(struct soap*, const char*, int, const _ns1__getFreeBusyResponse *, const char*);
SOAP_FMAC3 _ns1__getFreeBusyResponse * SOAP_FMAC4 soap_get__ns1__getFreeBusyResponse(struct soap*, _ns1__getFreeBusyResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getFreeBusyResponse * SOAP_FMAC4 soap_in__ns1__getFreeBusyResponse(struct soap*, const char*, _ns1__getFreeBusyResponse *, const char*);

SOAP_FMAC5 _ns1__getFreeBusyResponse * SOAP_FMAC6 soap_new__ns1__getFreeBusyResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getFreeBusyResponse(struct soap*, _ns1__getFreeBusyResponse*);
SOAP_FMAC5 _ns1__getFreeBusyResponse * SOAP_FMAC6 soap_instantiate__ns1__getFreeBusyResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getFreeBusyResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getFreeBusyRequest
#define SOAP_TYPE__ns1__getFreeBusyRequest (165)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getFreeBusyRequest(struct soap*, const char*, int, const _ns1__getFreeBusyRequest *, const char*);
SOAP_FMAC3 _ns1__getFreeBusyRequest * SOAP_FMAC4 soap_get__ns1__getFreeBusyRequest(struct soap*, _ns1__getFreeBusyRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__getFreeBusyRequest * SOAP_FMAC4 soap_in__ns1__getFreeBusyRequest(struct soap*, const char*, _ns1__getFreeBusyRequest *, const char*);

SOAP_FMAC5 _ns1__getFreeBusyRequest * SOAP_FMAC6 soap_new__ns1__getFreeBusyRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getFreeBusyRequest(struct soap*, _ns1__getFreeBusyRequest*);
SOAP_FMAC5 _ns1__getFreeBusyRequest * SOAP_FMAC6 soap_instantiate__ns1__getFreeBusyRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getFreeBusyRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getFolderListResponse
#define SOAP_TYPE__ns1__getFolderListResponse (164)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getFolderListResponse(struct soap*, const char*, int, const _ns1__getFolderListResponse *, const char*);
SOAP_FMAC3 _ns1__getFolderListResponse * SOAP_FMAC4 soap_get__ns1__getFolderListResponse(struct soap*, _ns1__getFolderListResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getFolderListResponse * SOAP_FMAC4 soap_in__ns1__getFolderListResponse(struct soap*, const char*, _ns1__getFolderListResponse *, const char*);

SOAP_FMAC5 _ns1__getFolderListResponse * SOAP_FMAC6 soap_new__ns1__getFolderListResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getFolderListResponse(struct soap*, _ns1__getFolderListResponse*);
SOAP_FMAC5 _ns1__getFolderListResponse * SOAP_FMAC6 soap_instantiate__ns1__getFolderListResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getFolderListResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getFolderListRequest
#define SOAP_TYPE__ns1__getFolderListRequest (163)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getFolderListRequest(struct soap*, const char*, int, const _ns1__getFolderListRequest *, const char*);
SOAP_FMAC3 _ns1__getFolderListRequest * SOAP_FMAC4 soap_get__ns1__getFolderListRequest(struct soap*, _ns1__getFolderListRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__getFolderListRequest * SOAP_FMAC4 soap_in__ns1__getFolderListRequest(struct soap*, const char*, _ns1__getFolderListRequest *, const char*);

SOAP_FMAC5 _ns1__getFolderListRequest * SOAP_FMAC6 soap_new__ns1__getFolderListRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getFolderListRequest(struct soap*, _ns1__getFolderListRequest*);
SOAP_FMAC5 _ns1__getFolderListRequest * SOAP_FMAC6 soap_instantiate__ns1__getFolderListRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getFolderListRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getDeltaResponse
#define SOAP_TYPE__ns1__getDeltaResponse (162)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getDeltaResponse(struct soap*, const char*, int, const _ns1__getDeltaResponse *, const char*);
SOAP_FMAC3 _ns1__getDeltaResponse * SOAP_FMAC4 soap_get__ns1__getDeltaResponse(struct soap*, _ns1__getDeltaResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getDeltaResponse * SOAP_FMAC4 soap_in__ns1__getDeltaResponse(struct soap*, const char*, _ns1__getDeltaResponse *, const char*);

SOAP_FMAC5 _ns1__getDeltaResponse * SOAP_FMAC6 soap_new__ns1__getDeltaResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getDeltaResponse(struct soap*, _ns1__getDeltaResponse*);
SOAP_FMAC5 _ns1__getDeltaResponse * SOAP_FMAC6 soap_instantiate__ns1__getDeltaResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getDeltaResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getDeltaRequest
#define SOAP_TYPE__ns1__getDeltaRequest (161)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getDeltaRequest(struct soap*, const char*, int, const _ns1__getDeltaRequest *, const char*);
SOAP_FMAC3 _ns1__getDeltaRequest * SOAP_FMAC4 soap_get__ns1__getDeltaRequest(struct soap*, _ns1__getDeltaRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__getDeltaRequest * SOAP_FMAC4 soap_in__ns1__getDeltaRequest(struct soap*, const char*, _ns1__getDeltaRequest *, const char*);

SOAP_FMAC5 _ns1__getDeltaRequest * SOAP_FMAC6 soap_new__ns1__getDeltaRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getDeltaRequest(struct soap*, _ns1__getDeltaRequest*);
SOAP_FMAC5 _ns1__getDeltaRequest * SOAP_FMAC6 soap_instantiate__ns1__getDeltaRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getDeltaRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getCustomListResponse
#define SOAP_TYPE__ns1__getCustomListResponse (160)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getCustomListResponse(struct soap*, const char*, int, const _ns1__getCustomListResponse *, const char*);
SOAP_FMAC3 _ns1__getCustomListResponse * SOAP_FMAC4 soap_get__ns1__getCustomListResponse(struct soap*, _ns1__getCustomListResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getCustomListResponse * SOAP_FMAC4 soap_in__ns1__getCustomListResponse(struct soap*, const char*, _ns1__getCustomListResponse *, const char*);

SOAP_FMAC5 _ns1__getCustomListResponse * SOAP_FMAC6 soap_new__ns1__getCustomListResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getCustomListResponse(struct soap*, _ns1__getCustomListResponse*);
SOAP_FMAC5 _ns1__getCustomListResponse * SOAP_FMAC6 soap_instantiate__ns1__getCustomListResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getCustomListResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getCategoryListResponse
#define SOAP_TYPE__ns1__getCategoryListResponse (159)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getCategoryListResponse(struct soap*, const char*, int, const _ns1__getCategoryListResponse *, const char*);
SOAP_FMAC3 _ns1__getCategoryListResponse * SOAP_FMAC4 soap_get__ns1__getCategoryListResponse(struct soap*, _ns1__getCategoryListResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getCategoryListResponse * SOAP_FMAC4 soap_in__ns1__getCategoryListResponse(struct soap*, const char*, _ns1__getCategoryListResponse *, const char*);

SOAP_FMAC5 _ns1__getCategoryListResponse * SOAP_FMAC6 soap_new__ns1__getCategoryListResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getCategoryListResponse(struct soap*, _ns1__getCategoryListResponse*);
SOAP_FMAC5 _ns1__getCategoryListResponse * SOAP_FMAC6 soap_instantiate__ns1__getCategoryListResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getCategoryListResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getAttachmentResponse
#define SOAP_TYPE__ns1__getAttachmentResponse (158)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getAttachmentResponse(struct soap*, const char*, int, const _ns1__getAttachmentResponse *, const char*);
SOAP_FMAC3 _ns1__getAttachmentResponse * SOAP_FMAC4 soap_get__ns1__getAttachmentResponse(struct soap*, _ns1__getAttachmentResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getAttachmentResponse * SOAP_FMAC4 soap_in__ns1__getAttachmentResponse(struct soap*, const char*, _ns1__getAttachmentResponse *, const char*);

SOAP_FMAC5 _ns1__getAttachmentResponse * SOAP_FMAC6 soap_new__ns1__getAttachmentResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getAttachmentResponse(struct soap*, _ns1__getAttachmentResponse*);
SOAP_FMAC5 _ns1__getAttachmentResponse * SOAP_FMAC6 soap_instantiate__ns1__getAttachmentResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getAttachmentResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getAttachmentRequest
#define SOAP_TYPE__ns1__getAttachmentRequest (157)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getAttachmentRequest(struct soap*, const char*, int, const _ns1__getAttachmentRequest *, const char*);
SOAP_FMAC3 _ns1__getAttachmentRequest * SOAP_FMAC4 soap_get__ns1__getAttachmentRequest(struct soap*, _ns1__getAttachmentRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__getAttachmentRequest * SOAP_FMAC4 soap_in__ns1__getAttachmentRequest(struct soap*, const char*, _ns1__getAttachmentRequest *, const char*);

SOAP_FMAC5 _ns1__getAttachmentRequest * SOAP_FMAC6 soap_new__ns1__getAttachmentRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getAttachmentRequest(struct soap*, _ns1__getAttachmentRequest*);
SOAP_FMAC5 _ns1__getAttachmentRequest * SOAP_FMAC6 soap_instantiate__ns1__getAttachmentRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getAttachmentRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__getAddressBookListResponse
#define SOAP_TYPE__ns1__getAddressBookListResponse (156)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__getAddressBookListResponse(struct soap*, const char*, int, const _ns1__getAddressBookListResponse *, const char*);
SOAP_FMAC3 _ns1__getAddressBookListResponse * SOAP_FMAC4 soap_get__ns1__getAddressBookListResponse(struct soap*, _ns1__getAddressBookListResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__getAddressBookListResponse * SOAP_FMAC4 soap_in__ns1__getAddressBookListResponse(struct soap*, const char*, _ns1__getAddressBookListResponse *, const char*);

SOAP_FMAC5 _ns1__getAddressBookListResponse * SOAP_FMAC6 soap_new__ns1__getAddressBookListResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__getAddressBookListResponse(struct soap*, _ns1__getAddressBookListResponse*);
SOAP_FMAC5 _ns1__getAddressBookListResponse * SOAP_FMAC6 soap_instantiate__ns1__getAddressBookListResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__getAddressBookListResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__destroyCursorResponse
#define SOAP_TYPE__ns1__destroyCursorResponse (155)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__destroyCursorResponse(struct soap*, const char*, int, const _ns1__destroyCursorResponse *, const char*);
SOAP_FMAC3 _ns1__destroyCursorResponse * SOAP_FMAC4 soap_get__ns1__destroyCursorResponse(struct soap*, _ns1__destroyCursorResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__destroyCursorResponse * SOAP_FMAC4 soap_in__ns1__destroyCursorResponse(struct soap*, const char*, _ns1__destroyCursorResponse *, const char*);

SOAP_FMAC5 _ns1__destroyCursorResponse * SOAP_FMAC6 soap_new__ns1__destroyCursorResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__destroyCursorResponse(struct soap*, _ns1__destroyCursorResponse*);
SOAP_FMAC5 _ns1__destroyCursorResponse * SOAP_FMAC6 soap_instantiate__ns1__destroyCursorResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__destroyCursorResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__destroyCursorRequest
#define SOAP_TYPE__ns1__destroyCursorRequest (154)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__destroyCursorRequest(struct soap*, const char*, int, const _ns1__destroyCursorRequest *, const char*);
SOAP_FMAC3 _ns1__destroyCursorRequest * SOAP_FMAC4 soap_get__ns1__destroyCursorRequest(struct soap*, _ns1__destroyCursorRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__destroyCursorRequest * SOAP_FMAC4 soap_in__ns1__destroyCursorRequest(struct soap*, const char*, _ns1__destroyCursorRequest *, const char*);

SOAP_FMAC5 _ns1__destroyCursorRequest * SOAP_FMAC6 soap_new__ns1__destroyCursorRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__destroyCursorRequest(struct soap*, _ns1__destroyCursorRequest*);
SOAP_FMAC5 _ns1__destroyCursorRequest * SOAP_FMAC6 soap_instantiate__ns1__destroyCursorRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__destroyCursorRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__delegateResponse
#define SOAP_TYPE__ns1__delegateResponse (153)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__delegateResponse(struct soap*, const char*, int, const _ns1__delegateResponse *, const char*);
SOAP_FMAC3 _ns1__delegateResponse * SOAP_FMAC4 soap_get__ns1__delegateResponse(struct soap*, _ns1__delegateResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__delegateResponse * SOAP_FMAC4 soap_in__ns1__delegateResponse(struct soap*, const char*, _ns1__delegateResponse *, const char*);

SOAP_FMAC5 _ns1__delegateResponse * SOAP_FMAC6 soap_new__ns1__delegateResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__delegateResponse(struct soap*, _ns1__delegateResponse*);
SOAP_FMAC5 _ns1__delegateResponse * SOAP_FMAC6 soap_instantiate__ns1__delegateResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__delegateResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__delegateRequest
#define SOAP_TYPE__ns1__delegateRequest (152)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__delegateRequest(struct soap*, const char*, int, const _ns1__delegateRequest *, const char*);
SOAP_FMAC3 _ns1__delegateRequest * SOAP_FMAC4 soap_get__ns1__delegateRequest(struct soap*, _ns1__delegateRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__delegateRequest * SOAP_FMAC4 soap_in__ns1__delegateRequest(struct soap*, const char*, _ns1__delegateRequest *, const char*);

SOAP_FMAC5 _ns1__delegateRequest * SOAP_FMAC6 soap_new__ns1__delegateRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__delegateRequest(struct soap*, _ns1__delegateRequest*);
SOAP_FMAC5 _ns1__delegateRequest * SOAP_FMAC6 soap_instantiate__ns1__delegateRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__delegateRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__declineResponse
#define SOAP_TYPE__ns1__declineResponse (151)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__declineResponse(struct soap*, const char*, int, const _ns1__declineResponse *, const char*);
SOAP_FMAC3 _ns1__declineResponse * SOAP_FMAC4 soap_get__ns1__declineResponse(struct soap*, _ns1__declineResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__declineResponse * SOAP_FMAC4 soap_in__ns1__declineResponse(struct soap*, const char*, _ns1__declineResponse *, const char*);

SOAP_FMAC5 _ns1__declineResponse * SOAP_FMAC6 soap_new__ns1__declineResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__declineResponse(struct soap*, _ns1__declineResponse*);
SOAP_FMAC5 _ns1__declineResponse * SOAP_FMAC6 soap_instantiate__ns1__declineResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__declineResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__declineRequest
#define SOAP_TYPE__ns1__declineRequest (150)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__declineRequest(struct soap*, const char*, int, const _ns1__declineRequest *, const char*);
SOAP_FMAC3 _ns1__declineRequest * SOAP_FMAC4 soap_get__ns1__declineRequest(struct soap*, _ns1__declineRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__declineRequest * SOAP_FMAC4 soap_in__ns1__declineRequest(struct soap*, const char*, _ns1__declineRequest *, const char*);

SOAP_FMAC5 _ns1__declineRequest * SOAP_FMAC6 soap_new__ns1__declineRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__declineRequest(struct soap*, _ns1__declineRequest*);
SOAP_FMAC5 _ns1__declineRequest * SOAP_FMAC6 soap_instantiate__ns1__declineRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__declineRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__createProxyAccessResponse
#define SOAP_TYPE__ns1__createProxyAccessResponse (149)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__createProxyAccessResponse(struct soap*, const char*, int, const _ns1__createProxyAccessResponse *, const char*);
SOAP_FMAC3 _ns1__createProxyAccessResponse * SOAP_FMAC4 soap_get__ns1__createProxyAccessResponse(struct soap*, _ns1__createProxyAccessResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__createProxyAccessResponse * SOAP_FMAC4 soap_in__ns1__createProxyAccessResponse(struct soap*, const char*, _ns1__createProxyAccessResponse *, const char*);

SOAP_FMAC5 _ns1__createProxyAccessResponse * SOAP_FMAC6 soap_new__ns1__createProxyAccessResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__createProxyAccessResponse(struct soap*, _ns1__createProxyAccessResponse*);
SOAP_FMAC5 _ns1__createProxyAccessResponse * SOAP_FMAC6 soap_instantiate__ns1__createProxyAccessResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__createProxyAccessResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__createProxyAccessRequest
#define SOAP_TYPE__ns1__createProxyAccessRequest (148)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__createProxyAccessRequest(struct soap*, const char*, int, const _ns1__createProxyAccessRequest *, const char*);
SOAP_FMAC3 _ns1__createProxyAccessRequest * SOAP_FMAC4 soap_get__ns1__createProxyAccessRequest(struct soap*, _ns1__createProxyAccessRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__createProxyAccessRequest * SOAP_FMAC4 soap_in__ns1__createProxyAccessRequest(struct soap*, const char*, _ns1__createProxyAccessRequest *, const char*);

SOAP_FMAC5 _ns1__createProxyAccessRequest * SOAP_FMAC6 soap_new__ns1__createProxyAccessRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__createProxyAccessRequest(struct soap*, _ns1__createProxyAccessRequest*);
SOAP_FMAC5 _ns1__createProxyAccessRequest * SOAP_FMAC6 soap_instantiate__ns1__createProxyAccessRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__createProxyAccessRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__createItemResponse
#define SOAP_TYPE__ns1__createItemResponse (147)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__createItemResponse(struct soap*, const char*, int, const _ns1__createItemResponse *, const char*);
SOAP_FMAC3 _ns1__createItemResponse * SOAP_FMAC4 soap_get__ns1__createItemResponse(struct soap*, _ns1__createItemResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__createItemResponse * SOAP_FMAC4 soap_in__ns1__createItemResponse(struct soap*, const char*, _ns1__createItemResponse *, const char*);

SOAP_FMAC5 _ns1__createItemResponse * SOAP_FMAC6 soap_new__ns1__createItemResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__createItemResponse(struct soap*, _ns1__createItemResponse*);
SOAP_FMAC5 _ns1__createItemResponse * SOAP_FMAC6 soap_instantiate__ns1__createItemResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__createItemResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__createItemRequest
#define SOAP_TYPE__ns1__createItemRequest (146)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__createItemRequest(struct soap*, const char*, int, const _ns1__createItemRequest *, const char*);
SOAP_FMAC3 _ns1__createItemRequest * SOAP_FMAC4 soap_get__ns1__createItemRequest(struct soap*, _ns1__createItemRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__createItemRequest * SOAP_FMAC4 soap_in__ns1__createItemRequest(struct soap*, const char*, _ns1__createItemRequest *, const char*);

SOAP_FMAC5 _ns1__createItemRequest * SOAP_FMAC6 soap_new__ns1__createItemRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__createItemRequest(struct soap*, _ns1__createItemRequest*);
SOAP_FMAC5 _ns1__createItemRequest * SOAP_FMAC6 soap_instantiate__ns1__createItemRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__createItemRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__createCursorResponse
#define SOAP_TYPE__ns1__createCursorResponse (145)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__createCursorResponse(struct soap*, const char*, int, const _ns1__createCursorResponse *, const char*);
SOAP_FMAC3 _ns1__createCursorResponse * SOAP_FMAC4 soap_get__ns1__createCursorResponse(struct soap*, _ns1__createCursorResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__createCursorResponse * SOAP_FMAC4 soap_in__ns1__createCursorResponse(struct soap*, const char*, _ns1__createCursorResponse *, const char*);

SOAP_FMAC5 _ns1__createCursorResponse * SOAP_FMAC6 soap_new__ns1__createCursorResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__createCursorResponse(struct soap*, _ns1__createCursorResponse*);
SOAP_FMAC5 _ns1__createCursorResponse * SOAP_FMAC6 soap_instantiate__ns1__createCursorResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__createCursorResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__createCursorRequest
#define SOAP_TYPE__ns1__createCursorRequest (144)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__createCursorRequest(struct soap*, const char*, int, const _ns1__createCursorRequest *, const char*);
SOAP_FMAC3 _ns1__createCursorRequest * SOAP_FMAC4 soap_get__ns1__createCursorRequest(struct soap*, _ns1__createCursorRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__createCursorRequest * SOAP_FMAC4 soap_in__ns1__createCursorRequest(struct soap*, const char*, _ns1__createCursorRequest *, const char*);

SOAP_FMAC5 _ns1__createCursorRequest * SOAP_FMAC6 soap_new__ns1__createCursorRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__createCursorRequest(struct soap*, _ns1__createCursorRequest*);
SOAP_FMAC5 _ns1__createCursorRequest * SOAP_FMAC6 soap_instantiate__ns1__createCursorRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__createCursorRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__completeResponse
#define SOAP_TYPE__ns1__completeResponse (143)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__completeResponse(struct soap*, const char*, int, const _ns1__completeResponse *, const char*);
SOAP_FMAC3 _ns1__completeResponse * SOAP_FMAC4 soap_get__ns1__completeResponse(struct soap*, _ns1__completeResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__completeResponse * SOAP_FMAC4 soap_in__ns1__completeResponse(struct soap*, const char*, _ns1__completeResponse *, const char*);

SOAP_FMAC5 _ns1__completeResponse * SOAP_FMAC6 soap_new__ns1__completeResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__completeResponse(struct soap*, _ns1__completeResponse*);
SOAP_FMAC5 _ns1__completeResponse * SOAP_FMAC6 soap_instantiate__ns1__completeResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__completeResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__completeRequest
#define SOAP_TYPE__ns1__completeRequest (142)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__completeRequest(struct soap*, const char*, int, const _ns1__completeRequest *, const char*);
SOAP_FMAC3 _ns1__completeRequest * SOAP_FMAC4 soap_get__ns1__completeRequest(struct soap*, _ns1__completeRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__completeRequest * SOAP_FMAC4 soap_in__ns1__completeRequest(struct soap*, const char*, _ns1__completeRequest *, const char*);

SOAP_FMAC5 _ns1__completeRequest * SOAP_FMAC6 soap_new__ns1__completeRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__completeRequest(struct soap*, _ns1__completeRequest*);
SOAP_FMAC5 _ns1__completeRequest * SOAP_FMAC6 soap_instantiate__ns1__completeRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__completeRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__closeFreeBusySessionResponse
#define SOAP_TYPE__ns1__closeFreeBusySessionResponse (141)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__closeFreeBusySessionResponse(struct soap*, const char*, int, const _ns1__closeFreeBusySessionResponse *, const char*);
SOAP_FMAC3 _ns1__closeFreeBusySessionResponse * SOAP_FMAC4 soap_get__ns1__closeFreeBusySessionResponse(struct soap*, _ns1__closeFreeBusySessionResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__closeFreeBusySessionResponse * SOAP_FMAC4 soap_in__ns1__closeFreeBusySessionResponse(struct soap*, const char*, _ns1__closeFreeBusySessionResponse *, const char*);

SOAP_FMAC5 _ns1__closeFreeBusySessionResponse * SOAP_FMAC6 soap_new__ns1__closeFreeBusySessionResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__closeFreeBusySessionResponse(struct soap*, _ns1__closeFreeBusySessionResponse*);
SOAP_FMAC5 _ns1__closeFreeBusySessionResponse * SOAP_FMAC6 soap_instantiate__ns1__closeFreeBusySessionResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__closeFreeBusySessionResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__closeFreeBusySessionRequest
#define SOAP_TYPE__ns1__closeFreeBusySessionRequest (140)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__closeFreeBusySessionRequest(struct soap*, const char*, int, const _ns1__closeFreeBusySessionRequest *, const char*);
SOAP_FMAC3 _ns1__closeFreeBusySessionRequest * SOAP_FMAC4 soap_get__ns1__closeFreeBusySessionRequest(struct soap*, _ns1__closeFreeBusySessionRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__closeFreeBusySessionRequest * SOAP_FMAC4 soap_in__ns1__closeFreeBusySessionRequest(struct soap*, const char*, _ns1__closeFreeBusySessionRequest *, const char*);

SOAP_FMAC5 _ns1__closeFreeBusySessionRequest * SOAP_FMAC6 soap_new__ns1__closeFreeBusySessionRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__closeFreeBusySessionRequest(struct soap*, _ns1__closeFreeBusySessionRequest*);
SOAP_FMAC5 _ns1__closeFreeBusySessionRequest * SOAP_FMAC6 soap_instantiate__ns1__closeFreeBusySessionRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__closeFreeBusySessionRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__addMembersResponse
#define SOAP_TYPE__ns1__addMembersResponse (139)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__addMembersResponse(struct soap*, const char*, int, const _ns1__addMembersResponse *, const char*);
SOAP_FMAC3 _ns1__addMembersResponse * SOAP_FMAC4 soap_get__ns1__addMembersResponse(struct soap*, _ns1__addMembersResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__addMembersResponse * SOAP_FMAC4 soap_in__ns1__addMembersResponse(struct soap*, const char*, _ns1__addMembersResponse *, const char*);

SOAP_FMAC5 _ns1__addMembersResponse * SOAP_FMAC6 soap_new__ns1__addMembersResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__addMembersResponse(struct soap*, _ns1__addMembersResponse*);
SOAP_FMAC5 _ns1__addMembersResponse * SOAP_FMAC6 soap_instantiate__ns1__addMembersResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__addMembersResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__addMembersRequest
#define SOAP_TYPE__ns1__addMembersRequest (138)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__addMembersRequest(struct soap*, const char*, int, const _ns1__addMembersRequest *, const char*);
SOAP_FMAC3 _ns1__addMembersRequest * SOAP_FMAC4 soap_get__ns1__addMembersRequest(struct soap*, _ns1__addMembersRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__addMembersRequest * SOAP_FMAC4 soap_in__ns1__addMembersRequest(struct soap*, const char*, _ns1__addMembersRequest *, const char*);

SOAP_FMAC5 _ns1__addMembersRequest * SOAP_FMAC6 soap_new__ns1__addMembersRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__addMembersRequest(struct soap*, _ns1__addMembersRequest*);
SOAP_FMAC5 _ns1__addMembersRequest * SOAP_FMAC6 soap_instantiate__ns1__addMembersRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__addMembersRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__addItemsResponse
#define SOAP_TYPE__ns1__addItemsResponse (137)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__addItemsResponse(struct soap*, const char*, int, const _ns1__addItemsResponse *, const char*);
SOAP_FMAC3 _ns1__addItemsResponse * SOAP_FMAC4 soap_get__ns1__addItemsResponse(struct soap*, _ns1__addItemsResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__addItemsResponse * SOAP_FMAC4 soap_in__ns1__addItemsResponse(struct soap*, const char*, _ns1__addItemsResponse *, const char*);

SOAP_FMAC5 _ns1__addItemsResponse * SOAP_FMAC6 soap_new__ns1__addItemsResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__addItemsResponse(struct soap*, _ns1__addItemsResponse*);
SOAP_FMAC5 _ns1__addItemsResponse * SOAP_FMAC6 soap_instantiate__ns1__addItemsResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__addItemsResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__addItemsRequest
#define SOAP_TYPE__ns1__addItemsRequest (136)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__addItemsRequest(struct soap*, const char*, int, const _ns1__addItemsRequest *, const char*);
SOAP_FMAC3 _ns1__addItemsRequest * SOAP_FMAC4 soap_get__ns1__addItemsRequest(struct soap*, _ns1__addItemsRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__addItemsRequest * SOAP_FMAC4 soap_in__ns1__addItemsRequest(struct soap*, const char*, _ns1__addItemsRequest *, const char*);

SOAP_FMAC5 _ns1__addItemsRequest * SOAP_FMAC6 soap_new__ns1__addItemsRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__addItemsRequest(struct soap*, _ns1__addItemsRequest*);
SOAP_FMAC5 _ns1__addItemsRequest * SOAP_FMAC6 soap_instantiate__ns1__addItemsRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__addItemsRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__addItemResponse
#define SOAP_TYPE__ns1__addItemResponse (135)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__addItemResponse(struct soap*, const char*, int, const _ns1__addItemResponse *, const char*);
SOAP_FMAC3 _ns1__addItemResponse * SOAP_FMAC4 soap_get__ns1__addItemResponse(struct soap*, _ns1__addItemResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__addItemResponse * SOAP_FMAC4 soap_in__ns1__addItemResponse(struct soap*, const char*, _ns1__addItemResponse *, const char*);

SOAP_FMAC5 _ns1__addItemResponse * SOAP_FMAC6 soap_new__ns1__addItemResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__addItemResponse(struct soap*, _ns1__addItemResponse*);
SOAP_FMAC5 _ns1__addItemResponse * SOAP_FMAC6 soap_instantiate__ns1__addItemResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__addItemResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__addItemRequest
#define SOAP_TYPE__ns1__addItemRequest (134)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__addItemRequest(struct soap*, const char*, int, const _ns1__addItemRequest *, const char*);
SOAP_FMAC3 _ns1__addItemRequest * SOAP_FMAC4 soap_get__ns1__addItemRequest(struct soap*, _ns1__addItemRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__addItemRequest * SOAP_FMAC4 soap_in__ns1__addItemRequest(struct soap*, const char*, _ns1__addItemRequest *, const char*);

SOAP_FMAC5 _ns1__addItemRequest * SOAP_FMAC6 soap_new__ns1__addItemRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__addItemRequest(struct soap*, _ns1__addItemRequest*);
SOAP_FMAC5 _ns1__addItemRequest * SOAP_FMAC6 soap_instantiate__ns1__addItemRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__addItemRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__acceptResponse
#define SOAP_TYPE__ns1__acceptResponse (133)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__acceptResponse(struct soap*, const char*, int, const _ns1__acceptResponse *, const char*);
SOAP_FMAC3 _ns1__acceptResponse * SOAP_FMAC4 soap_get__ns1__acceptResponse(struct soap*, _ns1__acceptResponse *, const char*, const char*);
SOAP_FMAC3 _ns1__acceptResponse * SOAP_FMAC4 soap_in__ns1__acceptResponse(struct soap*, const char*, _ns1__acceptResponse *, const char*);

SOAP_FMAC5 _ns1__acceptResponse * SOAP_FMAC6 soap_new__ns1__acceptResponse(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__acceptResponse(struct soap*, _ns1__acceptResponse*);
SOAP_FMAC5 _ns1__acceptResponse * SOAP_FMAC6 soap_instantiate__ns1__acceptResponse(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__acceptResponse(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE__ns1__acceptRequest
#define SOAP_TYPE__ns1__acceptRequest (132)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out__ns1__acceptRequest(struct soap*, const char*, int, const _ns1__acceptRequest *, const char*);
SOAP_FMAC3 _ns1__acceptRequest * SOAP_FMAC4 soap_get__ns1__acceptRequest(struct soap*, _ns1__acceptRequest *, const char*, const char*);
SOAP_FMAC3 _ns1__acceptRequest * SOAP_FMAC4 soap_in__ns1__acceptRequest(struct soap*, const char*, _ns1__acceptRequest *, const char*);

SOAP_FMAC5 _ns1__acceptRequest * SOAP_FMAC6 soap_new__ns1__acceptRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete__ns1__acceptRequest(struct soap*, _ns1__acceptRequest*);
SOAP_FMAC5 _ns1__acceptRequest * SOAP_FMAC6 soap_instantiate__ns1__acceptRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy__ns1__acceptRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__UserInfo
#define SOAP_TYPE_ns1__UserInfo (131)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__UserInfo(struct soap*, const char*, int, const ns1__UserInfo *, const char*);
SOAP_FMAC3 ns1__UserInfo * SOAP_FMAC4 soap_get_ns1__UserInfo(struct soap*, ns1__UserInfo *, const char*, const char*);
SOAP_FMAC3 ns1__UserInfo * SOAP_FMAC4 soap_in_ns1__UserInfo(struct soap*, const char*, ns1__UserInfo *, const char*);

SOAP_FMAC5 ns1__UserInfo * SOAP_FMAC6 soap_new_ns1__UserInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__UserInfo(struct soap*, ns1__UserInfo*);
SOAP_FMAC5 ns1__UserInfo * SOAP_FMAC6 soap_instantiate_ns1__UserInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__UserInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__TrustedApplication
#define SOAP_TYPE_ns1__TrustedApplication (130)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__TrustedApplication(struct soap*, const char*, int, const ns1__TrustedApplication *, const char*);
SOAP_FMAC3 ns1__TrustedApplication * SOAP_FMAC4 soap_get_ns1__TrustedApplication(struct soap*, ns1__TrustedApplication *, const char*, const char*);
SOAP_FMAC3 ns1__TrustedApplication * SOAP_FMAC4 soap_in_ns1__TrustedApplication(struct soap*, const char*, ns1__TrustedApplication *, const char*);

SOAP_FMAC5 ns1__TrustedApplication * SOAP_FMAC6 soap_new_ns1__TrustedApplication(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__TrustedApplication(struct soap*, ns1__TrustedApplication*);
SOAP_FMAC5 ns1__TrustedApplication * SOAP_FMAC6 soap_instantiate_ns1__TrustedApplication(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__TrustedApplication(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__TimezoneList
#define SOAP_TYPE_ns1__TimezoneList (129)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__TimezoneList(struct soap*, const char*, int, const ns1__TimezoneList *, const char*);
SOAP_FMAC3 ns1__TimezoneList * SOAP_FMAC4 soap_get_ns1__TimezoneList(struct soap*, ns1__TimezoneList *, const char*, const char*);
SOAP_FMAC3 ns1__TimezoneList * SOAP_FMAC4 soap_in_ns1__TimezoneList(struct soap*, const char*, ns1__TimezoneList *, const char*);

SOAP_FMAC5 ns1__TimezoneList * SOAP_FMAC6 soap_new_ns1__TimezoneList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__TimezoneList(struct soap*, ns1__TimezoneList*);
SOAP_FMAC5 ns1__TimezoneList * SOAP_FMAC6 soap_instantiate_ns1__TimezoneList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__TimezoneList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__TimezoneComponent
#define SOAP_TYPE_ns1__TimezoneComponent (128)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__TimezoneComponent(struct soap*, const char*, int, const ns1__TimezoneComponent *, const char*);
SOAP_FMAC3 ns1__TimezoneComponent * SOAP_FMAC4 soap_get_ns1__TimezoneComponent(struct soap*, ns1__TimezoneComponent *, const char*, const char*);
SOAP_FMAC3 ns1__TimezoneComponent * SOAP_FMAC4 soap_in_ns1__TimezoneComponent(struct soap*, const char*, ns1__TimezoneComponent *, const char*);

SOAP_FMAC5 ns1__TimezoneComponent * SOAP_FMAC6 soap_new_ns1__TimezoneComponent(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__TimezoneComponent(struct soap*, ns1__TimezoneComponent*);
SOAP_FMAC5 ns1__TimezoneComponent * SOAP_FMAC6 soap_instantiate_ns1__TimezoneComponent(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__TimezoneComponent(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Timezone
#define SOAP_TYPE_ns1__Timezone (127)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Timezone(struct soap*, const char*, int, const ns1__Timezone *, const char*);
SOAP_FMAC3 ns1__Timezone * SOAP_FMAC4 soap_get_ns1__Timezone(struct soap*, ns1__Timezone *, const char*, const char*);
SOAP_FMAC3 ns1__Timezone * SOAP_FMAC4 soap_in_ns1__Timezone(struct soap*, const char*, ns1__Timezone *, const char*);

SOAP_FMAC5 ns1__Timezone * SOAP_FMAC6 soap_new_ns1__Timezone(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Timezone(struct soap*, ns1__Timezone*);
SOAP_FMAC5 ns1__Timezone * SOAP_FMAC6 soap_instantiate_ns1__Timezone(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Timezone(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Task
#define SOAP_TYPE_ns1__Task (126)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Task(struct soap*, const char*, int, const ns1__Task *, const char*);
SOAP_FMAC3 ns1__Task * SOAP_FMAC4 soap_get_ns1__Task(struct soap*, ns1__Task *, const char*, const char*);
SOAP_FMAC3 ns1__Task * SOAP_FMAC4 soap_in_ns1__Task(struct soap*, const char*, ns1__Task *, const char*);

SOAP_FMAC5 ns1__Task * SOAP_FMAC6 soap_new_ns1__Task(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Task(struct soap*, ns1__Task*);
SOAP_FMAC5 ns1__Task * SOAP_FMAC6 soap_instantiate_ns1__Task(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Task(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__SystemFolder
#define SOAP_TYPE_ns1__SystemFolder (125)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__SystemFolder(struct soap*, const char*, int, const ns1__SystemFolder *, const char*);
SOAP_FMAC3 ns1__SystemFolder * SOAP_FMAC4 soap_get_ns1__SystemFolder(struct soap*, ns1__SystemFolder *, const char*, const char*);
SOAP_FMAC3 ns1__SystemFolder * SOAP_FMAC4 soap_in_ns1__SystemFolder(struct soap*, const char*, ns1__SystemFolder *, const char*);

SOAP_FMAC5 ns1__SystemFolder * SOAP_FMAC6 soap_new_ns1__SystemFolder(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__SystemFolder(struct soap*, ns1__SystemFolder*);
SOAP_FMAC5 ns1__SystemFolder * SOAP_FMAC6 soap_instantiate_ns1__SystemFolder(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__SystemFolder(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__StatusTracking
#define SOAP_TYPE_ns1__StatusTracking (124)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__StatusTracking(struct soap*, const char*, int, const ns1__StatusTracking *, const char*);
SOAP_FMAC3 ns1__StatusTracking * SOAP_FMAC4 soap_get_ns1__StatusTracking(struct soap*, ns1__StatusTracking *, const char*, const char*);
SOAP_FMAC3 ns1__StatusTracking * SOAP_FMAC4 soap_in_ns1__StatusTracking(struct soap*, const char*, ns1__StatusTracking *, const char*);

SOAP_FMAC5 ns1__StatusTracking * SOAP_FMAC6 soap_new_ns1__StatusTracking(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__StatusTracking(struct soap*, ns1__StatusTracking*);
SOAP_FMAC5 ns1__StatusTracking * SOAP_FMAC6 soap_instantiate_ns1__StatusTracking(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__StatusTracking(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Status
#define SOAP_TYPE_ns1__Status (123)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Status(struct soap*, const char*, int, const ns1__Status *, const char*);
SOAP_FMAC3 ns1__Status * SOAP_FMAC4 soap_get_ns1__Status(struct soap*, ns1__Status *, const char*, const char*);
SOAP_FMAC3 ns1__Status * SOAP_FMAC4 soap_in_ns1__Status(struct soap*, const char*, ns1__Status *, const char*);

SOAP_FMAC5 ns1__Status * SOAP_FMAC6 soap_new_ns1__Status(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Status(struct soap*, ns1__Status*);
SOAP_FMAC5 ns1__Status * SOAP_FMAC6 soap_instantiate_ns1__Status(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Status(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__SharedFolderNotification
#define SOAP_TYPE_ns1__SharedFolderNotification (122)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__SharedFolderNotification(struct soap*, const char*, int, const ns1__SharedFolderNotification *, const char*);
SOAP_FMAC3 ns1__SharedFolderNotification * SOAP_FMAC4 soap_get_ns1__SharedFolderNotification(struct soap*, ns1__SharedFolderNotification *, const char*, const char*);
SOAP_FMAC3 ns1__SharedFolderNotification * SOAP_FMAC4 soap_in_ns1__SharedFolderNotification(struct soap*, const char*, ns1__SharedFolderNotification *, const char*);

SOAP_FMAC5 ns1__SharedFolderNotification * SOAP_FMAC6 soap_new_ns1__SharedFolderNotification(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__SharedFolderNotification(struct soap*, ns1__SharedFolderNotification*);
SOAP_FMAC5 ns1__SharedFolderNotification * SOAP_FMAC6 soap_instantiate_ns1__SharedFolderNotification(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__SharedFolderNotification(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__SharedFolder
#define SOAP_TYPE_ns1__SharedFolder (121)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__SharedFolder(struct soap*, const char*, int, const ns1__SharedFolder *, const char*);
SOAP_FMAC3 ns1__SharedFolder * SOAP_FMAC4 soap_get_ns1__SharedFolder(struct soap*, ns1__SharedFolder *, const char*, const char*);
SOAP_FMAC3 ns1__SharedFolder * SOAP_FMAC4 soap_in_ns1__SharedFolder(struct soap*, const char*, ns1__SharedFolder *, const char*);

SOAP_FMAC5 ns1__SharedFolder * SOAP_FMAC6 soap_new_ns1__SharedFolder(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__SharedFolder(struct soap*, ns1__SharedFolder*);
SOAP_FMAC5 ns1__SharedFolder * SOAP_FMAC6 soap_instantiate_ns1__SharedFolder(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__SharedFolder(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__SettingsList
#define SOAP_TYPE_ns1__SettingsList (120)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__SettingsList(struct soap*, const char*, int, const ns1__SettingsList *, const char*);
SOAP_FMAC3 ns1__SettingsList * SOAP_FMAC4 soap_get_ns1__SettingsList(struct soap*, ns1__SettingsList *, const char*, const char*);
SOAP_FMAC3 ns1__SettingsList * SOAP_FMAC4 soap_in_ns1__SettingsList(struct soap*, const char*, ns1__SettingsList *, const char*);

SOAP_FMAC5 ns1__SettingsList * SOAP_FMAC6 soap_new_ns1__SettingsList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__SettingsList(struct soap*, ns1__SettingsList*);
SOAP_FMAC5 ns1__SettingsList * SOAP_FMAC6 soap_instantiate_ns1__SettingsList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__SettingsList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__SettingsGroup
#define SOAP_TYPE_ns1__SettingsGroup (119)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__SettingsGroup(struct soap*, const char*, int, const ns1__SettingsGroup *, const char*);
SOAP_FMAC3 ns1__SettingsGroup * SOAP_FMAC4 soap_get_ns1__SettingsGroup(struct soap*, ns1__SettingsGroup *, const char*, const char*);
SOAP_FMAC3 ns1__SettingsGroup * SOAP_FMAC4 soap_in_ns1__SettingsGroup(struct soap*, const char*, ns1__SettingsGroup *, const char*);

SOAP_FMAC5 ns1__SettingsGroup * SOAP_FMAC6 soap_new_ns1__SettingsGroup(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__SettingsGroup(struct soap*, ns1__SettingsGroup*);
SOAP_FMAC5 ns1__SettingsGroup * SOAP_FMAC6 soap_instantiate_ns1__SettingsGroup(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__SettingsGroup(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Settings
#define SOAP_TYPE_ns1__Settings (118)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Settings(struct soap*, const char*, int, const ns1__Settings *, const char*);
SOAP_FMAC3 ns1__Settings * SOAP_FMAC4 soap_get_ns1__Settings(struct soap*, ns1__Settings *, const char*, const char*);
SOAP_FMAC3 ns1__Settings * SOAP_FMAC4 soap_in_ns1__Settings(struct soap*, const char*, ns1__Settings *, const char*);

SOAP_FMAC5 ns1__Settings * SOAP_FMAC6 soap_new_ns1__Settings(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Settings(struct soap*, ns1__Settings*);
SOAP_FMAC5 ns1__Settings * SOAP_FMAC6 soap_instantiate_ns1__Settings(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Settings(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__SendOptions
#define SOAP_TYPE_ns1__SendOptions (117)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__SendOptions(struct soap*, const char*, int, const ns1__SendOptions *, const char*);
SOAP_FMAC3 ns1__SendOptions * SOAP_FMAC4 soap_get_ns1__SendOptions(struct soap*, ns1__SendOptions *, const char*, const char*);
SOAP_FMAC3 ns1__SendOptions * SOAP_FMAC4 soap_in_ns1__SendOptions(struct soap*, const char*, ns1__SendOptions *, const char*);

SOAP_FMAC5 ns1__SendOptions * SOAP_FMAC6 soap_new_ns1__SendOptions(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__SendOptions(struct soap*, ns1__SendOptions*);
SOAP_FMAC5 ns1__SendOptions * SOAP_FMAC6 soap_instantiate_ns1__SendOptions(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__SendOptions(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__SendOptionsRequestReply
#define SOAP_TYPE_ns1__SendOptionsRequestReply (116)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__SendOptionsRequestReply(struct soap*, const char*, int, const ns1__SendOptionsRequestReply *, const char*);
SOAP_FMAC3 ns1__SendOptionsRequestReply * SOAP_FMAC4 soap_get_ns1__SendOptionsRequestReply(struct soap*, ns1__SendOptionsRequestReply *, const char*, const char*);
SOAP_FMAC3 ns1__SendOptionsRequestReply * SOAP_FMAC4 soap_in_ns1__SendOptionsRequestReply(struct soap*, const char*, ns1__SendOptionsRequestReply *, const char*);

SOAP_FMAC5 ns1__SendOptionsRequestReply * SOAP_FMAC6 soap_new_ns1__SendOptionsRequestReply(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__SendOptionsRequestReply(struct soap*, ns1__SendOptionsRequestReply*);
SOAP_FMAC5 ns1__SendOptionsRequestReply * SOAP_FMAC6 soap_instantiate_ns1__SendOptionsRequestReply(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__SendOptionsRequestReply(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Rights
#define SOAP_TYPE_ns1__Rights (115)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Rights(struct soap*, const char*, int, const ns1__Rights *, const char*);
SOAP_FMAC3 ns1__Rights * SOAP_FMAC4 soap_get_ns1__Rights(struct soap*, ns1__Rights *, const char*, const char*);
SOAP_FMAC3 ns1__Rights * SOAP_FMAC4 soap_in_ns1__Rights(struct soap*, const char*, ns1__Rights *, const char*);

SOAP_FMAC5 ns1__Rights * SOAP_FMAC6 soap_new_ns1__Rights(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Rights(struct soap*, ns1__Rights*);
SOAP_FMAC5 ns1__Rights * SOAP_FMAC6 soap_instantiate_ns1__Rights(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Rights(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ReturnNotificationOptions
#define SOAP_TYPE_ns1__ReturnNotificationOptions (114)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ReturnNotificationOptions(struct soap*, const char*, int, const ns1__ReturnNotificationOptions *, const char*);
SOAP_FMAC3 ns1__ReturnNotificationOptions * SOAP_FMAC4 soap_get_ns1__ReturnNotificationOptions(struct soap*, ns1__ReturnNotificationOptions *, const char*, const char*);
SOAP_FMAC3 ns1__ReturnNotificationOptions * SOAP_FMAC4 soap_in_ns1__ReturnNotificationOptions(struct soap*, const char*, ns1__ReturnNotificationOptions *, const char*);

SOAP_FMAC5 ns1__ReturnNotificationOptions * SOAP_FMAC6 soap_new_ns1__ReturnNotificationOptions(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ReturnNotificationOptions(struct soap*, ns1__ReturnNotificationOptions*);
SOAP_FMAC5 ns1__ReturnNotificationOptions * SOAP_FMAC6 soap_instantiate_ns1__ReturnNotificationOptions(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ReturnNotificationOptions(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ReturnNotification
#define SOAP_TYPE_ns1__ReturnNotification (113)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ReturnNotification(struct soap*, const char*, int, const ns1__ReturnNotification *, const char*);
SOAP_FMAC3 ns1__ReturnNotification * SOAP_FMAC4 soap_get_ns1__ReturnNotification(struct soap*, ns1__ReturnNotification *, const char*, const char*);
SOAP_FMAC3 ns1__ReturnNotification * SOAP_FMAC4 soap_in_ns1__ReturnNotification(struct soap*, const char*, ns1__ReturnNotification *, const char*);

SOAP_FMAC5 ns1__ReturnNotification * SOAP_FMAC6 soap_new_ns1__ReturnNotification(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ReturnNotification(struct soap*, ns1__ReturnNotification*);
SOAP_FMAC5 ns1__ReturnNotification * SOAP_FMAC6 soap_instantiate_ns1__ReturnNotification(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ReturnNotification(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Resource
#define SOAP_TYPE_ns1__Resource (112)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Resource(struct soap*, const char*, int, const ns1__Resource *, const char*);
SOAP_FMAC3 ns1__Resource * SOAP_FMAC4 soap_get_ns1__Resource(struct soap*, ns1__Resource *, const char*, const char*);
SOAP_FMAC3 ns1__Resource * SOAP_FMAC4 soap_in_ns1__Resource(struct soap*, const char*, ns1__Resource *, const char*);

SOAP_FMAC5 ns1__Resource * SOAP_FMAC6 soap_new_ns1__Resource(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Resource(struct soap*, ns1__Resource*);
SOAP_FMAC5 ns1__Resource * SOAP_FMAC6 soap_instantiate_ns1__Resource(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Resource(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__RecurrenceDateType
#define SOAP_TYPE_ns1__RecurrenceDateType (111)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__RecurrenceDateType(struct soap*, const char*, int, const ns1__RecurrenceDateType *, const char*);
SOAP_FMAC3 ns1__RecurrenceDateType * SOAP_FMAC4 soap_get_ns1__RecurrenceDateType(struct soap*, ns1__RecurrenceDateType *, const char*, const char*);
SOAP_FMAC3 ns1__RecurrenceDateType * SOAP_FMAC4 soap_in_ns1__RecurrenceDateType(struct soap*, const char*, ns1__RecurrenceDateType *, const char*);

SOAP_FMAC5 ns1__RecurrenceDateType * SOAP_FMAC6 soap_new_ns1__RecurrenceDateType(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__RecurrenceDateType(struct soap*, ns1__RecurrenceDateType*);
SOAP_FMAC5 ns1__RecurrenceDateType * SOAP_FMAC6 soap_instantiate_ns1__RecurrenceDateType(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__RecurrenceDateType(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__RecipientStatus
#define SOAP_TYPE_ns1__RecipientStatus (110)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__RecipientStatus(struct soap*, const char*, int, const ns1__RecipientStatus *, const char*);
SOAP_FMAC3 ns1__RecipientStatus * SOAP_FMAC4 soap_get_ns1__RecipientStatus(struct soap*, ns1__RecipientStatus *, const char*, const char*);
SOAP_FMAC3 ns1__RecipientStatus * SOAP_FMAC4 soap_in_ns1__RecipientStatus(struct soap*, const char*, ns1__RecipientStatus *, const char*);

SOAP_FMAC5 ns1__RecipientStatus * SOAP_FMAC6 soap_new_ns1__RecipientStatus(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__RecipientStatus(struct soap*, ns1__RecipientStatus*);
SOAP_FMAC5 ns1__RecipientStatus * SOAP_FMAC6 soap_instantiate_ns1__RecipientStatus(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__RecipientStatus(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__RecipientList
#define SOAP_TYPE_ns1__RecipientList (109)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__RecipientList(struct soap*, const char*, int, const ns1__RecipientList *, const char*);
SOAP_FMAC3 ns1__RecipientList * SOAP_FMAC4 soap_get_ns1__RecipientList(struct soap*, ns1__RecipientList *, const char*, const char*);
SOAP_FMAC3 ns1__RecipientList * SOAP_FMAC4 soap_in_ns1__RecipientList(struct soap*, const char*, ns1__RecipientList *, const char*);

SOAP_FMAC5 ns1__RecipientList * SOAP_FMAC6 soap_new_ns1__RecipientList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__RecipientList(struct soap*, ns1__RecipientList*);
SOAP_FMAC5 ns1__RecipientList * SOAP_FMAC6 soap_instantiate_ns1__RecipientList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__RecipientList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Recipient
#define SOAP_TYPE_ns1__Recipient (108)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Recipient(struct soap*, const char*, int, const ns1__Recipient *, const char*);
SOAP_FMAC3 ns1__Recipient * SOAP_FMAC4 soap_get_ns1__Recipient(struct soap*, ns1__Recipient *, const char*, const char*);
SOAP_FMAC3 ns1__Recipient * SOAP_FMAC4 soap_in_ns1__Recipient(struct soap*, const char*, ns1__Recipient *, const char*);

SOAP_FMAC5 ns1__Recipient * SOAP_FMAC6 soap_new_ns1__Recipient(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Recipient(struct soap*, ns1__Recipient*);
SOAP_FMAC5 ns1__Recipient * SOAP_FMAC6 soap_instantiate_ns1__Recipient(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Recipient(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__QueryTarget
#define SOAP_TYPE_ns1__QueryTarget (107)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__QueryTarget(struct soap*, const char*, int, const ns1__QueryTarget *, const char*);
SOAP_FMAC3 ns1__QueryTarget * SOAP_FMAC4 soap_get_ns1__QueryTarget(struct soap*, ns1__QueryTarget *, const char*, const char*);
SOAP_FMAC3 ns1__QueryTarget * SOAP_FMAC4 soap_in_ns1__QueryTarget(struct soap*, const char*, ns1__QueryTarget *, const char*);

SOAP_FMAC5 ns1__QueryTarget * SOAP_FMAC6 soap_new_ns1__QueryTarget(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__QueryTarget(struct soap*, ns1__QueryTarget*);
SOAP_FMAC5 ns1__QueryTarget * SOAP_FMAC6 soap_instantiate_ns1__QueryTarget(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__QueryTarget(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__QueryFolder
#define SOAP_TYPE_ns1__QueryFolder (106)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__QueryFolder(struct soap*, const char*, int, const ns1__QueryFolder *, const char*);
SOAP_FMAC3 ns1__QueryFolder * SOAP_FMAC4 soap_get_ns1__QueryFolder(struct soap*, ns1__QueryFolder *, const char*, const char*);
SOAP_FMAC3 ns1__QueryFolder * SOAP_FMAC4 soap_in_ns1__QueryFolder(struct soap*, const char*, ns1__QueryFolder *, const char*);

SOAP_FMAC5 ns1__QueryFolder * SOAP_FMAC6 soap_new_ns1__QueryFolder(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__QueryFolder(struct soap*, ns1__QueryFolder*);
SOAP_FMAC5 ns1__QueryFolder * SOAP_FMAC6 soap_instantiate_ns1__QueryFolder(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__QueryFolder(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Query
#define SOAP_TYPE_ns1__Query (105)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Query(struct soap*, const char*, int, const ns1__Query *, const char*);
SOAP_FMAC3 ns1__Query * SOAP_FMAC4 soap_get_ns1__Query(struct soap*, ns1__Query *, const char*, const char*);
SOAP_FMAC3 ns1__Query * SOAP_FMAC4 soap_in_ns1__Query(struct soap*, const char*, ns1__Query *, const char*);

SOAP_FMAC5 ns1__Query * SOAP_FMAC6 soap_new_ns1__Query(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Query(struct soap*, ns1__Query*);
SOAP_FMAC5 ns1__Query * SOAP_FMAC6 soap_instantiate_ns1__Query(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Query(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ProxyList
#define SOAP_TYPE_ns1__ProxyList (104)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ProxyList(struct soap*, const char*, int, const ns1__ProxyList *, const char*);
SOAP_FMAC3 ns1__ProxyList * SOAP_FMAC4 soap_get_ns1__ProxyList(struct soap*, ns1__ProxyList *, const char*, const char*);
SOAP_FMAC3 ns1__ProxyList * SOAP_FMAC4 soap_in_ns1__ProxyList(struct soap*, const char*, ns1__ProxyList *, const char*);

SOAP_FMAC5 ns1__ProxyList * SOAP_FMAC6 soap_new_ns1__ProxyList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ProxyList(struct soap*, ns1__ProxyList*);
SOAP_FMAC5 ns1__ProxyList * SOAP_FMAC6 soap_instantiate_ns1__ProxyList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ProxyList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Proxy
#define SOAP_TYPE_ns1__Proxy (103)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Proxy(struct soap*, const char*, int, const ns1__Proxy *, const char*);
SOAP_FMAC3 ns1__Proxy * SOAP_FMAC4 soap_get_ns1__Proxy(struct soap*, ns1__Proxy *, const char*, const char*);
SOAP_FMAC3 ns1__Proxy * SOAP_FMAC4 soap_in_ns1__Proxy(struct soap*, const char*, ns1__Proxy *, const char*);

SOAP_FMAC5 ns1__Proxy * SOAP_FMAC6 soap_new_ns1__Proxy(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Proxy(struct soap*, ns1__Proxy*);
SOAP_FMAC5 ns1__Proxy * SOAP_FMAC6 soap_instantiate_ns1__Proxy(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Proxy(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ProblemList
#define SOAP_TYPE_ns1__ProblemList (102)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ProblemList(struct soap*, const char*, int, const ns1__ProblemList *, const char*);
SOAP_FMAC3 ns1__ProblemList * SOAP_FMAC4 soap_get_ns1__ProblemList(struct soap*, ns1__ProblemList *, const char*, const char*);
SOAP_FMAC3 ns1__ProblemList * SOAP_FMAC4 soap_in_ns1__ProblemList(struct soap*, const char*, ns1__ProblemList *, const char*);

SOAP_FMAC5 ns1__ProblemList * SOAP_FMAC6 soap_new_ns1__ProblemList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ProblemList(struct soap*, ns1__ProblemList*);
SOAP_FMAC5 ns1__ProblemList * SOAP_FMAC6 soap_instantiate_ns1__ProblemList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ProblemList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ProblemEntry
#define SOAP_TYPE_ns1__ProblemEntry (101)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ProblemEntry(struct soap*, const char*, int, const ns1__ProblemEntry *, const char*);
SOAP_FMAC3 ns1__ProblemEntry * SOAP_FMAC4 soap_get_ns1__ProblemEntry(struct soap*, ns1__ProblemEntry *, const char*, const char*);
SOAP_FMAC3 ns1__ProblemEntry * SOAP_FMAC4 soap_in_ns1__ProblemEntry(struct soap*, const char*, ns1__ProblemEntry *, const char*);

SOAP_FMAC5 ns1__ProblemEntry * SOAP_FMAC6 soap_new_ns1__ProblemEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ProblemEntry(struct soap*, ns1__ProblemEntry*);
SOAP_FMAC5 ns1__ProblemEntry * SOAP_FMAC6 soap_instantiate_ns1__ProblemEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ProblemEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PostalAddressList
#define SOAP_TYPE_ns1__PostalAddressList (100)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PostalAddressList(struct soap*, const char*, int, const ns1__PostalAddressList *, const char*);
SOAP_FMAC3 ns1__PostalAddressList * SOAP_FMAC4 soap_get_ns1__PostalAddressList(struct soap*, ns1__PostalAddressList *, const char*, const char*);
SOAP_FMAC3 ns1__PostalAddressList * SOAP_FMAC4 soap_in_ns1__PostalAddressList(struct soap*, const char*, ns1__PostalAddressList *, const char*);

SOAP_FMAC5 ns1__PostalAddressList * SOAP_FMAC6 soap_new_ns1__PostalAddressList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PostalAddressList(struct soap*, ns1__PostalAddressList*);
SOAP_FMAC5 ns1__PostalAddressList * SOAP_FMAC6 soap_instantiate_ns1__PostalAddressList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PostalAddressList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PostalAddress
#define SOAP_TYPE_ns1__PostalAddress (99)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PostalAddress(struct soap*, const char*, int, const ns1__PostalAddress *, const char*);
SOAP_FMAC3 ns1__PostalAddress * SOAP_FMAC4 soap_get_ns1__PostalAddress(struct soap*, ns1__PostalAddress *, const char*, const char*);
SOAP_FMAC3 ns1__PostalAddress * SOAP_FMAC4 soap_in_ns1__PostalAddress(struct soap*, const char*, ns1__PostalAddress *, const char*);

SOAP_FMAC5 ns1__PostalAddress * SOAP_FMAC6 soap_new_ns1__PostalAddress(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PostalAddress(struct soap*, ns1__PostalAddress*);
SOAP_FMAC5 ns1__PostalAddress * SOAP_FMAC6 soap_instantiate_ns1__PostalAddress(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PostalAddress(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PlainText
#define SOAP_TYPE_ns1__PlainText (98)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PlainText(struct soap*, const char*, int, const ns1__PlainText *, const char*);
SOAP_FMAC3 ns1__PlainText * SOAP_FMAC4 soap_get_ns1__PlainText(struct soap*, ns1__PlainText *, const char*, const char*);
SOAP_FMAC3 ns1__PlainText * SOAP_FMAC4 soap_in_ns1__PlainText(struct soap*, const char*, ns1__PlainText *, const char*);

SOAP_FMAC5 ns1__PlainText * SOAP_FMAC6 soap_new_ns1__PlainText(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PlainText(struct soap*, ns1__PlainText*);
SOAP_FMAC5 ns1__PlainText * SOAP_FMAC6 soap_instantiate_ns1__PlainText(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PlainText(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PhoneNumber
#define SOAP_TYPE_ns1__PhoneNumber (97)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PhoneNumber(struct soap*, const char*, int, const ns1__PhoneNumber *, const char*);
SOAP_FMAC3 ns1__PhoneNumber * SOAP_FMAC4 soap_get_ns1__PhoneNumber(struct soap*, ns1__PhoneNumber *, const char*, const char*);
SOAP_FMAC3 ns1__PhoneNumber * SOAP_FMAC4 soap_in_ns1__PhoneNumber(struct soap*, const char*, ns1__PhoneNumber *, const char*);

SOAP_FMAC5 ns1__PhoneNumber * SOAP_FMAC6 soap_new_ns1__PhoneNumber(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PhoneNumber(struct soap*, ns1__PhoneNumber*);
SOAP_FMAC5 ns1__PhoneNumber * SOAP_FMAC6 soap_instantiate_ns1__PhoneNumber(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PhoneNumber(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PhoneMessage
#define SOAP_TYPE_ns1__PhoneMessage (96)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PhoneMessage(struct soap*, const char*, int, const ns1__PhoneMessage *, const char*);
SOAP_FMAC3 ns1__PhoneMessage * SOAP_FMAC4 soap_get_ns1__PhoneMessage(struct soap*, ns1__PhoneMessage *, const char*, const char*);
SOAP_FMAC3 ns1__PhoneMessage * SOAP_FMAC4 soap_in_ns1__PhoneMessage(struct soap*, const char*, ns1__PhoneMessage *, const char*);

SOAP_FMAC5 ns1__PhoneMessage * SOAP_FMAC6 soap_new_ns1__PhoneMessage(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PhoneMessage(struct soap*, ns1__PhoneMessage*);
SOAP_FMAC5 ns1__PhoneMessage * SOAP_FMAC6 soap_instantiate_ns1__PhoneMessage(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PhoneMessage(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PhoneList
#define SOAP_TYPE_ns1__PhoneList (95)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PhoneList(struct soap*, const char*, int, const ns1__PhoneList *, const char*);
SOAP_FMAC3 ns1__PhoneList * SOAP_FMAC4 soap_get_ns1__PhoneList(struct soap*, ns1__PhoneList *, const char*, const char*);
SOAP_FMAC3 ns1__PhoneList * SOAP_FMAC4 soap_in_ns1__PhoneList(struct soap*, const char*, ns1__PhoneList *, const char*);

SOAP_FMAC5 ns1__PhoneList * SOAP_FMAC6 soap_new_ns1__PhoneList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PhoneList(struct soap*, ns1__PhoneList*);
SOAP_FMAC5 ns1__PhoneList * SOAP_FMAC6 soap_instantiate_ns1__PhoneList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PhoneList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PhoneFlags
#define SOAP_TYPE_ns1__PhoneFlags (94)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PhoneFlags(struct soap*, const char*, int, const ns1__PhoneFlags *, const char*);
SOAP_FMAC3 ns1__PhoneFlags * SOAP_FMAC4 soap_get_ns1__PhoneFlags(struct soap*, ns1__PhoneFlags *, const char*, const char*);
SOAP_FMAC3 ns1__PhoneFlags * SOAP_FMAC4 soap_in_ns1__PhoneFlags(struct soap*, const char*, ns1__PhoneFlags *, const char*);

SOAP_FMAC5 ns1__PhoneFlags * SOAP_FMAC6 soap_new_ns1__PhoneFlags(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PhoneFlags(struct soap*, ns1__PhoneFlags*);
SOAP_FMAC5 ns1__PhoneFlags * SOAP_FMAC6 soap_instantiate_ns1__PhoneFlags(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PhoneFlags(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__PersonalInfo
#define SOAP_TYPE_ns1__PersonalInfo (93)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__PersonalInfo(struct soap*, const char*, int, const ns1__PersonalInfo *, const char*);
SOAP_FMAC3 ns1__PersonalInfo * SOAP_FMAC4 soap_get_ns1__PersonalInfo(struct soap*, ns1__PersonalInfo *, const char*, const char*);
SOAP_FMAC3 ns1__PersonalInfo * SOAP_FMAC4 soap_in_ns1__PersonalInfo(struct soap*, const char*, ns1__PersonalInfo *, const char*);

SOAP_FMAC5 ns1__PersonalInfo * SOAP_FMAC6 soap_new_ns1__PersonalInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__PersonalInfo(struct soap*, ns1__PersonalInfo*);
SOAP_FMAC5 ns1__PersonalInfo * SOAP_FMAC6 soap_instantiate_ns1__PersonalInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__PersonalInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Organization
#define SOAP_TYPE_ns1__Organization (92)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Organization(struct soap*, const char*, int, const ns1__Organization *, const char*);
SOAP_FMAC3 ns1__Organization * SOAP_FMAC4 soap_get_ns1__Organization(struct soap*, ns1__Organization *, const char*, const char*);
SOAP_FMAC3 ns1__Organization * SOAP_FMAC4 soap_in_ns1__Organization(struct soap*, const char*, ns1__Organization *, const char*);

SOAP_FMAC5 ns1__Organization * SOAP_FMAC6 soap_new_ns1__Organization(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Organization(struct soap*, ns1__Organization*);
SOAP_FMAC5 ns1__Organization * SOAP_FMAC6 soap_instantiate_ns1__Organization(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Organization(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__OfficeInfo
#define SOAP_TYPE_ns1__OfficeInfo (91)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__OfficeInfo(struct soap*, const char*, int, const ns1__OfficeInfo *, const char*);
SOAP_FMAC3 ns1__OfficeInfo * SOAP_FMAC4 soap_get_ns1__OfficeInfo(struct soap*, ns1__OfficeInfo *, const char*, const char*);
SOAP_FMAC3 ns1__OfficeInfo * SOAP_FMAC4 soap_in_ns1__OfficeInfo(struct soap*, const char*, ns1__OfficeInfo *, const char*);

SOAP_FMAC5 ns1__OfficeInfo * SOAP_FMAC6 soap_new_ns1__OfficeInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__OfficeInfo(struct soap*, ns1__OfficeInfo*);
SOAP_FMAC5 ns1__OfficeInfo * SOAP_FMAC6 soap_instantiate_ns1__OfficeInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__OfficeInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Note
#define SOAP_TYPE_ns1__Note (90)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Note(struct soap*, const char*, int, const ns1__Note *, const char*);
SOAP_FMAC3 ns1__Note * SOAP_FMAC4 soap_get_ns1__Note(struct soap*, ns1__Note *, const char*, const char*);
SOAP_FMAC3 ns1__Note * SOAP_FMAC4 soap_in_ns1__Note(struct soap*, const char*, ns1__Note *, const char*);

SOAP_FMAC5 ns1__Note * SOAP_FMAC6 soap_new_ns1__Note(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Note(struct soap*, ns1__Note*);
SOAP_FMAC5 ns1__Note * SOAP_FMAC6 soap_instantiate_ns1__Note(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Note(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__NameAndEmail
#define SOAP_TYPE_ns1__NameAndEmail (89)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__NameAndEmail(struct soap*, const char*, int, const ns1__NameAndEmail *, const char*);
SOAP_FMAC3 ns1__NameAndEmail * SOAP_FMAC4 soap_get_ns1__NameAndEmail(struct soap*, ns1__NameAndEmail *, const char*, const char*);
SOAP_FMAC3 ns1__NameAndEmail * SOAP_FMAC4 soap_in_ns1__NameAndEmail(struct soap*, const char*, ns1__NameAndEmail *, const char*);

SOAP_FMAC5 ns1__NameAndEmail * SOAP_FMAC6 soap_new_ns1__NameAndEmail(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__NameAndEmail(struct soap*, ns1__NameAndEmail*);
SOAP_FMAC5 ns1__NameAndEmail * SOAP_FMAC6 soap_instantiate_ns1__NameAndEmail(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__NameAndEmail(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__MessagePart
#define SOAP_TYPE_ns1__MessagePart (88)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__MessagePart(struct soap*, const char*, int, const ns1__MessagePart *, const char*);
SOAP_FMAC3 ns1__MessagePart * SOAP_FMAC4 soap_get_ns1__MessagePart(struct soap*, ns1__MessagePart *, const char*, const char*);
SOAP_FMAC3 ns1__MessagePart * SOAP_FMAC4 soap_in_ns1__MessagePart(struct soap*, const char*, ns1__MessagePart *, const char*);

SOAP_FMAC5 ns1__MessagePart * SOAP_FMAC6 soap_new_ns1__MessagePart(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__MessagePart(struct soap*, ns1__MessagePart*);
SOAP_FMAC5 ns1__MessagePart * SOAP_FMAC6 soap_instantiate_ns1__MessagePart(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__MessagePart(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__MessageBody
#define SOAP_TYPE_ns1__MessageBody (87)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__MessageBody(struct soap*, const char*, int, const ns1__MessageBody *, const char*);
SOAP_FMAC3 ns1__MessageBody * SOAP_FMAC4 soap_get_ns1__MessageBody(struct soap*, ns1__MessageBody *, const char*, const char*);
SOAP_FMAC3 ns1__MessageBody * SOAP_FMAC4 soap_in_ns1__MessageBody(struct soap*, const char*, ns1__MessageBody *, const char*);

SOAP_FMAC5 ns1__MessageBody * SOAP_FMAC6 soap_new_ns1__MessageBody(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__MessageBody(struct soap*, ns1__MessageBody*);
SOAP_FMAC5 ns1__MessageBody * SOAP_FMAC6 soap_instantiate_ns1__MessageBody(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__MessageBody(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Mail
#define SOAP_TYPE_ns1__Mail (86)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Mail(struct soap*, const char*, int, const ns1__Mail *, const char*);
SOAP_FMAC3 ns1__Mail * SOAP_FMAC4 soap_get_ns1__Mail(struct soap*, ns1__Mail *, const char*, const char*);
SOAP_FMAC3 ns1__Mail * SOAP_FMAC4 soap_in_ns1__Mail(struct soap*, const char*, ns1__Mail *, const char*);

SOAP_FMAC5 ns1__Mail * SOAP_FMAC6 soap_new_ns1__Mail(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Mail(struct soap*, ns1__Mail*);
SOAP_FMAC5 ns1__Mail * SOAP_FMAC6 soap_instantiate_ns1__Mail(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Mail(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ItemStatus
#define SOAP_TYPE_ns1__ItemStatus (85)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemStatus(struct soap*, const char*, int, const ns1__ItemStatus *, const char*);
SOAP_FMAC3 ns1__ItemStatus * SOAP_FMAC4 soap_get_ns1__ItemStatus(struct soap*, ns1__ItemStatus *, const char*, const char*);
SOAP_FMAC3 ns1__ItemStatus * SOAP_FMAC4 soap_in_ns1__ItemStatus(struct soap*, const char*, ns1__ItemStatus *, const char*);

SOAP_FMAC5 ns1__ItemStatus * SOAP_FMAC6 soap_new_ns1__ItemStatus(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ItemStatus(struct soap*, ns1__ItemStatus*);
SOAP_FMAC5 ns1__ItemStatus * SOAP_FMAC6 soap_instantiate_ns1__ItemStatus(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ItemStatus(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Items
#define SOAP_TYPE_ns1__Items (84)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Items(struct soap*, const char*, int, const ns1__Items *, const char*);
SOAP_FMAC3 ns1__Items * SOAP_FMAC4 soap_get_ns1__Items(struct soap*, ns1__Items *, const char*, const char*);
SOAP_FMAC3 ns1__Items * SOAP_FMAC4 soap_in_ns1__Items(struct soap*, const char*, ns1__Items *, const char*);

SOAP_FMAC5 ns1__Items * SOAP_FMAC6 soap_new_ns1__Items(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Items(struct soap*, ns1__Items*);
SOAP_FMAC5 ns1__Items * SOAP_FMAC6 soap_instantiate_ns1__Items(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Items(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ItemRefList
#define SOAP_TYPE_ns1__ItemRefList (83)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemRefList(struct soap*, const char*, int, const ns1__ItemRefList *, const char*);
SOAP_FMAC3 ns1__ItemRefList * SOAP_FMAC4 soap_get_ns1__ItemRefList(struct soap*, ns1__ItemRefList *, const char*, const char*);
SOAP_FMAC3 ns1__ItemRefList * SOAP_FMAC4 soap_in_ns1__ItemRefList(struct soap*, const char*, ns1__ItemRefList *, const char*);

SOAP_FMAC5 ns1__ItemRefList * SOAP_FMAC6 soap_new_ns1__ItemRefList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ItemRefList(struct soap*, ns1__ItemRefList*);
SOAP_FMAC5 ns1__ItemRefList * SOAP_FMAC6 soap_instantiate_ns1__ItemRefList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ItemRefList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ItemRef
#define SOAP_TYPE_ns1__ItemRef (82)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemRef(struct soap*, const char*, int, const ns1__ItemRef *, const char*);
SOAP_FMAC3 ns1__ItemRef * SOAP_FMAC4 soap_get_ns1__ItemRef(struct soap*, ns1__ItemRef *, const char*, const char*);
SOAP_FMAC3 ns1__ItemRef * SOAP_FMAC4 soap_in_ns1__ItemRef(struct soap*, const char*, ns1__ItemRef *, const char*);

SOAP_FMAC5 ns1__ItemRef * SOAP_FMAC6 soap_new_ns1__ItemRef(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ItemRef(struct soap*, ns1__ItemRef*);
SOAP_FMAC5 ns1__ItemRef * SOAP_FMAC6 soap_instantiate_ns1__ItemRef(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ItemRef(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ItemOptions
#define SOAP_TYPE_ns1__ItemOptions (81)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemOptions(struct soap*, const char*, int, const ns1__ItemOptions *, const char*);
SOAP_FMAC3 ns1__ItemOptions * SOAP_FMAC4 soap_get_ns1__ItemOptions(struct soap*, ns1__ItemOptions *, const char*, const char*);
SOAP_FMAC3 ns1__ItemOptions * SOAP_FMAC4 soap_in_ns1__ItemOptions(struct soap*, const char*, ns1__ItemOptions *, const char*);

SOAP_FMAC5 ns1__ItemOptions * SOAP_FMAC6 soap_new_ns1__ItemOptions(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ItemOptions(struct soap*, ns1__ItemOptions*);
SOAP_FMAC5 ns1__ItemOptions * SOAP_FMAC6 soap_instantiate_ns1__ItemOptions(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ItemOptions(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ItemList
#define SOAP_TYPE_ns1__ItemList (80)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemList(struct soap*, const char*, int, const ns1__ItemList *, const char*);
SOAP_FMAC3 ns1__ItemList * SOAP_FMAC4 soap_get_ns1__ItemList(struct soap*, ns1__ItemList *, const char*, const char*);
SOAP_FMAC3 ns1__ItemList * SOAP_FMAC4 soap_in_ns1__ItemList(struct soap*, const char*, ns1__ItemList *, const char*);

SOAP_FMAC5 ns1__ItemList * SOAP_FMAC6 soap_new_ns1__ItemList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ItemList(struct soap*, ns1__ItemList*);
SOAP_FMAC5 ns1__ItemList * SOAP_FMAC6 soap_instantiate_ns1__ItemList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ItemList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ItemChanges
#define SOAP_TYPE_ns1__ItemChanges (79)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ItemChanges(struct soap*, const char*, int, const ns1__ItemChanges *, const char*);
SOAP_FMAC3 ns1__ItemChanges * SOAP_FMAC4 soap_get_ns1__ItemChanges(struct soap*, ns1__ItemChanges *, const char*, const char*);
SOAP_FMAC3 ns1__ItemChanges * SOAP_FMAC4 soap_in_ns1__ItemChanges(struct soap*, const char*, ns1__ItemChanges *, const char*);

SOAP_FMAC5 ns1__ItemChanges * SOAP_FMAC6 soap_new_ns1__ItemChanges(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ItemChanges(struct soap*, ns1__ItemChanges*);
SOAP_FMAC5 ns1__ItemChanges * SOAP_FMAC6 soap_instantiate_ns1__ItemChanges(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ItemChanges(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Item
#define SOAP_TYPE_ns1__Item (78)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Item(struct soap*, const char*, int, const ns1__Item *, const char*);
SOAP_FMAC3 ns1__Item * SOAP_FMAC4 soap_get_ns1__Item(struct soap*, ns1__Item *, const char*, const char*);
SOAP_FMAC3 ns1__Item * SOAP_FMAC4 soap_in_ns1__Item(struct soap*, const char*, ns1__Item *, const char*);

SOAP_FMAC5 ns1__Item * SOAP_FMAC6 soap_new_ns1__Item(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Item(struct soap*, ns1__Item*);
SOAP_FMAC5 ns1__Item * SOAP_FMAC6 soap_instantiate_ns1__Item(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Item(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ImAddressList
#define SOAP_TYPE_ns1__ImAddressList (77)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ImAddressList(struct soap*, const char*, int, const ns1__ImAddressList *, const char*);
SOAP_FMAC3 ns1__ImAddressList * SOAP_FMAC4 soap_get_ns1__ImAddressList(struct soap*, ns1__ImAddressList *, const char*, const char*);
SOAP_FMAC3 ns1__ImAddressList * SOAP_FMAC4 soap_in_ns1__ImAddressList(struct soap*, const char*, ns1__ImAddressList *, const char*);

SOAP_FMAC5 ns1__ImAddressList * SOAP_FMAC6 soap_new_ns1__ImAddressList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ImAddressList(struct soap*, ns1__ImAddressList*);
SOAP_FMAC5 ns1__ImAddressList * SOAP_FMAC6 soap_instantiate_ns1__ImAddressList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ImAddressList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ImAddress
#define SOAP_TYPE_ns1__ImAddress (76)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ImAddress(struct soap*, const char*, int, const ns1__ImAddress *, const char*);
SOAP_FMAC3 ns1__ImAddress * SOAP_FMAC4 soap_get_ns1__ImAddress(struct soap*, ns1__ImAddress *, const char*, const char*);
SOAP_FMAC3 ns1__ImAddress * SOAP_FMAC4 soap_in_ns1__ImAddress(struct soap*, const char*, ns1__ImAddress *, const char*);

SOAP_FMAC5 ns1__ImAddress * SOAP_FMAC6 soap_new_ns1__ImAddress(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ImAddress(struct soap*, ns1__ImAddress*);
SOAP_FMAC5 ns1__ImAddress * SOAP_FMAC6 soap_instantiate_ns1__ImAddress(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ImAddress(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__GroupMemberList
#define SOAP_TYPE_ns1__GroupMemberList (75)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__GroupMemberList(struct soap*, const char*, int, const ns1__GroupMemberList *, const char*);
SOAP_FMAC3 ns1__GroupMemberList * SOAP_FMAC4 soap_get_ns1__GroupMemberList(struct soap*, ns1__GroupMemberList *, const char*, const char*);
SOAP_FMAC3 ns1__GroupMemberList * SOAP_FMAC4 soap_in_ns1__GroupMemberList(struct soap*, const char*, ns1__GroupMemberList *, const char*);

SOAP_FMAC5 ns1__GroupMemberList * SOAP_FMAC6 soap_new_ns1__GroupMemberList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__GroupMemberList(struct soap*, ns1__GroupMemberList*);
SOAP_FMAC5 ns1__GroupMemberList * SOAP_FMAC6 soap_instantiate_ns1__GroupMemberList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__GroupMemberList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__GroupMember
#define SOAP_TYPE_ns1__GroupMember (74)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__GroupMember(struct soap*, const char*, int, const ns1__GroupMember *, const char*);
SOAP_FMAC3 ns1__GroupMember * SOAP_FMAC4 soap_get_ns1__GroupMember(struct soap*, ns1__GroupMember *, const char*, const char*);
SOAP_FMAC3 ns1__GroupMember * SOAP_FMAC4 soap_in_ns1__GroupMember(struct soap*, const char*, ns1__GroupMember *, const char*);

SOAP_FMAC5 ns1__GroupMember * SOAP_FMAC6 soap_new_ns1__GroupMember(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__GroupMember(struct soap*, ns1__GroupMember*);
SOAP_FMAC5 ns1__GroupMember * SOAP_FMAC6 soap_instantiate_ns1__GroupMember(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__GroupMember(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Group
#define SOAP_TYPE_ns1__Group (73)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Group(struct soap*, const char*, int, const ns1__Group *, const char*);
SOAP_FMAC3 ns1__Group * SOAP_FMAC4 soap_get_ns1__Group(struct soap*, ns1__Group *, const char*, const char*);
SOAP_FMAC3 ns1__Group * SOAP_FMAC4 soap_in_ns1__Group(struct soap*, const char*, ns1__Group *, const char*);

SOAP_FMAC5 ns1__Group * SOAP_FMAC6 soap_new_ns1__Group(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Group(struct soap*, ns1__Group*);
SOAP_FMAC5 ns1__Group * SOAP_FMAC6 soap_instantiate_ns1__Group(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Group(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FullName
#define SOAP_TYPE_ns1__FullName (72)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FullName(struct soap*, const char*, int, const ns1__FullName *, const char*);
SOAP_FMAC3 ns1__FullName * SOAP_FMAC4 soap_get_ns1__FullName(struct soap*, ns1__FullName *, const char*, const char*);
SOAP_FMAC3 ns1__FullName * SOAP_FMAC4 soap_in_ns1__FullName(struct soap*, const char*, ns1__FullName *, const char*);

SOAP_FMAC5 ns1__FullName * SOAP_FMAC6 soap_new_ns1__FullName(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FullName(struct soap*, ns1__FullName*);
SOAP_FMAC5 ns1__FullName * SOAP_FMAC6 soap_instantiate_ns1__FullName(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FullName(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__From
#define SOAP_TYPE_ns1__From (71)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__From(struct soap*, const char*, int, const ns1__From *, const char*);
SOAP_FMAC3 ns1__From * SOAP_FMAC4 soap_get_ns1__From(struct soap*, ns1__From *, const char*, const char*);
SOAP_FMAC3 ns1__From * SOAP_FMAC4 soap_in_ns1__From(struct soap*, const char*, ns1__From *, const char*);

SOAP_FMAC5 ns1__From * SOAP_FMAC6 soap_new_ns1__From(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__From(struct soap*, ns1__From*);
SOAP_FMAC5 ns1__From * SOAP_FMAC6 soap_instantiate_ns1__From(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__From(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FreeBusyUser
#define SOAP_TYPE_ns1__FreeBusyUser (70)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FreeBusyUser(struct soap*, const char*, int, const ns1__FreeBusyUser *, const char*);
SOAP_FMAC3 ns1__FreeBusyUser * SOAP_FMAC4 soap_get_ns1__FreeBusyUser(struct soap*, ns1__FreeBusyUser *, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyUser * SOAP_FMAC4 soap_in_ns1__FreeBusyUser(struct soap*, const char*, ns1__FreeBusyUser *, const char*);

SOAP_FMAC5 ns1__FreeBusyUser * SOAP_FMAC6 soap_new_ns1__FreeBusyUser(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FreeBusyUser(struct soap*, ns1__FreeBusyUser*);
SOAP_FMAC5 ns1__FreeBusyUser * SOAP_FMAC6 soap_instantiate_ns1__FreeBusyUser(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FreeBusyUser(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FreeBusyStats
#define SOAP_TYPE_ns1__FreeBusyStats (69)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FreeBusyStats(struct soap*, const char*, int, const ns1__FreeBusyStats *, const char*);
SOAP_FMAC3 ns1__FreeBusyStats * SOAP_FMAC4 soap_get_ns1__FreeBusyStats(struct soap*, ns1__FreeBusyStats *, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyStats * SOAP_FMAC4 soap_in_ns1__FreeBusyStats(struct soap*, const char*, ns1__FreeBusyStats *, const char*);

SOAP_FMAC5 ns1__FreeBusyStats * SOAP_FMAC6 soap_new_ns1__FreeBusyStats(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FreeBusyStats(struct soap*, ns1__FreeBusyStats*);
SOAP_FMAC5 ns1__FreeBusyStats * SOAP_FMAC6 soap_instantiate_ns1__FreeBusyStats(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FreeBusyStats(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FreeBusyInfoList
#define SOAP_TYPE_ns1__FreeBusyInfoList (68)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FreeBusyInfoList(struct soap*, const char*, int, const ns1__FreeBusyInfoList *, const char*);
SOAP_FMAC3 ns1__FreeBusyInfoList * SOAP_FMAC4 soap_get_ns1__FreeBusyInfoList(struct soap*, ns1__FreeBusyInfoList *, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyInfoList * SOAP_FMAC4 soap_in_ns1__FreeBusyInfoList(struct soap*, const char*, ns1__FreeBusyInfoList *, const char*);

SOAP_FMAC5 ns1__FreeBusyInfoList * SOAP_FMAC6 soap_new_ns1__FreeBusyInfoList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FreeBusyInfoList(struct soap*, ns1__FreeBusyInfoList*);
SOAP_FMAC5 ns1__FreeBusyInfoList * SOAP_FMAC6 soap_instantiate_ns1__FreeBusyInfoList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FreeBusyInfoList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FreeBusyUserList
#define SOAP_TYPE_ns1__FreeBusyUserList (67)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FreeBusyUserList(struct soap*, const char*, int, const ns1__FreeBusyUserList *, const char*);
SOAP_FMAC3 ns1__FreeBusyUserList * SOAP_FMAC4 soap_get_ns1__FreeBusyUserList(struct soap*, ns1__FreeBusyUserList *, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyUserList * SOAP_FMAC4 soap_in_ns1__FreeBusyUserList(struct soap*, const char*, ns1__FreeBusyUserList *, const char*);

SOAP_FMAC5 ns1__FreeBusyUserList * SOAP_FMAC6 soap_new_ns1__FreeBusyUserList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FreeBusyUserList(struct soap*, ns1__FreeBusyUserList*);
SOAP_FMAC5 ns1__FreeBusyUserList * SOAP_FMAC6 soap_instantiate_ns1__FreeBusyUserList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FreeBusyUserList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FreeBusyInfo
#define SOAP_TYPE_ns1__FreeBusyInfo (66)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FreeBusyInfo(struct soap*, const char*, int, const ns1__FreeBusyInfo *, const char*);
SOAP_FMAC3 ns1__FreeBusyInfo * SOAP_FMAC4 soap_get_ns1__FreeBusyInfo(struct soap*, ns1__FreeBusyInfo *, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyInfo * SOAP_FMAC4 soap_in_ns1__FreeBusyInfo(struct soap*, const char*, ns1__FreeBusyInfo *, const char*);

SOAP_FMAC5 ns1__FreeBusyInfo * SOAP_FMAC6 soap_new_ns1__FreeBusyInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FreeBusyInfo(struct soap*, ns1__FreeBusyInfo*);
SOAP_FMAC5 ns1__FreeBusyInfo * SOAP_FMAC6 soap_instantiate_ns1__FreeBusyInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FreeBusyInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FreeBusyBlock
#define SOAP_TYPE_ns1__FreeBusyBlock (65)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FreeBusyBlock(struct soap*, const char*, int, const ns1__FreeBusyBlock *, const char*);
SOAP_FMAC3 ns1__FreeBusyBlock * SOAP_FMAC4 soap_get_ns1__FreeBusyBlock(struct soap*, ns1__FreeBusyBlock *, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyBlock * SOAP_FMAC4 soap_in_ns1__FreeBusyBlock(struct soap*, const char*, ns1__FreeBusyBlock *, const char*);

SOAP_FMAC5 ns1__FreeBusyBlock * SOAP_FMAC6 soap_new_ns1__FreeBusyBlock(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FreeBusyBlock(struct soap*, ns1__FreeBusyBlock*);
SOAP_FMAC5 ns1__FreeBusyBlock * SOAP_FMAC6 soap_instantiate_ns1__FreeBusyBlock(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FreeBusyBlock(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FreeBusyBlockList
#define SOAP_TYPE_ns1__FreeBusyBlockList (64)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FreeBusyBlockList(struct soap*, const char*, int, const ns1__FreeBusyBlockList *, const char*);
SOAP_FMAC3 ns1__FreeBusyBlockList * SOAP_FMAC4 soap_get_ns1__FreeBusyBlockList(struct soap*, ns1__FreeBusyBlockList *, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyBlockList * SOAP_FMAC4 soap_in_ns1__FreeBusyBlockList(struct soap*, const char*, ns1__FreeBusyBlockList *, const char*);

SOAP_FMAC5 ns1__FreeBusyBlockList * SOAP_FMAC6 soap_new_ns1__FreeBusyBlockList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FreeBusyBlockList(struct soap*, ns1__FreeBusyBlockList*);
SOAP_FMAC5 ns1__FreeBusyBlockList * SOAP_FMAC6 soap_instantiate_ns1__FreeBusyBlockList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FreeBusyBlockList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FolderList
#define SOAP_TYPE_ns1__FolderList (63)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FolderList(struct soap*, const char*, int, const ns1__FolderList *, const char*);
SOAP_FMAC3 ns1__FolderList * SOAP_FMAC4 soap_get_ns1__FolderList(struct soap*, ns1__FolderList *, const char*, const char*);
SOAP_FMAC3 ns1__FolderList * SOAP_FMAC4 soap_in_ns1__FolderList(struct soap*, const char*, ns1__FolderList *, const char*);

SOAP_FMAC5 ns1__FolderList * SOAP_FMAC6 soap_new_ns1__FolderList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FolderList(struct soap*, ns1__FolderList*);
SOAP_FMAC5 ns1__FolderList * SOAP_FMAC6 soap_instantiate_ns1__FolderList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FolderList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FolderACLEntry
#define SOAP_TYPE_ns1__FolderACLEntry (62)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FolderACLEntry(struct soap*, const char*, int, const ns1__FolderACLEntry *, const char*);
SOAP_FMAC3 ns1__FolderACLEntry * SOAP_FMAC4 soap_get_ns1__FolderACLEntry(struct soap*, ns1__FolderACLEntry *, const char*, const char*);
SOAP_FMAC3 ns1__FolderACLEntry * SOAP_FMAC4 soap_in_ns1__FolderACLEntry(struct soap*, const char*, ns1__FolderACLEntry *, const char*);

SOAP_FMAC5 ns1__FolderACLEntry * SOAP_FMAC6 soap_new_ns1__FolderACLEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FolderACLEntry(struct soap*, ns1__FolderACLEntry*);
SOAP_FMAC5 ns1__FolderACLEntry * SOAP_FMAC6 soap_instantiate_ns1__FolderACLEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FolderACLEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FolderACL
#define SOAP_TYPE_ns1__FolderACL (61)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FolderACL(struct soap*, const char*, int, const ns1__FolderACL *, const char*);
SOAP_FMAC3 ns1__FolderACL * SOAP_FMAC4 soap_get_ns1__FolderACL(struct soap*, ns1__FolderACL *, const char*, const char*);
SOAP_FMAC3 ns1__FolderACL * SOAP_FMAC4 soap_in_ns1__FolderACL(struct soap*, const char*, ns1__FolderACL *, const char*);

SOAP_FMAC5 ns1__FolderACL * SOAP_FMAC6 soap_new_ns1__FolderACL(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FolderACL(struct soap*, ns1__FolderACL*);
SOAP_FMAC5 ns1__FolderACL * SOAP_FMAC6 soap_instantiate_ns1__FolderACL(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FolderACL(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Folder
#define SOAP_TYPE_ns1__Folder (60)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Folder(struct soap*, const char*, int, const ns1__Folder *, const char*);
SOAP_FMAC3 ns1__Folder * SOAP_FMAC4 soap_get_ns1__Folder(struct soap*, ns1__Folder *, const char*, const char*);
SOAP_FMAC3 ns1__Folder * SOAP_FMAC4 soap_in_ns1__Folder(struct soap*, const char*, ns1__Folder *, const char*);

SOAP_FMAC5 ns1__Folder * SOAP_FMAC6 soap_new_ns1__Folder(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Folder(struct soap*, ns1__Folder*);
SOAP_FMAC5 ns1__Folder * SOAP_FMAC6 soap_instantiate_ns1__Folder(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Folder(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FilterGroup
#define SOAP_TYPE_ns1__FilterGroup (59)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FilterGroup(struct soap*, const char*, int, const ns1__FilterGroup *, const char*);
SOAP_FMAC3 ns1__FilterGroup * SOAP_FMAC4 soap_get_ns1__FilterGroup(struct soap*, ns1__FilterGroup *, const char*, const char*);
SOAP_FMAC3 ns1__FilterGroup * SOAP_FMAC4 soap_in_ns1__FilterGroup(struct soap*, const char*, ns1__FilterGroup *, const char*);

SOAP_FMAC5 ns1__FilterGroup * SOAP_FMAC6 soap_new_ns1__FilterGroup(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FilterGroup(struct soap*, ns1__FilterGroup*);
SOAP_FMAC5 ns1__FilterGroup * SOAP_FMAC6 soap_instantiate_ns1__FilterGroup(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FilterGroup(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FilterEntry
#define SOAP_TYPE_ns1__FilterEntry (58)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FilterEntry(struct soap*, const char*, int, const ns1__FilterEntry *, const char*);
SOAP_FMAC3 ns1__FilterEntry * SOAP_FMAC4 soap_get_ns1__FilterEntry(struct soap*, ns1__FilterEntry *, const char*, const char*);
SOAP_FMAC3 ns1__FilterEntry * SOAP_FMAC4 soap_in_ns1__FilterEntry(struct soap*, const char*, ns1__FilterEntry *, const char*);

SOAP_FMAC5 ns1__FilterEntry * SOAP_FMAC6 soap_new_ns1__FilterEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FilterEntry(struct soap*, ns1__FilterEntry*);
SOAP_FMAC5 ns1__FilterEntry * SOAP_FMAC6 soap_instantiate_ns1__FilterEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FilterEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__FilterElement
#define SOAP_TYPE_ns1__FilterElement (57)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__FilterElement(struct soap*, const char*, int, const ns1__FilterElement *, const char*);
SOAP_FMAC3 ns1__FilterElement * SOAP_FMAC4 soap_get_ns1__FilterElement(struct soap*, ns1__FilterElement *, const char*, const char*);
SOAP_FMAC3 ns1__FilterElement * SOAP_FMAC4 soap_in_ns1__FilterElement(struct soap*, const char*, ns1__FilterElement *, const char*);

SOAP_FMAC5 ns1__FilterElement * SOAP_FMAC6 soap_new_ns1__FilterElement(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__FilterElement(struct soap*, ns1__FilterElement*);
SOAP_FMAC5 ns1__FilterElement * SOAP_FMAC6 soap_instantiate_ns1__FilterElement(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__FilterElement(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Filter
#define SOAP_TYPE_ns1__Filter (56)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Filter(struct soap*, const char*, int, const ns1__Filter *, const char*);
SOAP_FMAC3 ns1__Filter * SOAP_FMAC4 soap_get_ns1__Filter(struct soap*, ns1__Filter *, const char*, const char*);
SOAP_FMAC3 ns1__Filter * SOAP_FMAC4 soap_in_ns1__Filter(struct soap*, const char*, ns1__Filter *, const char*);

SOAP_FMAC5 ns1__Filter * SOAP_FMAC6 soap_new_ns1__Filter(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Filter(struct soap*, ns1__Filter*);
SOAP_FMAC5 ns1__Filter * SOAP_FMAC6 soap_instantiate_ns1__Filter(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Filter(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__EmailAddressList
#define SOAP_TYPE_ns1__EmailAddressList (55)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__EmailAddressList(struct soap*, const char*, int, const ns1__EmailAddressList *, const char*);
SOAP_FMAC3 ns1__EmailAddressList * SOAP_FMAC4 soap_get_ns1__EmailAddressList(struct soap*, ns1__EmailAddressList *, const char*, const char*);
SOAP_FMAC3 ns1__EmailAddressList * SOAP_FMAC4 soap_in_ns1__EmailAddressList(struct soap*, const char*, ns1__EmailAddressList *, const char*);

SOAP_FMAC5 ns1__EmailAddressList * SOAP_FMAC6 soap_new_ns1__EmailAddressList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__EmailAddressList(struct soap*, ns1__EmailAddressList*);
SOAP_FMAC5 ns1__EmailAddressList * SOAP_FMAC6 soap_instantiate_ns1__EmailAddressList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__EmailAddressList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__DocumentRef
#define SOAP_TYPE_ns1__DocumentRef (54)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__DocumentRef(struct soap*, const char*, int, const ns1__DocumentRef *, const char*);
SOAP_FMAC3 ns1__DocumentRef * SOAP_FMAC4 soap_get_ns1__DocumentRef(struct soap*, ns1__DocumentRef *, const char*, const char*);
SOAP_FMAC3 ns1__DocumentRef * SOAP_FMAC4 soap_in_ns1__DocumentRef(struct soap*, const char*, ns1__DocumentRef *, const char*);

SOAP_FMAC5 ns1__DocumentRef * SOAP_FMAC6 soap_new_ns1__DocumentRef(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__DocumentRef(struct soap*, ns1__DocumentRef*);
SOAP_FMAC5 ns1__DocumentRef * SOAP_FMAC6 soap_instantiate_ns1__DocumentRef(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__DocumentRef(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Distribution
#define SOAP_TYPE_ns1__Distribution (53)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Distribution(struct soap*, const char*, int, const ns1__Distribution *, const char*);
SOAP_FMAC3 ns1__Distribution * SOAP_FMAC4 soap_get_ns1__Distribution(struct soap*, ns1__Distribution *, const char*, const char*);
SOAP_FMAC3 ns1__Distribution * SOAP_FMAC4 soap_in_ns1__Distribution(struct soap*, const char*, ns1__Distribution *, const char*);

SOAP_FMAC5 ns1__Distribution * SOAP_FMAC6 soap_new_ns1__Distribution(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Distribution(struct soap*, ns1__Distribution*);
SOAP_FMAC5 ns1__Distribution * SOAP_FMAC6 soap_instantiate_ns1__Distribution(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Distribution(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__DeltaList
#define SOAP_TYPE_ns1__DeltaList (52)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__DeltaList(struct soap*, const char*, int, const ns1__DeltaList *, const char*);
SOAP_FMAC3 ns1__DeltaList * SOAP_FMAC4 soap_get_ns1__DeltaList(struct soap*, ns1__DeltaList *, const char*, const char*);
SOAP_FMAC3 ns1__DeltaList * SOAP_FMAC4 soap_in_ns1__DeltaList(struct soap*, const char*, ns1__DeltaList *, const char*);

SOAP_FMAC5 ns1__DeltaList * SOAP_FMAC6 soap_new_ns1__DeltaList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__DeltaList(struct soap*, ns1__DeltaList*);
SOAP_FMAC5 ns1__DeltaList * SOAP_FMAC6 soap_instantiate_ns1__DeltaList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__DeltaList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__DelegateeStatus
#define SOAP_TYPE_ns1__DelegateeStatus (51)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__DelegateeStatus(struct soap*, const char*, int, const ns1__DelegateeStatus *, const char*);
SOAP_FMAC3 ns1__DelegateeStatus * SOAP_FMAC4 soap_get_ns1__DelegateeStatus(struct soap*, ns1__DelegateeStatus *, const char*, const char*);
SOAP_FMAC3 ns1__DelegateeStatus * SOAP_FMAC4 soap_in_ns1__DelegateeStatus(struct soap*, const char*, ns1__DelegateeStatus *, const char*);

SOAP_FMAC5 ns1__DelegateeStatus * SOAP_FMAC6 soap_new_ns1__DelegateeStatus(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__DelegateeStatus(struct soap*, ns1__DelegateeStatus*);
SOAP_FMAC5 ns1__DelegateeStatus * SOAP_FMAC6 soap_instantiate_ns1__DelegateeStatus(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__DelegateeStatus(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__DelegatedStatus
#define SOAP_TYPE_ns1__DelegatedStatus (50)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__DelegatedStatus(struct soap*, const char*, int, const ns1__DelegatedStatus *, const char*);
SOAP_FMAC3 ns1__DelegatedStatus * SOAP_FMAC4 soap_get_ns1__DelegatedStatus(struct soap*, ns1__DelegatedStatus *, const char*, const char*);
SOAP_FMAC3 ns1__DelegatedStatus * SOAP_FMAC4 soap_in_ns1__DelegatedStatus(struct soap*, const char*, ns1__DelegatedStatus *, const char*);

SOAP_FMAC5 ns1__DelegatedStatus * SOAP_FMAC6 soap_new_ns1__DelegatedStatus(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__DelegatedStatus(struct soap*, ns1__DelegatedStatus*);
SOAP_FMAC5 ns1__DelegatedStatus * SOAP_FMAC6 soap_instantiate_ns1__DelegatedStatus(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__DelegatedStatus(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__DayOfWeek
#define SOAP_TYPE_ns1__DayOfWeek (49)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__DayOfWeek(struct soap*, const char*, int, const ns1__DayOfWeek *, const char*);
SOAP_FMAC3 ns1__DayOfWeek * SOAP_FMAC4 soap_get_ns1__DayOfWeek(struct soap*, ns1__DayOfWeek *, const char*, const char*);
SOAP_FMAC3 ns1__DayOfWeek * SOAP_FMAC4 soap_in_ns1__DayOfWeek(struct soap*, const char*, ns1__DayOfWeek *, const char*);

SOAP_FMAC5 ns1__DayOfWeek * SOAP_FMAC6 soap_new_ns1__DayOfWeek(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__DayOfWeek(struct soap*, ns1__DayOfWeek*);
SOAP_FMAC5 ns1__DayOfWeek * SOAP_FMAC6 soap_instantiate_ns1__DayOfWeek(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__DayOfWeek(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__CustomList
#define SOAP_TYPE_ns1__CustomList (48)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__CustomList(struct soap*, const char*, int, const ns1__CustomList *, const char*);
SOAP_FMAC3 ns1__CustomList * SOAP_FMAC4 soap_get_ns1__CustomList(struct soap*, ns1__CustomList *, const char*, const char*);
SOAP_FMAC3 ns1__CustomList * SOAP_FMAC4 soap_in_ns1__CustomList(struct soap*, const char*, ns1__CustomList *, const char*);

SOAP_FMAC5 ns1__CustomList * SOAP_FMAC6 soap_new_ns1__CustomList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__CustomList(struct soap*, ns1__CustomList*);
SOAP_FMAC5 ns1__CustomList * SOAP_FMAC6 soap_instantiate_ns1__CustomList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__CustomList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Custom
#define SOAP_TYPE_ns1__Custom (47)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Custom(struct soap*, const char*, int, const ns1__Custom *, const char*);
SOAP_FMAC3 ns1__Custom * SOAP_FMAC4 soap_get_ns1__Custom(struct soap*, ns1__Custom *, const char*, const char*);
SOAP_FMAC3 ns1__Custom * SOAP_FMAC4 soap_in_ns1__Custom(struct soap*, const char*, ns1__Custom *, const char*);

SOAP_FMAC5 ns1__Custom * SOAP_FMAC6 soap_new_ns1__Custom(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Custom(struct soap*, ns1__Custom*);
SOAP_FMAC5 ns1__Custom * SOAP_FMAC6 soap_instantiate_ns1__Custom(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Custom(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ContactFolder
#define SOAP_TYPE_ns1__ContactFolder (46)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ContactFolder(struct soap*, const char*, int, const ns1__ContactFolder *, const char*);
SOAP_FMAC3 ns1__ContactFolder * SOAP_FMAC4 soap_get_ns1__ContactFolder(struct soap*, ns1__ContactFolder *, const char*, const char*);
SOAP_FMAC3 ns1__ContactFolder * SOAP_FMAC4 soap_in_ns1__ContactFolder(struct soap*, const char*, ns1__ContactFolder *, const char*);

SOAP_FMAC5 ns1__ContactFolder * SOAP_FMAC6 soap_new_ns1__ContactFolder(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ContactFolder(struct soap*, ns1__ContactFolder*);
SOAP_FMAC5 ns1__ContactFolder * SOAP_FMAC6 soap_instantiate_ns1__ContactFolder(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ContactFolder(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ContainerRef
#define SOAP_TYPE_ns1__ContainerRef (45)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ContainerRef(struct soap*, const char*, int, const ns1__ContainerRef *, const char*);
SOAP_FMAC3 ns1__ContainerRef * SOAP_FMAC4 soap_get_ns1__ContainerRef(struct soap*, ns1__ContainerRef *, const char*, const char*);
SOAP_FMAC3 ns1__ContainerRef * SOAP_FMAC4 soap_in_ns1__ContainerRef(struct soap*, const char*, ns1__ContainerRef *, const char*);

SOAP_FMAC5 ns1__ContainerRef * SOAP_FMAC6 soap_new_ns1__ContainerRef(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ContainerRef(struct soap*, ns1__ContainerRef*);
SOAP_FMAC5 ns1__ContainerRef * SOAP_FMAC6 soap_instantiate_ns1__ContainerRef(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ContainerRef(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__ContainerItem
#define SOAP_TYPE_ns1__ContainerItem (44)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__ContainerItem(struct soap*, const char*, int, const ns1__ContainerItem *, const char*);
SOAP_FMAC3 ns1__ContainerItem * SOAP_FMAC4 soap_get_ns1__ContainerItem(struct soap*, ns1__ContainerItem *, const char*, const char*);
SOAP_FMAC3 ns1__ContainerItem * SOAP_FMAC4 soap_in_ns1__ContainerItem(struct soap*, const char*, ns1__ContainerItem *, const char*);

SOAP_FMAC5 ns1__ContainerItem * SOAP_FMAC6 soap_new_ns1__ContainerItem(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__ContainerItem(struct soap*, ns1__ContainerItem*);
SOAP_FMAC5 ns1__ContainerItem * SOAP_FMAC6 soap_instantiate_ns1__ContainerItem(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__ContainerItem(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Contact
#define SOAP_TYPE_ns1__Contact (43)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Contact(struct soap*, const char*, int, const ns1__Contact *, const char*);
SOAP_FMAC3 ns1__Contact * SOAP_FMAC4 soap_get_ns1__Contact(struct soap*, ns1__Contact *, const char*, const char*);
SOAP_FMAC3 ns1__Contact * SOAP_FMAC4 soap_in_ns1__Contact(struct soap*, const char*, ns1__Contact *, const char*);

SOAP_FMAC5 ns1__Contact * SOAP_FMAC6 soap_new_ns1__Contact(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Contact(struct soap*, ns1__Contact*);
SOAP_FMAC5 ns1__Contact * SOAP_FMAC6 soap_instantiate_ns1__Contact(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Contact(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__CommentStatus
#define SOAP_TYPE_ns1__CommentStatus (42)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__CommentStatus(struct soap*, const char*, int, const ns1__CommentStatus *, const char*);
SOAP_FMAC3 ns1__CommentStatus * SOAP_FMAC4 soap_get_ns1__CommentStatus(struct soap*, ns1__CommentStatus *, const char*, const char*);
SOAP_FMAC3 ns1__CommentStatus * SOAP_FMAC4 soap_in_ns1__CommentStatus(struct soap*, const char*, ns1__CommentStatus *, const char*);

SOAP_FMAC5 ns1__CommentStatus * SOAP_FMAC6 soap_new_ns1__CommentStatus(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__CommentStatus(struct soap*, ns1__CommentStatus*);
SOAP_FMAC5 ns1__CommentStatus * SOAP_FMAC6 soap_instantiate_ns1__CommentStatus(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__CommentStatus(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__CategoryRefList
#define SOAP_TYPE_ns1__CategoryRefList (41)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__CategoryRefList(struct soap*, const char*, int, const ns1__CategoryRefList *, const char*);
SOAP_FMAC3 ns1__CategoryRefList * SOAP_FMAC4 soap_get_ns1__CategoryRefList(struct soap*, ns1__CategoryRefList *, const char*, const char*);
SOAP_FMAC3 ns1__CategoryRefList * SOAP_FMAC4 soap_in_ns1__CategoryRefList(struct soap*, const char*, ns1__CategoryRefList *, const char*);

SOAP_FMAC5 ns1__CategoryRefList * SOAP_FMAC6 soap_new_ns1__CategoryRefList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__CategoryRefList(struct soap*, ns1__CategoryRefList*);
SOAP_FMAC5 ns1__CategoryRefList * SOAP_FMAC6 soap_instantiate_ns1__CategoryRefList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__CategoryRefList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__CategoryList
#define SOAP_TYPE_ns1__CategoryList (40)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__CategoryList(struct soap*, const char*, int, const ns1__CategoryList *, const char*);
SOAP_FMAC3 ns1__CategoryList * SOAP_FMAC4 soap_get_ns1__CategoryList(struct soap*, ns1__CategoryList *, const char*, const char*);
SOAP_FMAC3 ns1__CategoryList * SOAP_FMAC4 soap_in_ns1__CategoryList(struct soap*, const char*, ns1__CategoryList *, const char*);

SOAP_FMAC5 ns1__CategoryList * SOAP_FMAC6 soap_new_ns1__CategoryList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__CategoryList(struct soap*, ns1__CategoryList*);
SOAP_FMAC5 ns1__CategoryList * SOAP_FMAC6 soap_instantiate_ns1__CategoryList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__CategoryList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Category
#define SOAP_TYPE_ns1__Category (39)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Category(struct soap*, const char*, int, const ns1__Category *, const char*);
SOAP_FMAC3 ns1__Category * SOAP_FMAC4 soap_get_ns1__Category(struct soap*, ns1__Category *, const char*, const char*);
SOAP_FMAC3 ns1__Category * SOAP_FMAC4 soap_in_ns1__Category(struct soap*, const char*, ns1__Category *, const char*);

SOAP_FMAC5 ns1__Category * SOAP_FMAC6 soap_new_ns1__Category(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Category(struct soap*, ns1__Category*);
SOAP_FMAC5 ns1__Category * SOAP_FMAC6 soap_instantiate_ns1__Category(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Category(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__CalendarItem
#define SOAP_TYPE_ns1__CalendarItem (38)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__CalendarItem(struct soap*, const char*, int, const ns1__CalendarItem *, const char*);
SOAP_FMAC3 ns1__CalendarItem * SOAP_FMAC4 soap_get_ns1__CalendarItem(struct soap*, ns1__CalendarItem *, const char*, const char*);
SOAP_FMAC3 ns1__CalendarItem * SOAP_FMAC4 soap_in_ns1__CalendarItem(struct soap*, const char*, ns1__CalendarItem *, const char*);

SOAP_FMAC5 ns1__CalendarItem * SOAP_FMAC6 soap_new_ns1__CalendarItem(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__CalendarItem(struct soap*, ns1__CalendarItem*);
SOAP_FMAC5 ns1__CalendarItem * SOAP_FMAC6 soap_instantiate_ns1__CalendarItem(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__CalendarItem(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__BoxEntry
#define SOAP_TYPE_ns1__BoxEntry (37)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__BoxEntry(struct soap*, const char*, int, const ns1__BoxEntry *, const char*);
SOAP_FMAC3 ns1__BoxEntry * SOAP_FMAC4 soap_get_ns1__BoxEntry(struct soap*, ns1__BoxEntry *, const char*, const char*);
SOAP_FMAC3 ns1__BoxEntry * SOAP_FMAC4 soap_in_ns1__BoxEntry(struct soap*, const char*, ns1__BoxEntry *, const char*);

SOAP_FMAC5 ns1__BoxEntry * SOAP_FMAC6 soap_new_ns1__BoxEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__BoxEntry(struct soap*, ns1__BoxEntry*);
SOAP_FMAC5 ns1__BoxEntry * SOAP_FMAC6 soap_instantiate_ns1__BoxEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__BoxEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Authentication
#define SOAP_TYPE_ns1__Authentication (36)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Authentication(struct soap*, const char*, int, const ns1__Authentication *, const char*);
SOAP_FMAC3 ns1__Authentication * SOAP_FMAC4 soap_get_ns1__Authentication(struct soap*, ns1__Authentication *, const char*, const char*);
SOAP_FMAC3 ns1__Authentication * SOAP_FMAC4 soap_in_ns1__Authentication(struct soap*, const char*, ns1__Authentication *, const char*);

SOAP_FMAC5 ns1__Authentication * SOAP_FMAC6 soap_new_ns1__Authentication(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Authentication(struct soap*, ns1__Authentication*);
SOAP_FMAC5 ns1__Authentication * SOAP_FMAC6 soap_instantiate_ns1__Authentication(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Authentication(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AttachmentItemInfo
#define SOAP_TYPE_ns1__AttachmentItemInfo (35)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AttachmentItemInfo(struct soap*, const char*, int, const ns1__AttachmentItemInfo *, const char*);
SOAP_FMAC3 ns1__AttachmentItemInfo * SOAP_FMAC4 soap_get_ns1__AttachmentItemInfo(struct soap*, ns1__AttachmentItemInfo *, const char*, const char*);
SOAP_FMAC3 ns1__AttachmentItemInfo * SOAP_FMAC4 soap_in_ns1__AttachmentItemInfo(struct soap*, const char*, ns1__AttachmentItemInfo *, const char*);

SOAP_FMAC5 ns1__AttachmentItemInfo * SOAP_FMAC6 soap_new_ns1__AttachmentItemInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AttachmentItemInfo(struct soap*, ns1__AttachmentItemInfo*);
SOAP_FMAC5 ns1__AttachmentItemInfo * SOAP_FMAC6 soap_instantiate_ns1__AttachmentItemInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AttachmentItemInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AttachmentInfo
#define SOAP_TYPE_ns1__AttachmentInfo (34)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AttachmentInfo(struct soap*, const char*, int, const ns1__AttachmentInfo *, const char*);
SOAP_FMAC3 ns1__AttachmentInfo * SOAP_FMAC4 soap_get_ns1__AttachmentInfo(struct soap*, ns1__AttachmentInfo *, const char*, const char*);
SOAP_FMAC3 ns1__AttachmentInfo * SOAP_FMAC4 soap_in_ns1__AttachmentInfo(struct soap*, const char*, ns1__AttachmentInfo *, const char*);

SOAP_FMAC5 ns1__AttachmentInfo * SOAP_FMAC6 soap_new_ns1__AttachmentInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AttachmentInfo(struct soap*, ns1__AttachmentInfo*);
SOAP_FMAC5 ns1__AttachmentInfo * SOAP_FMAC6 soap_instantiate_ns1__AttachmentInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AttachmentInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AttachmentID
#define SOAP_TYPE_ns1__AttachmentID (33)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AttachmentID(struct soap*, const char*, int, const ns1__AttachmentID *, const char*);
SOAP_FMAC3 ns1__AttachmentID * SOAP_FMAC4 soap_get_ns1__AttachmentID(struct soap*, ns1__AttachmentID *, const char*, const char*);
SOAP_FMAC3 ns1__AttachmentID * SOAP_FMAC4 soap_in_ns1__AttachmentID(struct soap*, const char*, ns1__AttachmentID *, const char*);

SOAP_FMAC5 ns1__AttachmentID * SOAP_FMAC6 soap_new_ns1__AttachmentID(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AttachmentID(struct soap*, ns1__AttachmentID*);
SOAP_FMAC5 ns1__AttachmentID * SOAP_FMAC6 soap_instantiate_ns1__AttachmentID(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AttachmentID(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Appointment
#define SOAP_TYPE_ns1__Appointment (32)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Appointment(struct soap*, const char*, int, const ns1__Appointment *, const char*);
SOAP_FMAC3 ns1__Appointment * SOAP_FMAC4 soap_get_ns1__Appointment(struct soap*, ns1__Appointment *, const char*, const char*);
SOAP_FMAC3 ns1__Appointment * SOAP_FMAC4 soap_in_ns1__Appointment(struct soap*, const char*, ns1__Appointment *, const char*);

SOAP_FMAC5 ns1__Appointment * SOAP_FMAC6 soap_new_ns1__Appointment(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Appointment(struct soap*, ns1__Appointment*);
SOAP_FMAC5 ns1__Appointment * SOAP_FMAC6 soap_instantiate_ns1__Appointment(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Appointment(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__Alarm
#define SOAP_TYPE_ns1__Alarm (31)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__Alarm(struct soap*, const char*, int, const ns1__Alarm *, const char*);
SOAP_FMAC3 ns1__Alarm * SOAP_FMAC4 soap_get_ns1__Alarm(struct soap*, ns1__Alarm *, const char*, const char*);
SOAP_FMAC3 ns1__Alarm * SOAP_FMAC4 soap_in_ns1__Alarm(struct soap*, const char*, ns1__Alarm *, const char*);

SOAP_FMAC5 ns1__Alarm * SOAP_FMAC6 soap_new_ns1__Alarm(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__Alarm(struct soap*, ns1__Alarm*);
SOAP_FMAC5 ns1__Alarm * SOAP_FMAC6 soap_instantiate_ns1__Alarm(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__Alarm(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AddressBookList
#define SOAP_TYPE_ns1__AddressBookList (30)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AddressBookList(struct soap*, const char*, int, const ns1__AddressBookList *, const char*);
SOAP_FMAC3 ns1__AddressBookList * SOAP_FMAC4 soap_get_ns1__AddressBookList(struct soap*, ns1__AddressBookList *, const char*, const char*);
SOAP_FMAC3 ns1__AddressBookList * SOAP_FMAC4 soap_in_ns1__AddressBookList(struct soap*, const char*, ns1__AddressBookList *, const char*);

SOAP_FMAC5 ns1__AddressBookList * SOAP_FMAC6 soap_new_ns1__AddressBookList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AddressBookList(struct soap*, ns1__AddressBookList*);
SOAP_FMAC5 ns1__AddressBookList * SOAP_FMAC6 soap_instantiate_ns1__AddressBookList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AddressBookList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AddressBookItem
#define SOAP_TYPE_ns1__AddressBookItem (29)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AddressBookItem(struct soap*, const char*, int, const ns1__AddressBookItem *, const char*);
SOAP_FMAC3 ns1__AddressBookItem * SOAP_FMAC4 soap_get_ns1__AddressBookItem(struct soap*, ns1__AddressBookItem *, const char*, const char*);
SOAP_FMAC3 ns1__AddressBookItem * SOAP_FMAC4 soap_in_ns1__AddressBookItem(struct soap*, const char*, ns1__AddressBookItem *, const char*);

SOAP_FMAC5 ns1__AddressBookItem * SOAP_FMAC6 soap_new_ns1__AddressBookItem(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AddressBookItem(struct soap*, ns1__AddressBookItem*);
SOAP_FMAC5 ns1__AddressBookItem * SOAP_FMAC6 soap_instantiate_ns1__AddressBookItem(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AddressBookItem(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AddressBook
#define SOAP_TYPE_ns1__AddressBook (28)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AddressBook(struct soap*, const char*, int, const ns1__AddressBook *, const char*);
SOAP_FMAC3 ns1__AddressBook * SOAP_FMAC4 soap_get_ns1__AddressBook(struct soap*, ns1__AddressBook *, const char*, const char*);
SOAP_FMAC3 ns1__AddressBook * SOAP_FMAC4 soap_in_ns1__AddressBook(struct soap*, const char*, ns1__AddressBook *, const char*);

SOAP_FMAC5 ns1__AddressBook * SOAP_FMAC6 soap_new_ns1__AddressBook(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AddressBook(struct soap*, ns1__AddressBook*);
SOAP_FMAC5 ns1__AddressBook * SOAP_FMAC6 soap_instantiate_ns1__AddressBook(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AddressBook(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AccessRightList
#define SOAP_TYPE_ns1__AccessRightList (27)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AccessRightList(struct soap*, const char*, int, const ns1__AccessRightList *, const char*);
SOAP_FMAC3 ns1__AccessRightList * SOAP_FMAC4 soap_get_ns1__AccessRightList(struct soap*, ns1__AccessRightList *, const char*, const char*);
SOAP_FMAC3 ns1__AccessRightList * SOAP_FMAC4 soap_in_ns1__AccessRightList(struct soap*, const char*, ns1__AccessRightList *, const char*);

SOAP_FMAC5 ns1__AccessRightList * SOAP_FMAC6 soap_new_ns1__AccessRightList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AccessRightList(struct soap*, ns1__AccessRightList*);
SOAP_FMAC5 ns1__AccessRightList * SOAP_FMAC6 soap_instantiate_ns1__AccessRightList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AccessRightList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AccessRightEntry
#define SOAP_TYPE_ns1__AccessRightEntry (26)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AccessRightEntry(struct soap*, const char*, int, const ns1__AccessRightEntry *, const char*);
SOAP_FMAC3 ns1__AccessRightEntry * SOAP_FMAC4 soap_get_ns1__AccessRightEntry(struct soap*, ns1__AccessRightEntry *, const char*, const char*);
SOAP_FMAC3 ns1__AccessRightEntry * SOAP_FMAC4 soap_in_ns1__AccessRightEntry(struct soap*, const char*, ns1__AccessRightEntry *, const char*);

SOAP_FMAC5 ns1__AccessRightEntry * SOAP_FMAC6 soap_new_ns1__AccessRightEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AccessRightEntry(struct soap*, ns1__AccessRightEntry*);
SOAP_FMAC5 ns1__AccessRightEntry * SOAP_FMAC6 soap_instantiate_ns1__AccessRightEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AccessRightEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AccessRightChanges
#define SOAP_TYPE_ns1__AccessRightChanges (25)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AccessRightChanges(struct soap*, const char*, int, const ns1__AccessRightChanges *, const char*);
SOAP_FMAC3 ns1__AccessRightChanges * SOAP_FMAC4 soap_get_ns1__AccessRightChanges(struct soap*, ns1__AccessRightChanges *, const char*, const char*);
SOAP_FMAC3 ns1__AccessRightChanges * SOAP_FMAC4 soap_in_ns1__AccessRightChanges(struct soap*, const char*, ns1__AccessRightChanges *, const char*);

SOAP_FMAC5 ns1__AccessRightChanges * SOAP_FMAC6 soap_new_ns1__AccessRightChanges(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AccessRightChanges(struct soap*, ns1__AccessRightChanges*);
SOAP_FMAC5 ns1__AccessRightChanges * SOAP_FMAC6 soap_instantiate_ns1__AccessRightChanges(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AccessRightChanges(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AccessRight
#define SOAP_TYPE_ns1__AccessRight (24)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AccessRight(struct soap*, const char*, int, const ns1__AccessRight *, const char*);
SOAP_FMAC3 ns1__AccessRight * SOAP_FMAC4 soap_get_ns1__AccessRight(struct soap*, ns1__AccessRight *, const char*, const char*);
SOAP_FMAC3 ns1__AccessRight * SOAP_FMAC4 soap_in_ns1__AccessRight(struct soap*, const char*, ns1__AccessRight *, const char*);

SOAP_FMAC5 ns1__AccessRight * SOAP_FMAC6 soap_new_ns1__AccessRight(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AccessRight(struct soap*, ns1__AccessRight*);
SOAP_FMAC5 ns1__AccessRight * SOAP_FMAC6 soap_instantiate_ns1__AccessRight(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AccessRight(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AccessMiscRight
#define SOAP_TYPE_ns1__AccessMiscRight (23)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AccessMiscRight(struct soap*, const char*, int, const ns1__AccessMiscRight *, const char*);
SOAP_FMAC3 ns1__AccessMiscRight * SOAP_FMAC4 soap_get_ns1__AccessMiscRight(struct soap*, ns1__AccessMiscRight *, const char*, const char*);
SOAP_FMAC3 ns1__AccessMiscRight * SOAP_FMAC4 soap_in_ns1__AccessMiscRight(struct soap*, const char*, ns1__AccessMiscRight *, const char*);

SOAP_FMAC5 ns1__AccessMiscRight * SOAP_FMAC6 soap_new_ns1__AccessMiscRight(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AccessMiscRight(struct soap*, ns1__AccessMiscRight*);
SOAP_FMAC5 ns1__AccessMiscRight * SOAP_FMAC6 soap_instantiate_ns1__AccessMiscRight(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AccessMiscRight(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AccessControlList
#define SOAP_TYPE_ns1__AccessControlList (22)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AccessControlList(struct soap*, const char*, int, const ns1__AccessControlList *, const char*);
SOAP_FMAC3 ns1__AccessControlList * SOAP_FMAC4 soap_get_ns1__AccessControlList(struct soap*, ns1__AccessControlList *, const char*, const char*);
SOAP_FMAC3 ns1__AccessControlList * SOAP_FMAC4 soap_in_ns1__AccessControlList(struct soap*, const char*, ns1__AccessControlList *, const char*);

SOAP_FMAC5 ns1__AccessControlList * SOAP_FMAC6 soap_new_ns1__AccessControlList(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AccessControlList(struct soap*, ns1__AccessControlList*);
SOAP_FMAC5 ns1__AccessControlList * SOAP_FMAC6 soap_instantiate_ns1__AccessControlList(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AccessControlList(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_ns1__AccessControlListEntry
#define SOAP_TYPE_ns1__AccessControlListEntry (21)
#endif

SOAP_FMAC3 int SOAP_FMAC4 soap_out_ns1__AccessControlListEntry(struct soap*, const char*, int, const ns1__AccessControlListEntry *, const char*);
SOAP_FMAC3 ns1__AccessControlListEntry * SOAP_FMAC4 soap_get_ns1__AccessControlListEntry(struct soap*, ns1__AccessControlListEntry *, const char*, const char*);
SOAP_FMAC3 ns1__AccessControlListEntry * SOAP_FMAC4 soap_in_ns1__AccessControlListEntry(struct soap*, const char*, ns1__AccessControlListEntry *, const char*);

SOAP_FMAC5 ns1__AccessControlListEntry * SOAP_FMAC6 soap_new_ns1__AccessControlListEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_ns1__AccessControlListEntry(struct soap*, ns1__AccessControlListEntry*);
SOAP_FMAC5 ns1__AccessControlListEntry * SOAP_FMAC6 soap_instantiate_ns1__AccessControlListEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_ns1__AccessControlListEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_xsd__base64Binary
#define SOAP_TYPE_xsd__base64Binary (11)
#endif
SOAP_FMAC3 int SOAP_FMAC4 soap_out_xsd__base64Binary(struct soap*, const char*, int, const xsd__base64Binary *, const char*);
SOAP_FMAC3 xsd__base64Binary * SOAP_FMAC4 soap_get_xsd__base64Binary(struct soap*, xsd__base64Binary *, const char*, const char*);
SOAP_FMAC3 xsd__base64Binary * SOAP_FMAC4 soap_in_xsd__base64Binary(struct soap*, const char*, xsd__base64Binary *, const char*);

SOAP_FMAC5 xsd__base64Binary * SOAP_FMAC6 soap_new_xsd__base64Binary(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_xsd__base64Binary(struct soap*, xsd__base64Binary*);
SOAP_FMAC5 xsd__base64Binary * SOAP_FMAC6 soap_instantiate_xsd__base64Binary(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_xsd__base64Binary(struct soap*, int, int, void*, const void*, size_t);

#ifndef WITH_NOGLOBAL

#ifndef SOAP_TYPE_SOAP_ENV__Fault
#define SOAP_TYPE_SOAP_ENV__Fault (460)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_SOAP_ENV__Fault(struct soap*, struct SOAP_ENV__Fault const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_SOAP_ENV__Fault(struct soap*, const struct SOAP_ENV__Fault *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_SOAP_ENV__Fault(struct soap*, struct SOAP_ENV__Fault *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_SOAP_ENV__Fault(struct soap*, const struct SOAP_ENV__Fault *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_SOAP_ENV__Fault(struct soap*, const char*, int, const struct SOAP_ENV__Fault *, const char*);
SOAP_FMAC3 struct SOAP_ENV__Fault * SOAP_FMAC4 soap_get_SOAP_ENV__Fault(struct soap*, struct SOAP_ENV__Fault *, const char*, const char*);
SOAP_FMAC3 struct SOAP_ENV__Fault * SOAP_FMAC4 soap_in_SOAP_ENV__Fault(struct soap*, const char*, struct SOAP_ENV__Fault *, const char*);

#endif

#ifndef WITH_NOGLOBAL

#ifndef SOAP_TYPE_SOAP_ENV__Detail
#define SOAP_TYPE_SOAP_ENV__Detail (457)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_SOAP_ENV__Detail(struct soap*, struct SOAP_ENV__Detail const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_SOAP_ENV__Detail(struct soap*, const struct SOAP_ENV__Detail *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_SOAP_ENV__Detail(struct soap*, struct SOAP_ENV__Detail *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_SOAP_ENV__Detail(struct soap*, const struct SOAP_ENV__Detail *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_SOAP_ENV__Detail(struct soap*, const char*, int, const struct SOAP_ENV__Detail *, const char*);
SOAP_FMAC3 struct SOAP_ENV__Detail * SOAP_FMAC4 soap_get_SOAP_ENV__Detail(struct soap*, struct SOAP_ENV__Detail *, const char*, const char*);
SOAP_FMAC3 struct SOAP_ENV__Detail * SOAP_FMAC4 soap_in_SOAP_ENV__Detail(struct soap*, const char*, struct SOAP_ENV__Detail *, const char*);

#endif

#ifndef WITH_NOGLOBAL

#ifndef SOAP_TYPE_SOAP_ENV__Code
#define SOAP_TYPE_SOAP_ENV__Code (456)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_SOAP_ENV__Code(struct soap*, struct SOAP_ENV__Code const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_SOAP_ENV__Code(struct soap*, const struct SOAP_ENV__Code *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_SOAP_ENV__Code(struct soap*, struct SOAP_ENV__Code *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_SOAP_ENV__Code(struct soap*, const struct SOAP_ENV__Code *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_SOAP_ENV__Code(struct soap*, const char*, int, const struct SOAP_ENV__Code *, const char*);
SOAP_FMAC3 struct SOAP_ENV__Code * SOAP_FMAC4 soap_get_SOAP_ENV__Code(struct soap*, struct SOAP_ENV__Code *, const char*, const char*);
SOAP_FMAC3 struct SOAP_ENV__Code * SOAP_FMAC4 soap_in_SOAP_ENV__Code(struct soap*, const char*, struct SOAP_ENV__Code *, const char*);

#endif

#ifndef SOAP_TYPE___ns1__getFreeBusyRequest
#define SOAP_TYPE___ns1__getFreeBusyRequest (455)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__getFreeBusyRequest(struct soap*, struct __ns1__getFreeBusyRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__getFreeBusyRequest(struct soap*, const struct __ns1__getFreeBusyRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__getFreeBusyRequest(struct soap*, struct __ns1__getFreeBusyRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__getFreeBusyRequest(struct soap*, const struct __ns1__getFreeBusyRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__getFreeBusyRequest(struct soap*, const char*, int, const struct __ns1__getFreeBusyRequest *, const char*);
SOAP_FMAC3 struct __ns1__getFreeBusyRequest * SOAP_FMAC4 soap_get___ns1__getFreeBusyRequest(struct soap*, struct __ns1__getFreeBusyRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__getFreeBusyRequest * SOAP_FMAC4 soap_in___ns1__getFreeBusyRequest(struct soap*, const char*, struct __ns1__getFreeBusyRequest *, const char*);

#ifndef SOAP_TYPE___ns1__closeFreeBusySessionRequest
#define SOAP_TYPE___ns1__closeFreeBusySessionRequest (451)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__closeFreeBusySessionRequest(struct soap*, struct __ns1__closeFreeBusySessionRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__closeFreeBusySessionRequest(struct soap*, const struct __ns1__closeFreeBusySessionRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__closeFreeBusySessionRequest(struct soap*, struct __ns1__closeFreeBusySessionRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__closeFreeBusySessionRequest(struct soap*, const struct __ns1__closeFreeBusySessionRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__closeFreeBusySessionRequest(struct soap*, const char*, int, const struct __ns1__closeFreeBusySessionRequest *, const char*);
SOAP_FMAC3 struct __ns1__closeFreeBusySessionRequest * SOAP_FMAC4 soap_get___ns1__closeFreeBusySessionRequest(struct soap*, struct __ns1__closeFreeBusySessionRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__closeFreeBusySessionRequest * SOAP_FMAC4 soap_in___ns1__closeFreeBusySessionRequest(struct soap*, const char*, struct __ns1__closeFreeBusySessionRequest *, const char*);

#ifndef SOAP_TYPE___ns1__startFreeBusySessionRequest
#define SOAP_TYPE___ns1__startFreeBusySessionRequest (447)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__startFreeBusySessionRequest(struct soap*, struct __ns1__startFreeBusySessionRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__startFreeBusySessionRequest(struct soap*, const struct __ns1__startFreeBusySessionRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__startFreeBusySessionRequest(struct soap*, struct __ns1__startFreeBusySessionRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__startFreeBusySessionRequest(struct soap*, const struct __ns1__startFreeBusySessionRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__startFreeBusySessionRequest(struct soap*, const char*, int, const struct __ns1__startFreeBusySessionRequest *, const char*);
SOAP_FMAC3 struct __ns1__startFreeBusySessionRequest * SOAP_FMAC4 soap_get___ns1__startFreeBusySessionRequest(struct soap*, struct __ns1__startFreeBusySessionRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__startFreeBusySessionRequest * SOAP_FMAC4 soap_in___ns1__startFreeBusySessionRequest(struct soap*, const char*, struct __ns1__startFreeBusySessionRequest *, const char*);

#ifndef SOAP_TYPE___ns1__removeItemRequest
#define SOAP_TYPE___ns1__removeItemRequest (443)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__removeItemRequest(struct soap*, struct __ns1__removeItemRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__removeItemRequest(struct soap*, const struct __ns1__removeItemRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__removeItemRequest(struct soap*, struct __ns1__removeItemRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__removeItemRequest(struct soap*, const struct __ns1__removeItemRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__removeItemRequest(struct soap*, const char*, int, const struct __ns1__removeItemRequest *, const char*);
SOAP_FMAC3 struct __ns1__removeItemRequest * SOAP_FMAC4 soap_get___ns1__removeItemRequest(struct soap*, struct __ns1__removeItemRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__removeItemRequest * SOAP_FMAC4 soap_in___ns1__removeItemRequest(struct soap*, const char*, struct __ns1__removeItemRequest *, const char*);

#ifndef SOAP_TYPE___ns1__purgeItemRequest
#define SOAP_TYPE___ns1__purgeItemRequest (439)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__purgeItemRequest(struct soap*, struct __ns1__purgeItemRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__purgeItemRequest(struct soap*, const struct __ns1__purgeItemRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__purgeItemRequest(struct soap*, struct __ns1__purgeItemRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__purgeItemRequest(struct soap*, const struct __ns1__purgeItemRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__purgeItemRequest(struct soap*, const char*, int, const struct __ns1__purgeItemRequest *, const char*);
SOAP_FMAC3 struct __ns1__purgeItemRequest * SOAP_FMAC4 soap_get___ns1__purgeItemRequest(struct soap*, struct __ns1__purgeItemRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__purgeItemRequest * SOAP_FMAC4 soap_in___ns1__purgeItemRequest(struct soap*, const char*, struct __ns1__purgeItemRequest *, const char*);

#ifndef SOAP_TYPE___ns1__modifyItemRequest
#define SOAP_TYPE___ns1__modifyItemRequest (435)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__modifyItemRequest(struct soap*, struct __ns1__modifyItemRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__modifyItemRequest(struct soap*, const struct __ns1__modifyItemRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__modifyItemRequest(struct soap*, struct __ns1__modifyItemRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__modifyItemRequest(struct soap*, const struct __ns1__modifyItemRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__modifyItemRequest(struct soap*, const char*, int, const struct __ns1__modifyItemRequest *, const char*);
SOAP_FMAC3 struct __ns1__modifyItemRequest * SOAP_FMAC4 soap_get___ns1__modifyItemRequest(struct soap*, struct __ns1__modifyItemRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__modifyItemRequest * SOAP_FMAC4 soap_in___ns1__modifyItemRequest(struct soap*, const char*, struct __ns1__modifyItemRequest *, const char*);

#ifndef SOAP_TYPE___ns1__sendItemRequest
#define SOAP_TYPE___ns1__sendItemRequest (431)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__sendItemRequest(struct soap*, struct __ns1__sendItemRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__sendItemRequest(struct soap*, const struct __ns1__sendItemRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__sendItemRequest(struct soap*, struct __ns1__sendItemRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__sendItemRequest(struct soap*, const struct __ns1__sendItemRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__sendItemRequest(struct soap*, const char*, int, const struct __ns1__sendItemRequest *, const char*);
SOAP_FMAC3 struct __ns1__sendItemRequest * SOAP_FMAC4 soap_get___ns1__sendItemRequest(struct soap*, struct __ns1__sendItemRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__sendItemRequest * SOAP_FMAC4 soap_in___ns1__sendItemRequest(struct soap*, const char*, struct __ns1__sendItemRequest *, const char*);

#ifndef SOAP_TYPE___ns1__createItemRequest
#define SOAP_TYPE___ns1__createItemRequest (427)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__createItemRequest(struct soap*, struct __ns1__createItemRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__createItemRequest(struct soap*, const struct __ns1__createItemRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__createItemRequest(struct soap*, struct __ns1__createItemRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__createItemRequest(struct soap*, const struct __ns1__createItemRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__createItemRequest(struct soap*, const char*, int, const struct __ns1__createItemRequest *, const char*);
SOAP_FMAC3 struct __ns1__createItemRequest * SOAP_FMAC4 soap_get___ns1__createItemRequest(struct soap*, struct __ns1__createItemRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__createItemRequest * SOAP_FMAC4 soap_in___ns1__createItemRequest(struct soap*, const char*, struct __ns1__createItemRequest *, const char*);

#ifndef SOAP_TYPE___ns1__getDeltaRequest
#define SOAP_TYPE___ns1__getDeltaRequest (423)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__getDeltaRequest(struct soap*, struct __ns1__getDeltaRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__getDeltaRequest(struct soap*, const struct __ns1__getDeltaRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__getDeltaRequest(struct soap*, struct __ns1__getDeltaRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__getDeltaRequest(struct soap*, const struct __ns1__getDeltaRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__getDeltaRequest(struct soap*, const char*, int, const struct __ns1__getDeltaRequest *, const char*);
SOAP_FMAC3 struct __ns1__getDeltaRequest * SOAP_FMAC4 soap_get___ns1__getDeltaRequest(struct soap*, struct __ns1__getDeltaRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__getDeltaRequest * SOAP_FMAC4 soap_in___ns1__getDeltaRequest(struct soap*, const char*, struct __ns1__getDeltaRequest *, const char*);

#ifndef SOAP_TYPE___ns1__getFolderListRequest
#define SOAP_TYPE___ns1__getFolderListRequest (419)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__getFolderListRequest(struct soap*, struct __ns1__getFolderListRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__getFolderListRequest(struct soap*, const struct __ns1__getFolderListRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__getFolderListRequest(struct soap*, struct __ns1__getFolderListRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__getFolderListRequest(struct soap*, const struct __ns1__getFolderListRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__getFolderListRequest(struct soap*, const char*, int, const struct __ns1__getFolderListRequest *, const char*);
SOAP_FMAC3 struct __ns1__getFolderListRequest * SOAP_FMAC4 soap_get___ns1__getFolderListRequest(struct soap*, struct __ns1__getFolderListRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__getFolderListRequest * SOAP_FMAC4 soap_in___ns1__getFolderListRequest(struct soap*, const char*, struct __ns1__getFolderListRequest *, const char*);

#ifndef SOAP_TYPE___ns1__getItemsRequest
#define SOAP_TYPE___ns1__getItemsRequest (415)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__getItemsRequest(struct soap*, struct __ns1__getItemsRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__getItemsRequest(struct soap*, const struct __ns1__getItemsRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__getItemsRequest(struct soap*, struct __ns1__getItemsRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__getItemsRequest(struct soap*, const struct __ns1__getItemsRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__getItemsRequest(struct soap*, const char*, int, const struct __ns1__getItemsRequest *, const char*);
SOAP_FMAC3 struct __ns1__getItemsRequest * SOAP_FMAC4 soap_get___ns1__getItemsRequest(struct soap*, struct __ns1__getItemsRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__getItemsRequest * SOAP_FMAC4 soap_in___ns1__getItemsRequest(struct soap*, const char*, struct __ns1__getItemsRequest *, const char*);

#ifndef SOAP_TYPE___ns1__getItemRequest
#define SOAP_TYPE___ns1__getItemRequest (411)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__getItemRequest(struct soap*, struct __ns1__getItemRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__getItemRequest(struct soap*, const struct __ns1__getItemRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__getItemRequest(struct soap*, struct __ns1__getItemRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__getItemRequest(struct soap*, const struct __ns1__getItemRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__getItemRequest(struct soap*, const char*, int, const struct __ns1__getItemRequest *, const char*);
SOAP_FMAC3 struct __ns1__getItemRequest * SOAP_FMAC4 soap_get___ns1__getItemRequest(struct soap*, struct __ns1__getItemRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__getItemRequest * SOAP_FMAC4 soap_in___ns1__getItemRequest(struct soap*, const char*, struct __ns1__getItemRequest *, const char*);

#ifndef SOAP_TYPE___ns1__getAddressBookListRequest
#define SOAP_TYPE___ns1__getAddressBookListRequest (407)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__getAddressBookListRequest(struct soap*, struct __ns1__getAddressBookListRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__getAddressBookListRequest(struct soap*, const struct __ns1__getAddressBookListRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__getAddressBookListRequest(struct soap*, struct __ns1__getAddressBookListRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__getAddressBookListRequest(struct soap*, const struct __ns1__getAddressBookListRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__getAddressBookListRequest(struct soap*, const char*, int, const struct __ns1__getAddressBookListRequest *, const char*);
SOAP_FMAC3 struct __ns1__getAddressBookListRequest * SOAP_FMAC4 soap_get___ns1__getAddressBookListRequest(struct soap*, struct __ns1__getAddressBookListRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__getAddressBookListRequest * SOAP_FMAC4 soap_in___ns1__getAddressBookListRequest(struct soap*, const char*, struct __ns1__getAddressBookListRequest *, const char*);

SOAP_FMAC5 struct __ns1__getAddressBookListRequest * SOAP_FMAC6 soap_new___ns1__getAddressBookListRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete___ns1__getAddressBookListRequest(struct soap*, struct __ns1__getAddressBookListRequest*);
SOAP_FMAC5 struct __ns1__getAddressBookListRequest * SOAP_FMAC6 soap_instantiate___ns1__getAddressBookListRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy___ns1__getAddressBookListRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE___ns1__getCategoryListRequest
#define SOAP_TYPE___ns1__getCategoryListRequest (404)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__getCategoryListRequest(struct soap*, struct __ns1__getCategoryListRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__getCategoryListRequest(struct soap*, const struct __ns1__getCategoryListRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__getCategoryListRequest(struct soap*, struct __ns1__getCategoryListRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__getCategoryListRequest(struct soap*, const struct __ns1__getCategoryListRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__getCategoryListRequest(struct soap*, const char*, int, const struct __ns1__getCategoryListRequest *, const char*);
SOAP_FMAC3 struct __ns1__getCategoryListRequest * SOAP_FMAC4 soap_get___ns1__getCategoryListRequest(struct soap*, struct __ns1__getCategoryListRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__getCategoryListRequest * SOAP_FMAC4 soap_in___ns1__getCategoryListRequest(struct soap*, const char*, struct __ns1__getCategoryListRequest *, const char*);

SOAP_FMAC5 struct __ns1__getCategoryListRequest * SOAP_FMAC6 soap_new___ns1__getCategoryListRequest(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete___ns1__getCategoryListRequest(struct soap*, struct __ns1__getCategoryListRequest*);
SOAP_FMAC5 struct __ns1__getCategoryListRequest * SOAP_FMAC6 soap_instantiate___ns1__getCategoryListRequest(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy___ns1__getCategoryListRequest(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE___ns1__loginRequest
#define SOAP_TYPE___ns1__loginRequest (401)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize___ns1__loginRequest(struct soap*, struct __ns1__loginRequest const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark___ns1__loginRequest(struct soap*, const struct __ns1__loginRequest *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default___ns1__loginRequest(struct soap*, struct __ns1__loginRequest *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put___ns1__loginRequest(struct soap*, const struct __ns1__loginRequest *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out___ns1__loginRequest(struct soap*, const char*, int, const struct __ns1__loginRequest *, const char*);
SOAP_FMAC3 struct __ns1__loginRequest * SOAP_FMAC4 soap_get___ns1__loginRequest(struct soap*, struct __ns1__loginRequest *, const char*, const char*);
SOAP_FMAC3 struct __ns1__loginRequest * SOAP_FMAC4 soap_in___ns1__loginRequest(struct soap*, const char*, struct __ns1__loginRequest *, const char*);

#ifndef WITH_NOGLOBAL

#ifndef SOAP_TYPE_SOAP_ENV__Header
#define SOAP_TYPE_SOAP_ENV__Header (397)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_SOAP_ENV__Header(struct soap*, struct SOAP_ENV__Header const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_SOAP_ENV__Header(struct soap*, const struct SOAP_ENV__Header *);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_SOAP_ENV__Header(struct soap*, struct SOAP_ENV__Header *);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_SOAP_ENV__Header(struct soap*, const struct SOAP_ENV__Header *, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_SOAP_ENV__Header(struct soap*, const char*, int, const struct SOAP_ENV__Header *, const char*);
SOAP_FMAC3 struct SOAP_ENV__Header * SOAP_FMAC4 soap_get_SOAP_ENV__Header(struct soap*, struct SOAP_ENV__Header *, const char*, const char*);
SOAP_FMAC3 struct SOAP_ENV__Header * SOAP_FMAC4 soap_in_SOAP_ENV__Header(struct soap*, const char*, struct SOAP_ENV__Header *, const char*);

SOAP_FMAC5 struct SOAP_ENV__Header * SOAP_FMAC6 soap_new_SOAP_ENV__Header(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_SOAP_ENV__Header(struct soap*, struct SOAP_ENV__Header*);
SOAP_FMAC5 struct SOAP_ENV__Header * SOAP_FMAC6 soap_instantiate_SOAP_ENV__Header(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_SOAP_ENV__Header(struct soap*, int, int, void*, const void*, size_t);

#endif

#ifndef WITH_NOGLOBAL

#ifndef SOAP_TYPE_PointerToSOAP_ENV__Code
#define SOAP_TYPE_PointerToSOAP_ENV__Code (462)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerToSOAP_ENV__Code(struct soap*, struct SOAP_ENV__Code *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerToSOAP_ENV__Code(struct soap*, struct SOAP_ENV__Code *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerToSOAP_ENV__Code(struct soap*, struct SOAP_ENV__Code *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerToSOAP_ENV__Code(struct soap*, const char *, int, struct SOAP_ENV__Code *const*, const char *);
SOAP_FMAC3 struct SOAP_ENV__Code ** SOAP_FMAC4 soap_get_PointerToSOAP_ENV__Code(struct soap*, struct SOAP_ENV__Code **, const char*, const char*);
SOAP_FMAC3 struct SOAP_ENV__Code ** SOAP_FMAC4 soap_in_PointerToSOAP_ENV__Code(struct soap*, const char*, struct SOAP_ENV__Code **, const char*);

#endif

#ifndef WITH_NOGLOBAL

#ifndef SOAP_TYPE_PointerToSOAP_ENV__Detail
#define SOAP_TYPE_PointerToSOAP_ENV__Detail (461)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerToSOAP_ENV__Detail(struct soap*, struct SOAP_ENV__Detail *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerToSOAP_ENV__Detail(struct soap*, struct SOAP_ENV__Detail *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerToSOAP_ENV__Detail(struct soap*, struct SOAP_ENV__Detail *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerToSOAP_ENV__Detail(struct soap*, const char *, int, struct SOAP_ENV__Detail *const*, const char *);
SOAP_FMAC3 struct SOAP_ENV__Detail ** SOAP_FMAC4 soap_get_PointerToSOAP_ENV__Detail(struct soap*, struct SOAP_ENV__Detail **, const char*, const char*);
SOAP_FMAC3 struct SOAP_ENV__Detail ** SOAP_FMAC4 soap_in_PointerToSOAP_ENV__Detail(struct soap*, const char*, struct SOAP_ENV__Detail **, const char*);

#endif

#ifndef SOAP_TYPE_PointerTo_ns1__getFreeBusyResponse
#define SOAP_TYPE_PointerTo_ns1__getFreeBusyResponse (453)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getFreeBusyResponse(struct soap*, _ns1__getFreeBusyResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getFreeBusyResponse(struct soap*, _ns1__getFreeBusyResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getFreeBusyResponse(struct soap*, _ns1__getFreeBusyResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getFreeBusyResponse(struct soap*, const char *, int, _ns1__getFreeBusyResponse *const*, const char *);
SOAP_FMAC3 _ns1__getFreeBusyResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__getFreeBusyResponse(struct soap*, _ns1__getFreeBusyResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__getFreeBusyResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__getFreeBusyResponse(struct soap*, const char*, _ns1__getFreeBusyResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getFreeBusyRequest
#define SOAP_TYPE_PointerTo_ns1__getFreeBusyRequest (452)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getFreeBusyRequest(struct soap*, _ns1__getFreeBusyRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getFreeBusyRequest(struct soap*, _ns1__getFreeBusyRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getFreeBusyRequest(struct soap*, _ns1__getFreeBusyRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getFreeBusyRequest(struct soap*, const char *, int, _ns1__getFreeBusyRequest *const*, const char *);
SOAP_FMAC3 _ns1__getFreeBusyRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__getFreeBusyRequest(struct soap*, _ns1__getFreeBusyRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__getFreeBusyRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__getFreeBusyRequest(struct soap*, const char*, _ns1__getFreeBusyRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__closeFreeBusySessionResponse
#define SOAP_TYPE_PointerTo_ns1__closeFreeBusySessionResponse (449)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__closeFreeBusySessionResponse(struct soap*, _ns1__closeFreeBusySessionResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__closeFreeBusySessionResponse(struct soap*, _ns1__closeFreeBusySessionResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__closeFreeBusySessionResponse(struct soap*, _ns1__closeFreeBusySessionResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__closeFreeBusySessionResponse(struct soap*, const char *, int, _ns1__closeFreeBusySessionResponse *const*, const char *);
SOAP_FMAC3 _ns1__closeFreeBusySessionResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__closeFreeBusySessionResponse(struct soap*, _ns1__closeFreeBusySessionResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__closeFreeBusySessionResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__closeFreeBusySessionResponse(struct soap*, const char*, _ns1__closeFreeBusySessionResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__closeFreeBusySessionRequest
#define SOAP_TYPE_PointerTo_ns1__closeFreeBusySessionRequest (448)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__closeFreeBusySessionRequest(struct soap*, _ns1__closeFreeBusySessionRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__closeFreeBusySessionRequest(struct soap*, _ns1__closeFreeBusySessionRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__closeFreeBusySessionRequest(struct soap*, _ns1__closeFreeBusySessionRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__closeFreeBusySessionRequest(struct soap*, const char *, int, _ns1__closeFreeBusySessionRequest *const*, const char *);
SOAP_FMAC3 _ns1__closeFreeBusySessionRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__closeFreeBusySessionRequest(struct soap*, _ns1__closeFreeBusySessionRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__closeFreeBusySessionRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__closeFreeBusySessionRequest(struct soap*, const char*, _ns1__closeFreeBusySessionRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__startFreeBusySessionResponse
#define SOAP_TYPE_PointerTo_ns1__startFreeBusySessionResponse (445)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__startFreeBusySessionResponse(struct soap*, _ns1__startFreeBusySessionResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__startFreeBusySessionResponse(struct soap*, _ns1__startFreeBusySessionResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__startFreeBusySessionResponse(struct soap*, _ns1__startFreeBusySessionResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__startFreeBusySessionResponse(struct soap*, const char *, int, _ns1__startFreeBusySessionResponse *const*, const char *);
SOAP_FMAC3 _ns1__startFreeBusySessionResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__startFreeBusySessionResponse(struct soap*, _ns1__startFreeBusySessionResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__startFreeBusySessionResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__startFreeBusySessionResponse(struct soap*, const char*, _ns1__startFreeBusySessionResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__startFreeBusySessionRequest
#define SOAP_TYPE_PointerTo_ns1__startFreeBusySessionRequest (444)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__startFreeBusySessionRequest(struct soap*, _ns1__startFreeBusySessionRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__startFreeBusySessionRequest(struct soap*, _ns1__startFreeBusySessionRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__startFreeBusySessionRequest(struct soap*, _ns1__startFreeBusySessionRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__startFreeBusySessionRequest(struct soap*, const char *, int, _ns1__startFreeBusySessionRequest *const*, const char *);
SOAP_FMAC3 _ns1__startFreeBusySessionRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__startFreeBusySessionRequest(struct soap*, _ns1__startFreeBusySessionRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__startFreeBusySessionRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__startFreeBusySessionRequest(struct soap*, const char*, _ns1__startFreeBusySessionRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__removeItemResponse
#define SOAP_TYPE_PointerTo_ns1__removeItemResponse (441)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__removeItemResponse(struct soap*, _ns1__removeItemResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__removeItemResponse(struct soap*, _ns1__removeItemResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__removeItemResponse(struct soap*, _ns1__removeItemResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__removeItemResponse(struct soap*, const char *, int, _ns1__removeItemResponse *const*, const char *);
SOAP_FMAC3 _ns1__removeItemResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__removeItemResponse(struct soap*, _ns1__removeItemResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__removeItemResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__removeItemResponse(struct soap*, const char*, _ns1__removeItemResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__removeItemRequest
#define SOAP_TYPE_PointerTo_ns1__removeItemRequest (440)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__removeItemRequest(struct soap*, _ns1__removeItemRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__removeItemRequest(struct soap*, _ns1__removeItemRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__removeItemRequest(struct soap*, _ns1__removeItemRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__removeItemRequest(struct soap*, const char *, int, _ns1__removeItemRequest *const*, const char *);
SOAP_FMAC3 _ns1__removeItemRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__removeItemRequest(struct soap*, _ns1__removeItemRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__removeItemRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__removeItemRequest(struct soap*, const char*, _ns1__removeItemRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__purgeItemResponse
#define SOAP_TYPE_PointerTo_ns1__purgeItemResponse (437)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__purgeItemResponse(struct soap*, _ns1__purgeItemResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__purgeItemResponse(struct soap*, _ns1__purgeItemResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__purgeItemResponse(struct soap*, _ns1__purgeItemResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__purgeItemResponse(struct soap*, const char *, int, _ns1__purgeItemResponse *const*, const char *);
SOAP_FMAC3 _ns1__purgeItemResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__purgeItemResponse(struct soap*, _ns1__purgeItemResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__purgeItemResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__purgeItemResponse(struct soap*, const char*, _ns1__purgeItemResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__purgeItemRequest
#define SOAP_TYPE_PointerTo_ns1__purgeItemRequest (436)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__purgeItemRequest(struct soap*, _ns1__purgeItemRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__purgeItemRequest(struct soap*, _ns1__purgeItemRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__purgeItemRequest(struct soap*, _ns1__purgeItemRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__purgeItemRequest(struct soap*, const char *, int, _ns1__purgeItemRequest *const*, const char *);
SOAP_FMAC3 _ns1__purgeItemRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__purgeItemRequest(struct soap*, _ns1__purgeItemRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__purgeItemRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__purgeItemRequest(struct soap*, const char*, _ns1__purgeItemRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__modifyItemResponse
#define SOAP_TYPE_PointerTo_ns1__modifyItemResponse (433)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__modifyItemResponse(struct soap*, _ns1__modifyItemResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__modifyItemResponse(struct soap*, _ns1__modifyItemResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__modifyItemResponse(struct soap*, _ns1__modifyItemResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__modifyItemResponse(struct soap*, const char *, int, _ns1__modifyItemResponse *const*, const char *);
SOAP_FMAC3 _ns1__modifyItemResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__modifyItemResponse(struct soap*, _ns1__modifyItemResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__modifyItemResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__modifyItemResponse(struct soap*, const char*, _ns1__modifyItemResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__modifyItemRequest
#define SOAP_TYPE_PointerTo_ns1__modifyItemRequest (432)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__modifyItemRequest(struct soap*, _ns1__modifyItemRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__modifyItemRequest(struct soap*, _ns1__modifyItemRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__modifyItemRequest(struct soap*, _ns1__modifyItemRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__modifyItemRequest(struct soap*, const char *, int, _ns1__modifyItemRequest *const*, const char *);
SOAP_FMAC3 _ns1__modifyItemRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__modifyItemRequest(struct soap*, _ns1__modifyItemRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__modifyItemRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__modifyItemRequest(struct soap*, const char*, _ns1__modifyItemRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__sendItemResponse
#define SOAP_TYPE_PointerTo_ns1__sendItemResponse (429)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__sendItemResponse(struct soap*, _ns1__sendItemResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__sendItemResponse(struct soap*, _ns1__sendItemResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__sendItemResponse(struct soap*, _ns1__sendItemResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__sendItemResponse(struct soap*, const char *, int, _ns1__sendItemResponse *const*, const char *);
SOAP_FMAC3 _ns1__sendItemResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__sendItemResponse(struct soap*, _ns1__sendItemResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__sendItemResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__sendItemResponse(struct soap*, const char*, _ns1__sendItemResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__sendItemRequest
#define SOAP_TYPE_PointerTo_ns1__sendItemRequest (428)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__sendItemRequest(struct soap*, _ns1__sendItemRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__sendItemRequest(struct soap*, _ns1__sendItemRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__sendItemRequest(struct soap*, _ns1__sendItemRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__sendItemRequest(struct soap*, const char *, int, _ns1__sendItemRequest *const*, const char *);
SOAP_FMAC3 _ns1__sendItemRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__sendItemRequest(struct soap*, _ns1__sendItemRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__sendItemRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__sendItemRequest(struct soap*, const char*, _ns1__sendItemRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__createItemResponse
#define SOAP_TYPE_PointerTo_ns1__createItemResponse (425)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__createItemResponse(struct soap*, _ns1__createItemResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__createItemResponse(struct soap*, _ns1__createItemResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__createItemResponse(struct soap*, _ns1__createItemResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__createItemResponse(struct soap*, const char *, int, _ns1__createItemResponse *const*, const char *);
SOAP_FMAC3 _ns1__createItemResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__createItemResponse(struct soap*, _ns1__createItemResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__createItemResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__createItemResponse(struct soap*, const char*, _ns1__createItemResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__createItemRequest
#define SOAP_TYPE_PointerTo_ns1__createItemRequest (424)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__createItemRequest(struct soap*, _ns1__createItemRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__createItemRequest(struct soap*, _ns1__createItemRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__createItemRequest(struct soap*, _ns1__createItemRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__createItemRequest(struct soap*, const char *, int, _ns1__createItemRequest *const*, const char *);
SOAP_FMAC3 _ns1__createItemRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__createItemRequest(struct soap*, _ns1__createItemRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__createItemRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__createItemRequest(struct soap*, const char*, _ns1__createItemRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getDeltaResponse
#define SOAP_TYPE_PointerTo_ns1__getDeltaResponse (421)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getDeltaResponse(struct soap*, _ns1__getDeltaResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getDeltaResponse(struct soap*, _ns1__getDeltaResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getDeltaResponse(struct soap*, _ns1__getDeltaResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getDeltaResponse(struct soap*, const char *, int, _ns1__getDeltaResponse *const*, const char *);
SOAP_FMAC3 _ns1__getDeltaResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__getDeltaResponse(struct soap*, _ns1__getDeltaResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__getDeltaResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__getDeltaResponse(struct soap*, const char*, _ns1__getDeltaResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getDeltaRequest
#define SOAP_TYPE_PointerTo_ns1__getDeltaRequest (420)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getDeltaRequest(struct soap*, _ns1__getDeltaRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getDeltaRequest(struct soap*, _ns1__getDeltaRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getDeltaRequest(struct soap*, _ns1__getDeltaRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getDeltaRequest(struct soap*, const char *, int, _ns1__getDeltaRequest *const*, const char *);
SOAP_FMAC3 _ns1__getDeltaRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__getDeltaRequest(struct soap*, _ns1__getDeltaRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__getDeltaRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__getDeltaRequest(struct soap*, const char*, _ns1__getDeltaRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getFolderListResponse
#define SOAP_TYPE_PointerTo_ns1__getFolderListResponse (417)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getFolderListResponse(struct soap*, _ns1__getFolderListResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getFolderListResponse(struct soap*, _ns1__getFolderListResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getFolderListResponse(struct soap*, _ns1__getFolderListResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getFolderListResponse(struct soap*, const char *, int, _ns1__getFolderListResponse *const*, const char *);
SOAP_FMAC3 _ns1__getFolderListResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__getFolderListResponse(struct soap*, _ns1__getFolderListResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__getFolderListResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__getFolderListResponse(struct soap*, const char*, _ns1__getFolderListResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getFolderListRequest
#define SOAP_TYPE_PointerTo_ns1__getFolderListRequest (416)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getFolderListRequest(struct soap*, _ns1__getFolderListRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getFolderListRequest(struct soap*, _ns1__getFolderListRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getFolderListRequest(struct soap*, _ns1__getFolderListRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getFolderListRequest(struct soap*, const char *, int, _ns1__getFolderListRequest *const*, const char *);
SOAP_FMAC3 _ns1__getFolderListRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__getFolderListRequest(struct soap*, _ns1__getFolderListRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__getFolderListRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__getFolderListRequest(struct soap*, const char*, _ns1__getFolderListRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getItemsResponse
#define SOAP_TYPE_PointerTo_ns1__getItemsResponse (413)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getItemsResponse(struct soap*, _ns1__getItemsResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getItemsResponse(struct soap*, _ns1__getItemsResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getItemsResponse(struct soap*, _ns1__getItemsResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getItemsResponse(struct soap*, const char *, int, _ns1__getItemsResponse *const*, const char *);
SOAP_FMAC3 _ns1__getItemsResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__getItemsResponse(struct soap*, _ns1__getItemsResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__getItemsResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__getItemsResponse(struct soap*, const char*, _ns1__getItemsResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getItemsRequest
#define SOAP_TYPE_PointerTo_ns1__getItemsRequest (412)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getItemsRequest(struct soap*, _ns1__getItemsRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getItemsRequest(struct soap*, _ns1__getItemsRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getItemsRequest(struct soap*, _ns1__getItemsRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getItemsRequest(struct soap*, const char *, int, _ns1__getItemsRequest *const*, const char *);
SOAP_FMAC3 _ns1__getItemsRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__getItemsRequest(struct soap*, _ns1__getItemsRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__getItemsRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__getItemsRequest(struct soap*, const char*, _ns1__getItemsRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getItemResponse
#define SOAP_TYPE_PointerTo_ns1__getItemResponse (409)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getItemResponse(struct soap*, _ns1__getItemResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getItemResponse(struct soap*, _ns1__getItemResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getItemResponse(struct soap*, _ns1__getItemResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getItemResponse(struct soap*, const char *, int, _ns1__getItemResponse *const*, const char *);
SOAP_FMAC3 _ns1__getItemResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__getItemResponse(struct soap*, _ns1__getItemResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__getItemResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__getItemResponse(struct soap*, const char*, _ns1__getItemResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getItemRequest
#define SOAP_TYPE_PointerTo_ns1__getItemRequest (408)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getItemRequest(struct soap*, _ns1__getItemRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getItemRequest(struct soap*, _ns1__getItemRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getItemRequest(struct soap*, _ns1__getItemRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getItemRequest(struct soap*, const char *, int, _ns1__getItemRequest *const*, const char *);
SOAP_FMAC3 _ns1__getItemRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__getItemRequest(struct soap*, _ns1__getItemRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__getItemRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__getItemRequest(struct soap*, const char*, _ns1__getItemRequest **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getAddressBookListResponse
#define SOAP_TYPE_PointerTo_ns1__getAddressBookListResponse (405)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getAddressBookListResponse(struct soap*, _ns1__getAddressBookListResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getAddressBookListResponse(struct soap*, _ns1__getAddressBookListResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getAddressBookListResponse(struct soap*, _ns1__getAddressBookListResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getAddressBookListResponse(struct soap*, const char *, int, _ns1__getAddressBookListResponse *const*, const char *);
SOAP_FMAC3 _ns1__getAddressBookListResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__getAddressBookListResponse(struct soap*, _ns1__getAddressBookListResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__getAddressBookListResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__getAddressBookListResponse(struct soap*, const char*, _ns1__getAddressBookListResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__getCategoryListResponse
#define SOAP_TYPE_PointerTo_ns1__getCategoryListResponse (402)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__getCategoryListResponse(struct soap*, _ns1__getCategoryListResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__getCategoryListResponse(struct soap*, _ns1__getCategoryListResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__getCategoryListResponse(struct soap*, _ns1__getCategoryListResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__getCategoryListResponse(struct soap*, const char *, int, _ns1__getCategoryListResponse *const*, const char *);
SOAP_FMAC3 _ns1__getCategoryListResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__getCategoryListResponse(struct soap*, _ns1__getCategoryListResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__getCategoryListResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__getCategoryListResponse(struct soap*, const char*, _ns1__getCategoryListResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__loginResponse
#define SOAP_TYPE_PointerTo_ns1__loginResponse (399)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__loginResponse(struct soap*, _ns1__loginResponse *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__loginResponse(struct soap*, _ns1__loginResponse *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__loginResponse(struct soap*, _ns1__loginResponse *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__loginResponse(struct soap*, const char *, int, _ns1__loginResponse *const*, const char *);
SOAP_FMAC3 _ns1__loginResponse ** SOAP_FMAC4 soap_get_PointerTo_ns1__loginResponse(struct soap*, _ns1__loginResponse **, const char*, const char*);
SOAP_FMAC3 _ns1__loginResponse ** SOAP_FMAC4 soap_in_PointerTo_ns1__loginResponse(struct soap*, const char*, _ns1__loginResponse **, const char*);

#ifndef SOAP_TYPE_PointerTo_ns1__loginRequest
#define SOAP_TYPE_PointerTo_ns1__loginRequest (398)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTo_ns1__loginRequest(struct soap*, _ns1__loginRequest *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTo_ns1__loginRequest(struct soap*, _ns1__loginRequest *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTo_ns1__loginRequest(struct soap*, _ns1__loginRequest *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTo_ns1__loginRequest(struct soap*, const char *, int, _ns1__loginRequest *const*, const char *);
SOAP_FMAC3 _ns1__loginRequest ** SOAP_FMAC4 soap_get_PointerTo_ns1__loginRequest(struct soap*, _ns1__loginRequest **, const char*, const char*);
SOAP_FMAC3 _ns1__loginRequest ** SOAP_FMAC4 soap_in_PointerTo_ns1__loginRequest(struct soap*, const char*, _ns1__loginRequest **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Alarm
#define SOAP_TYPE_PointerTons1__Alarm (396)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Alarm(struct soap*, ns1__Alarm *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Alarm(struct soap*, ns1__Alarm *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Alarm(struct soap*, ns1__Alarm *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Alarm(struct soap*, const char *, int, ns1__Alarm *const*, const char *);
SOAP_FMAC3 ns1__Alarm ** SOAP_FMAC4 soap_get_PointerTons1__Alarm(struct soap*, ns1__Alarm **, const char*, const char*);
SOAP_FMAC3 ns1__Alarm ** SOAP_FMAC4 soap_in_PointerTons1__Alarm(struct soap*, const char*, ns1__Alarm **, const char*);

#ifndef SOAP_TYPE_PointerTons1__PhoneFlags
#define SOAP_TYPE_PointerTons1__PhoneFlags (395)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__PhoneFlags(struct soap*, ns1__PhoneFlags *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__PhoneFlags(struct soap*, ns1__PhoneFlags *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__PhoneFlags(struct soap*, ns1__PhoneFlags *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__PhoneFlags(struct soap*, const char *, int, ns1__PhoneFlags *const*, const char *);
SOAP_FMAC3 ns1__PhoneFlags ** SOAP_FMAC4 soap_get_PointerTons1__PhoneFlags(struct soap*, ns1__PhoneFlags **, const char*, const char*);
SOAP_FMAC3 ns1__PhoneFlags ** SOAP_FMAC4 soap_in_PointerTons1__PhoneFlags(struct soap*, const char*, ns1__PhoneFlags **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AccessControlList
#define SOAP_TYPE_PointerTons1__AccessControlList (394)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AccessControlList(struct soap*, ns1__AccessControlList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AccessControlList(struct soap*, ns1__AccessControlList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AccessControlList(struct soap*, ns1__AccessControlList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AccessControlList(struct soap*, const char *, int, ns1__AccessControlList *const*, const char *);
SOAP_FMAC3 ns1__AccessControlList ** SOAP_FMAC4 soap_get_PointerTons1__AccessControlList(struct soap*, ns1__AccessControlList **, const char*, const char*);
SOAP_FMAC3 ns1__AccessControlList ** SOAP_FMAC4 soap_in_PointerTons1__AccessControlList(struct soap*, const char*, ns1__AccessControlList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__RecurrenceDateType
#define SOAP_TYPE_PointerTons1__RecurrenceDateType (393)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__RecurrenceDateType(struct soap*, ns1__RecurrenceDateType *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__RecurrenceDateType(struct soap*, ns1__RecurrenceDateType *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__RecurrenceDateType(struct soap*, ns1__RecurrenceDateType *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__RecurrenceDateType(struct soap*, const char *, int, ns1__RecurrenceDateType *const*, const char *);
SOAP_FMAC3 ns1__RecurrenceDateType ** SOAP_FMAC4 soap_get_PointerTons1__RecurrenceDateType(struct soap*, ns1__RecurrenceDateType **, const char*, const char*);
SOAP_FMAC3 ns1__RecurrenceDateType ** SOAP_FMAC4 soap_in_PointerTons1__RecurrenceDateType(struct soap*, const char*, ns1__RecurrenceDateType **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemOptions
#define SOAP_TYPE_PointerTons1__ItemOptions (392)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemOptions(struct soap*, ns1__ItemOptions *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemOptions(struct soap*, ns1__ItemOptions *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemOptions(struct soap*, ns1__ItemOptions *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemOptions(struct soap*, const char *, int, ns1__ItemOptions *const*, const char *);
SOAP_FMAC3 ns1__ItemOptions ** SOAP_FMAC4 soap_get_PointerTons1__ItemOptions(struct soap*, ns1__ItemOptions **, const char*, const char*);
SOAP_FMAC3 ns1__ItemOptions ** SOAP_FMAC4 soap_in_PointerTons1__ItemOptions(struct soap*, const char*, ns1__ItemOptions **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AttachmentInfo
#define SOAP_TYPE_PointerTons1__AttachmentInfo (391)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AttachmentInfo(struct soap*, ns1__AttachmentInfo *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AttachmentInfo(struct soap*, ns1__AttachmentInfo *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AttachmentInfo(struct soap*, ns1__AttachmentInfo *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AttachmentInfo(struct soap*, const char *, int, ns1__AttachmentInfo *const*, const char *);
SOAP_FMAC3 ns1__AttachmentInfo ** SOAP_FMAC4 soap_get_PointerTons1__AttachmentInfo(struct soap*, ns1__AttachmentInfo **, const char*, const char*);
SOAP_FMAC3 ns1__AttachmentInfo ** SOAP_FMAC4 soap_in_PointerTons1__AttachmentInfo(struct soap*, const char*, ns1__AttachmentInfo **, const char*);

#ifndef SOAP_TYPE_PointerTons1__MessageBody
#define SOAP_TYPE_PointerTons1__MessageBody (390)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__MessageBody(struct soap*, ns1__MessageBody *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__MessageBody(struct soap*, ns1__MessageBody *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__MessageBody(struct soap*, ns1__MessageBody *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__MessageBody(struct soap*, const char *, int, ns1__MessageBody *const*, const char *);
SOAP_FMAC3 ns1__MessageBody ** SOAP_FMAC4 soap_get_PointerTons1__MessageBody(struct soap*, ns1__MessageBody **, const char*, const char*);
SOAP_FMAC3 ns1__MessageBody ** SOAP_FMAC4 soap_in_PointerTons1__MessageBody(struct soap*, const char*, ns1__MessageBody **, const char*);

#ifndef SOAP_TYPE_PointerTons1__PersonalInfo
#define SOAP_TYPE_PointerTons1__PersonalInfo (389)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__PersonalInfo(struct soap*, ns1__PersonalInfo *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__PersonalInfo(struct soap*, ns1__PersonalInfo *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__PersonalInfo(struct soap*, ns1__PersonalInfo *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__PersonalInfo(struct soap*, const char *, int, ns1__PersonalInfo *const*, const char *);
SOAP_FMAC3 ns1__PersonalInfo ** SOAP_FMAC4 soap_get_PointerTons1__PersonalInfo(struct soap*, ns1__PersonalInfo **, const char*, const char*);
SOAP_FMAC3 ns1__PersonalInfo ** SOAP_FMAC4 soap_in_PointerTons1__PersonalInfo(struct soap*, const char*, ns1__PersonalInfo **, const char*);

#ifndef SOAP_TYPE_PointerTons1__OfficeInfo
#define SOAP_TYPE_PointerTons1__OfficeInfo (388)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__OfficeInfo(struct soap*, ns1__OfficeInfo *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__OfficeInfo(struct soap*, ns1__OfficeInfo *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__OfficeInfo(struct soap*, ns1__OfficeInfo *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__OfficeInfo(struct soap*, const char *, int, ns1__OfficeInfo *const*, const char *);
SOAP_FMAC3 ns1__OfficeInfo ** SOAP_FMAC4 soap_get_PointerTons1__OfficeInfo(struct soap*, ns1__OfficeInfo **, const char*, const char*);
SOAP_FMAC3 ns1__OfficeInfo ** SOAP_FMAC4 soap_in_PointerTons1__OfficeInfo(struct soap*, const char*, ns1__OfficeInfo **, const char*);

#ifndef SOAP_TYPE_PointerTons1__PostalAddressList
#define SOAP_TYPE_PointerTons1__PostalAddressList (387)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__PostalAddressList(struct soap*, ns1__PostalAddressList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__PostalAddressList(struct soap*, ns1__PostalAddressList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__PostalAddressList(struct soap*, ns1__PostalAddressList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__PostalAddressList(struct soap*, const char *, int, ns1__PostalAddressList *const*, const char *);
SOAP_FMAC3 ns1__PostalAddressList ** SOAP_FMAC4 soap_get_PointerTons1__PostalAddressList(struct soap*, ns1__PostalAddressList **, const char*, const char*);
SOAP_FMAC3 ns1__PostalAddressList ** SOAP_FMAC4 soap_in_PointerTons1__PostalAddressList(struct soap*, const char*, ns1__PostalAddressList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__PhoneList
#define SOAP_TYPE_PointerTons1__PhoneList (386)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__PhoneList(struct soap*, ns1__PhoneList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__PhoneList(struct soap*, ns1__PhoneList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__PhoneList(struct soap*, ns1__PhoneList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__PhoneList(struct soap*, const char *, int, ns1__PhoneList *const*, const char *);
SOAP_FMAC3 ns1__PhoneList ** SOAP_FMAC4 soap_get_PointerTons1__PhoneList(struct soap*, ns1__PhoneList **, const char*, const char*);
SOAP_FMAC3 ns1__PhoneList ** SOAP_FMAC4 soap_in_PointerTons1__PhoneList(struct soap*, const char*, ns1__PhoneList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ImAddressList
#define SOAP_TYPE_PointerTons1__ImAddressList (385)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ImAddressList(struct soap*, ns1__ImAddressList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ImAddressList(struct soap*, ns1__ImAddressList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ImAddressList(struct soap*, ns1__ImAddressList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ImAddressList(struct soap*, const char *, int, ns1__ImAddressList *const*, const char *);
SOAP_FMAC3 ns1__ImAddressList ** SOAP_FMAC4 soap_get_PointerTons1__ImAddressList(struct soap*, ns1__ImAddressList **, const char*, const char*);
SOAP_FMAC3 ns1__ImAddressList ** SOAP_FMAC4 soap_in_PointerTons1__ImAddressList(struct soap*, const char*, ns1__ImAddressList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__EmailAddressList
#define SOAP_TYPE_PointerTons1__EmailAddressList (384)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__EmailAddressList(struct soap*, ns1__EmailAddressList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__EmailAddressList(struct soap*, ns1__EmailAddressList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__EmailAddressList(struct soap*, ns1__EmailAddressList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__EmailAddressList(struct soap*, const char *, int, ns1__EmailAddressList *const*, const char *);
SOAP_FMAC3 ns1__EmailAddressList ** SOAP_FMAC4 soap_get_PointerTons1__EmailAddressList(struct soap*, ns1__EmailAddressList **, const char*, const char*);
SOAP_FMAC3 ns1__EmailAddressList ** SOAP_FMAC4 soap_in_PointerTons1__EmailAddressList(struct soap*, const char*, ns1__EmailAddressList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FullName
#define SOAP_TYPE_PointerTons1__FullName (383)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FullName(struct soap*, ns1__FullName *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FullName(struct soap*, ns1__FullName *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FullName(struct soap*, ns1__FullName *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FullName(struct soap*, const char *, int, ns1__FullName *const*, const char *);
SOAP_FMAC3 ns1__FullName ** SOAP_FMAC4 soap_get_PointerTons1__FullName(struct soap*, ns1__FullName **, const char*, const char*);
SOAP_FMAC3 ns1__FullName ** SOAP_FMAC4 soap_in_PointerTons1__FullName(struct soap*, const char*, ns1__FullName **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FolderACL
#define SOAP_TYPE_PointerTons1__FolderACL (382)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FolderACL(struct soap*, ns1__FolderACL *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FolderACL(struct soap*, ns1__FolderACL *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FolderACL(struct soap*, ns1__FolderACL *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FolderACL(struct soap*, const char *, int, ns1__FolderACL *const*, const char *);
SOAP_FMAC3 ns1__FolderACL ** SOAP_FMAC4 soap_get_PointerTons1__FolderACL(struct soap*, ns1__FolderACL **, const char*, const char*);
SOAP_FMAC3 ns1__FolderACL ** SOAP_FMAC4 soap_in_PointerTons1__FolderACL(struct soap*, const char*, ns1__FolderACL **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Query
#define SOAP_TYPE_PointerTons1__Query (381)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Query(struct soap*, ns1__Query *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Query(struct soap*, ns1__Query *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Query(struct soap*, ns1__Query *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Query(struct soap*, const char *, int, ns1__Query *const*, const char *);
SOAP_FMAC3 ns1__Query ** SOAP_FMAC4 soap_get_PointerTons1__Query(struct soap*, ns1__Query **, const char*, const char*);
SOAP_FMAC3 ns1__Query ** SOAP_FMAC4 soap_in_PointerTons1__Query(struct soap*, const char*, ns1__Query **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FreeBusyBlockList
#define SOAP_TYPE_PointerTons1__FreeBusyBlockList (380)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FreeBusyBlockList(struct soap*, ns1__FreeBusyBlockList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FreeBusyBlockList(struct soap*, ns1__FreeBusyBlockList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FreeBusyBlockList(struct soap*, ns1__FreeBusyBlockList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FreeBusyBlockList(struct soap*, const char *, int, ns1__FreeBusyBlockList *const*, const char *);
SOAP_FMAC3 ns1__FreeBusyBlockList ** SOAP_FMAC4 soap_get_PointerTons1__FreeBusyBlockList(struct soap*, ns1__FreeBusyBlockList **, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyBlockList ** SOAP_FMAC4 soap_in_PointerTons1__FreeBusyBlockList(struct soap*, const char*, ns1__FreeBusyBlockList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemClass
#define SOAP_TYPE_PointerTons1__ItemClass (379)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemClass(struct soap*, enum ns1__ItemClass *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemClass(struct soap*, enum ns1__ItemClass *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemClass(struct soap*, enum ns1__ItemClass *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemClass(struct soap*, const char *, int, enum ns1__ItemClass *const*, const char *);
SOAP_FMAC3 enum ns1__ItemClass ** SOAP_FMAC4 soap_get_PointerTons1__ItemClass(struct soap*, enum ns1__ItemClass **, const char*, const char*);
SOAP_FMAC3 enum ns1__ItemClass ** SOAP_FMAC4 soap_in_PointerTons1__ItemClass(struct soap*, const char*, enum ns1__ItemClass **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemSource
#define SOAP_TYPE_PointerTons1__ItemSource (378)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemSource(struct soap*, enum ns1__ItemSource *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemSource(struct soap*, enum ns1__ItemSource *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemSource(struct soap*, enum ns1__ItemSource *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemSource(struct soap*, const char *, int, enum ns1__ItemSource *const*, const char *);
SOAP_FMAC3 enum ns1__ItemSource ** SOAP_FMAC4 soap_get_PointerTons1__ItemSource(struct soap*, enum ns1__ItemSource **, const char*, const char*);
SOAP_FMAC3 enum ns1__ItemSource ** SOAP_FMAC4 soap_in_PointerTons1__ItemSource(struct soap*, const char*, enum ns1__ItemSource **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemStatus
#define SOAP_TYPE_PointerTons1__ItemStatus (377)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemStatus(struct soap*, ns1__ItemStatus *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemStatus(struct soap*, ns1__ItemStatus *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemStatus(struct soap*, ns1__ItemStatus *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemStatus(struct soap*, const char *, int, ns1__ItemStatus *const*, const char *);
SOAP_FMAC3 ns1__ItemStatus ** SOAP_FMAC4 soap_get_PointerTons1__ItemStatus(struct soap*, ns1__ItemStatus **, const char*, const char*);
SOAP_FMAC3 ns1__ItemStatus ** SOAP_FMAC4 soap_in_PointerTons1__ItemStatus(struct soap*, const char*, ns1__ItemStatus **, const char*);

#ifndef SOAP_TYPE_PointerTons1__RecipientStatus
#define SOAP_TYPE_PointerTons1__RecipientStatus (376)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__RecipientStatus(struct soap*, ns1__RecipientStatus *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__RecipientStatus(struct soap*, ns1__RecipientStatus *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__RecipientStatus(struct soap*, ns1__RecipientStatus *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__RecipientStatus(struct soap*, const char *, int, ns1__RecipientStatus *const*, const char *);
SOAP_FMAC3 ns1__RecipientStatus ** SOAP_FMAC4 soap_get_PointerTons1__RecipientStatus(struct soap*, ns1__RecipientStatus **, const char*, const char*);
SOAP_FMAC3 ns1__RecipientStatus ** SOAP_FMAC4 soap_in_PointerTons1__RecipientStatus(struct soap*, const char*, ns1__RecipientStatus **, const char*);

#ifndef SOAP_TYPE_PointerTons1__SharedFolderNotification
#define SOAP_TYPE_PointerTons1__SharedFolderNotification (375)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__SharedFolderNotification(struct soap*, ns1__SharedFolderNotification *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__SharedFolderNotification(struct soap*, ns1__SharedFolderNotification *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__SharedFolderNotification(struct soap*, ns1__SharedFolderNotification *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__SharedFolderNotification(struct soap*, const char *, int, ns1__SharedFolderNotification *const*, const char *);
SOAP_FMAC3 ns1__SharedFolderNotification ** SOAP_FMAC4 soap_get_PointerTons1__SharedFolderNotification(struct soap*, ns1__SharedFolderNotification **, const char*, const char*);
SOAP_FMAC3 ns1__SharedFolderNotification ** SOAP_FMAC4 soap_in_PointerTons1__SharedFolderNotification(struct soap*, const char*, ns1__SharedFolderNotification **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FilterElement
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FilterElement (374)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__FilterElement(struct soap*, const char *, int, std::vector<ns1__FilterElement * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__FilterElement * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FilterElement * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__FilterElement(struct soap*, const char*, std::vector<ns1__FilterElement * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__CategoryRefList
#define SOAP_TYPE_PointerTons1__CategoryRefList (372)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__CategoryRefList(struct soap*, ns1__CategoryRefList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__CategoryRefList(struct soap*, ns1__CategoryRefList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__CategoryRefList(struct soap*, ns1__CategoryRefList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__CategoryRefList(struct soap*, const char *, int, ns1__CategoryRefList *const*, const char *);
SOAP_FMAC3 ns1__CategoryRefList ** SOAP_FMAC4 soap_get_PointerTons1__CategoryRefList(struct soap*, ns1__CategoryRefList **, const char*, const char*);
SOAP_FMAC3 ns1__CategoryRefList ** SOAP_FMAC4 soap_in_PointerTons1__CategoryRefList(struct soap*, const char*, ns1__CategoryRefList **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef (371)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, const char *, int, std::vector<ns1__ContainerRef * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__ContainerRef * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__ContainerRef * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, const char*, std::vector<ns1__ContainerRef * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__ContainerRef
#define SOAP_TYPE_PointerTons1__ContainerRef (369)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ContainerRef(struct soap*, ns1__ContainerRef *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ContainerRef(struct soap*, ns1__ContainerRef *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ContainerRef(struct soap*, ns1__ContainerRef *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ContainerRef(struct soap*, const char *, int, ns1__ContainerRef *const*, const char *);
SOAP_FMAC3 ns1__ContainerRef ** SOAP_FMAC4 soap_get_PointerTons1__ContainerRef(struct soap*, ns1__ContainerRef **, const char*, const char*);
SOAP_FMAC3 ns1__ContainerRef ** SOAP_FMAC4 soap_in_PointerTons1__ContainerRef(struct soap*, const char*, ns1__ContainerRef **, const char*);

#ifndef SOAP_TYPE_PointerTons1__CategoryType
#define SOAP_TYPE_PointerTons1__CategoryType (368)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__CategoryType(struct soap*, enum ns1__CategoryType *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__CategoryType(struct soap*, enum ns1__CategoryType *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__CategoryType(struct soap*, enum ns1__CategoryType *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__CategoryType(struct soap*, const char *, int, enum ns1__CategoryType *const*, const char *);
SOAP_FMAC3 enum ns1__CategoryType ** SOAP_FMAC4 soap_get_PointerTons1__CategoryType(struct soap*, enum ns1__CategoryType **, const char*, const char*);
SOAP_FMAC3 enum ns1__CategoryType ** SOAP_FMAC4 soap_in_PointerTons1__CategoryType(struct soap*, const char*, enum ns1__CategoryType **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AccessMiscRight
#define SOAP_TYPE_PointerTons1__AccessMiscRight (367)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AccessMiscRight(struct soap*, ns1__AccessMiscRight *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AccessMiscRight(struct soap*, ns1__AccessMiscRight *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AccessMiscRight(struct soap*, ns1__AccessMiscRight *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AccessMiscRight(struct soap*, const char *, int, ns1__AccessMiscRight *const*, const char *);
SOAP_FMAC3 ns1__AccessMiscRight ** SOAP_FMAC4 soap_get_PointerTons1__AccessMiscRight(struct soap*, ns1__AccessMiscRight **, const char*, const char*);
SOAP_FMAC3 ns1__AccessMiscRight ** SOAP_FMAC4 soap_in_PointerTons1__AccessMiscRight(struct soap*, const char*, ns1__AccessMiscRight **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AccessRight
#define SOAP_TYPE_PointerTons1__AccessRight (366)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AccessRight(struct soap*, ns1__AccessRight *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AccessRight(struct soap*, ns1__AccessRight *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AccessRight(struct soap*, ns1__AccessRight *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AccessRight(struct soap*, const char *, int, ns1__AccessRight *const*, const char *);
SOAP_FMAC3 ns1__AccessRight ** SOAP_FMAC4 soap_get_PointerTons1__AccessRight(struct soap*, ns1__AccessRight **, const char*, const char*);
SOAP_FMAC3 ns1__AccessRight ** SOAP_FMAC4 soap_in_PointerTons1__AccessRight(struct soap*, const char*, ns1__AccessRight **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Rights
#define SOAP_TYPE_PointerTons1__Rights (365)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Rights(struct soap*, ns1__Rights *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Rights(struct soap*, ns1__Rights *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Rights(struct soap*, ns1__Rights *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Rights(struct soap*, const char *, int, ns1__Rights *const*, const char *);
SOAP_FMAC3 ns1__Rights ** SOAP_FMAC4 soap_get_PointerTons1__Rights(struct soap*, ns1__Rights **, const char*, const char*);
SOAP_FMAC3 ns1__Rights ** SOAP_FMAC4 soap_in_PointerTons1__Rights(struct soap*, const char*, ns1__Rights **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FreeBusyUserList
#define SOAP_TYPE_PointerTons1__FreeBusyUserList (364)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FreeBusyUserList(struct soap*, ns1__FreeBusyUserList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FreeBusyUserList(struct soap*, ns1__FreeBusyUserList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FreeBusyUserList(struct soap*, ns1__FreeBusyUserList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FreeBusyUserList(struct soap*, const char *, int, ns1__FreeBusyUserList *const*, const char *);
SOAP_FMAC3 ns1__FreeBusyUserList ** SOAP_FMAC4 soap_get_PointerTons1__FreeBusyUserList(struct soap*, ns1__FreeBusyUserList **, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyUserList ** SOAP_FMAC4 soap_in_PointerTons1__FreeBusyUserList(struct soap*, const char*, ns1__FreeBusyUserList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemList
#define SOAP_TYPE_PointerTons1__ItemList (363)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemList(struct soap*, ns1__ItemList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemList(struct soap*, ns1__ItemList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemList(struct soap*, ns1__ItemList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemList(struct soap*, const char *, int, ns1__ItemList *const*, const char *);
SOAP_FMAC3 ns1__ItemList ** SOAP_FMAC4 soap_get_PointerTons1__ItemList(struct soap*, ns1__ItemList **, const char*, const char*);
SOAP_FMAC3 ns1__ItemList ** SOAP_FMAC4 soap_in_PointerTons1__ItemList(struct soap*, const char*, ns1__ItemList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__SettingsList
#define SOAP_TYPE_PointerTons1__SettingsList (362)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__SettingsList(struct soap*, ns1__SettingsList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__SettingsList(struct soap*, ns1__SettingsList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__SettingsList(struct soap*, ns1__SettingsList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__SettingsList(struct soap*, const char *, int, ns1__SettingsList *const*, const char *);
SOAP_FMAC3 ns1__SettingsList ** SOAP_FMAC4 soap_get_PointerTons1__SettingsList(struct soap*, ns1__SettingsList **, const char*, const char*);
SOAP_FMAC3 ns1__SettingsList ** SOAP_FMAC4 soap_in_PointerTons1__SettingsList(struct soap*, const char*, ns1__SettingsList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AccessRightChanges
#define SOAP_TYPE_PointerTons1__AccessRightChanges (361)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AccessRightChanges(struct soap*, ns1__AccessRightChanges *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AccessRightChanges(struct soap*, ns1__AccessRightChanges *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AccessRightChanges(struct soap*, ns1__AccessRightChanges *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AccessRightChanges(struct soap*, const char *, int, ns1__AccessRightChanges *const*, const char *);
SOAP_FMAC3 ns1__AccessRightChanges ** SOAP_FMAC4 soap_get_PointerTons1__AccessRightChanges(struct soap*, ns1__AccessRightChanges **, const char*, const char*);
SOAP_FMAC3 ns1__AccessRightChanges ** SOAP_FMAC4 soap_in_PointerTons1__AccessRightChanges(struct soap*, const char*, ns1__AccessRightChanges **, const char*);

#ifndef SOAP_TYPE_PointerTons1__UserInfo
#define SOAP_TYPE_PointerTons1__UserInfo (360)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__UserInfo(struct soap*, ns1__UserInfo *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__UserInfo(struct soap*, ns1__UserInfo *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__UserInfo(struct soap*, ns1__UserInfo *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__UserInfo(struct soap*, const char *, int, ns1__UserInfo *const*, const char *);
SOAP_FMAC3 ns1__UserInfo ** SOAP_FMAC4 soap_get_PointerTons1__UserInfo(struct soap*, ns1__UserInfo **, const char*, const char*);
SOAP_FMAC3 ns1__UserInfo ** SOAP_FMAC4 soap_in_PointerTons1__UserInfo(struct soap*, const char*, ns1__UserInfo **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Authentication
#define SOAP_TYPE_PointerTons1__Authentication (359)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Authentication(struct soap*, ns1__Authentication *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Authentication(struct soap*, ns1__Authentication *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Authentication(struct soap*, ns1__Authentication *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Authentication(struct soap*, const char *, int, ns1__Authentication *const*, const char *);
SOAP_FMAC3 ns1__Authentication ** SOAP_FMAC4 soap_get_PointerTons1__Authentication(struct soap*, ns1__Authentication **, const char*, const char*);
SOAP_FMAC3 ns1__Authentication ** SOAP_FMAC4 soap_in_PointerTons1__Authentication(struct soap*, const char*, ns1__Authentication **, const char*);

#ifndef SOAP_TYPE_PointerTons1__TimezoneList
#define SOAP_TYPE_PointerTons1__TimezoneList (358)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__TimezoneList(struct soap*, ns1__TimezoneList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__TimezoneList(struct soap*, ns1__TimezoneList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__TimezoneList(struct soap*, ns1__TimezoneList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__TimezoneList(struct soap*, const char *, int, ns1__TimezoneList *const*, const char *);
SOAP_FMAC3 ns1__TimezoneList ** SOAP_FMAC4 soap_get_PointerTons1__TimezoneList(struct soap*, ns1__TimezoneList **, const char*, const char*);
SOAP_FMAC3 ns1__TimezoneList ** SOAP_FMAC4 soap_in_PointerTons1__TimezoneList(struct soap*, const char*, ns1__TimezoneList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Settings
#define SOAP_TYPE_PointerTons1__Settings (357)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Settings(struct soap*, ns1__Settings *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Settings(struct soap*, ns1__Settings *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Settings(struct soap*, ns1__Settings *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Settings(struct soap*, const char *, int, ns1__Settings *const*, const char *);
SOAP_FMAC3 ns1__Settings ** SOAP_FMAC4 soap_get_PointerTons1__Settings(struct soap*, ns1__Settings **, const char*, const char*);
SOAP_FMAC3 ns1__Settings ** SOAP_FMAC4 soap_in_PointerTons1__Settings(struct soap*, const char*, ns1__Settings **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ProxyList
#define SOAP_TYPE_PointerTons1__ProxyList (356)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ProxyList(struct soap*, ns1__ProxyList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ProxyList(struct soap*, ns1__ProxyList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ProxyList(struct soap*, ns1__ProxyList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ProxyList(struct soap*, const char *, int, ns1__ProxyList *const*, const char *);
SOAP_FMAC3 ns1__ProxyList ** SOAP_FMAC4 soap_get_PointerTons1__ProxyList(struct soap*, ns1__ProxyList **, const char*, const char*);
SOAP_FMAC3 ns1__ProxyList ** SOAP_FMAC4 soap_in_PointerTons1__ProxyList(struct soap*, const char*, ns1__ProxyList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AccessRightList
#define SOAP_TYPE_PointerTons1__AccessRightList (355)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AccessRightList(struct soap*, ns1__AccessRightList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AccessRightList(struct soap*, ns1__AccessRightList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AccessRightList(struct soap*, ns1__AccessRightList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AccessRightList(struct soap*, const char *, int, ns1__AccessRightList *const*, const char *);
SOAP_FMAC3 ns1__AccessRightList ** SOAP_FMAC4 soap_get_PointerTons1__AccessRightList(struct soap*, ns1__AccessRightList **, const char*, const char*);
SOAP_FMAC3 ns1__AccessRightList ** SOAP_FMAC4 soap_in_PointerTons1__AccessRightList(struct soap*, const char*, ns1__AccessRightList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__View
#define SOAP_TYPE_PointerTons1__View (354)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__View(struct soap*, std::string *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__View(struct soap*, std::string *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__View(struct soap*, std::string *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__View(struct soap*, const char *, int, std::string *const*, const char *);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_get_PointerTons1__View(struct soap*, std::string **, const char*, const char*);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_in_PointerTons1__View(struct soap*, const char*, std::string **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FreeBusyInfoList
#define SOAP_TYPE_PointerTons1__FreeBusyInfoList (353)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FreeBusyInfoList(struct soap*, ns1__FreeBusyInfoList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FreeBusyInfoList(struct soap*, ns1__FreeBusyInfoList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FreeBusyInfoList(struct soap*, ns1__FreeBusyInfoList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FreeBusyInfoList(struct soap*, const char *, int, ns1__FreeBusyInfoList *const*, const char *);
SOAP_FMAC3 ns1__FreeBusyInfoList ** SOAP_FMAC4 soap_get_PointerTons1__FreeBusyInfoList(struct soap*, ns1__FreeBusyInfoList **, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyInfoList ** SOAP_FMAC4 soap_in_PointerTons1__FreeBusyInfoList(struct soap*, const char*, ns1__FreeBusyInfoList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FreeBusyStats
#define SOAP_TYPE_PointerTons1__FreeBusyStats (352)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FreeBusyStats(struct soap*, ns1__FreeBusyStats *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FreeBusyStats(struct soap*, ns1__FreeBusyStats *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FreeBusyStats(struct soap*, ns1__FreeBusyStats *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FreeBusyStats(struct soap*, const char *, int, ns1__FreeBusyStats *const*, const char *);
SOAP_FMAC3 ns1__FreeBusyStats ** SOAP_FMAC4 soap_get_PointerTons1__FreeBusyStats(struct soap*, ns1__FreeBusyStats **, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyStats ** SOAP_FMAC4 soap_in_PointerTons1__FreeBusyStats(struct soap*, const char*, ns1__FreeBusyStats **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FolderList
#define SOAP_TYPE_PointerTons1__FolderList (351)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FolderList(struct soap*, ns1__FolderList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FolderList(struct soap*, ns1__FolderList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FolderList(struct soap*, ns1__FolderList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FolderList(struct soap*, const char *, int, ns1__FolderList *const*, const char *);
SOAP_FMAC3 ns1__FolderList ** SOAP_FMAC4 soap_get_PointerTons1__FolderList(struct soap*, ns1__FolderList **, const char*, const char*);
SOAP_FMAC3 ns1__FolderList ** SOAP_FMAC4 soap_in_PointerTons1__FolderList(struct soap*, const char*, ns1__FolderList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__DeltaList
#define SOAP_TYPE_PointerTons1__DeltaList (350)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__DeltaList(struct soap*, ns1__DeltaList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__DeltaList(struct soap*, ns1__DeltaList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__DeltaList(struct soap*, ns1__DeltaList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__DeltaList(struct soap*, const char *, int, ns1__DeltaList *const*, const char *);
SOAP_FMAC3 ns1__DeltaList ** SOAP_FMAC4 soap_get_PointerTons1__DeltaList(struct soap*, ns1__DeltaList **, const char*, const char*);
SOAP_FMAC3 ns1__DeltaList ** SOAP_FMAC4 soap_in_PointerTons1__DeltaList(struct soap*, const char*, ns1__DeltaList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__CustomList
#define SOAP_TYPE_PointerTons1__CustomList (349)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__CustomList(struct soap*, ns1__CustomList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__CustomList(struct soap*, ns1__CustomList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__CustomList(struct soap*, ns1__CustomList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__CustomList(struct soap*, const char *, int, ns1__CustomList *const*, const char *);
SOAP_FMAC3 ns1__CustomList ** SOAP_FMAC4 soap_get_PointerTons1__CustomList(struct soap*, ns1__CustomList **, const char*, const char*);
SOAP_FMAC3 ns1__CustomList ** SOAP_FMAC4 soap_in_PointerTons1__CustomList(struct soap*, const char*, ns1__CustomList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__CategoryList
#define SOAP_TYPE_PointerTons1__CategoryList (348)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__CategoryList(struct soap*, ns1__CategoryList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__CategoryList(struct soap*, ns1__CategoryList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__CategoryList(struct soap*, ns1__CategoryList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__CategoryList(struct soap*, const char *, int, ns1__CategoryList *const*, const char *);
SOAP_FMAC3 ns1__CategoryList ** SOAP_FMAC4 soap_get_PointerTons1__CategoryList(struct soap*, ns1__CategoryList **, const char*, const char*);
SOAP_FMAC3 ns1__CategoryList ** SOAP_FMAC4 soap_in_PointerTons1__CategoryList(struct soap*, const char*, ns1__CategoryList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AddressBookList
#define SOAP_TYPE_PointerTons1__AddressBookList (347)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AddressBookList(struct soap*, ns1__AddressBookList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AddressBookList(struct soap*, ns1__AddressBookList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AddressBookList(struct soap*, ns1__AddressBookList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AddressBookList(struct soap*, const char *, int, ns1__AddressBookList *const*, const char *);
SOAP_FMAC3 ns1__AddressBookList ** SOAP_FMAC4 soap_get_PointerTons1__AddressBookList(struct soap*, ns1__AddressBookList **, const char*, const char*);
SOAP_FMAC3 ns1__AddressBookList ** SOAP_FMAC4 soap_in_PointerTons1__AddressBookList(struct soap*, const char*, ns1__AddressBookList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Distribution
#define SOAP_TYPE_PointerTons1__Distribution (346)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Distribution(struct soap*, ns1__Distribution *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Distribution(struct soap*, ns1__Distribution *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Distribution(struct soap*, ns1__Distribution *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Distribution(struct soap*, const char *, int, ns1__Distribution *const*, const char *);
SOAP_FMAC3 ns1__Distribution ** SOAP_FMAC4 soap_get_PointerTons1__Distribution(struct soap*, ns1__Distribution **, const char*, const char*);
SOAP_FMAC3 ns1__Distribution ** SOAP_FMAC4 soap_in_PointerTons1__Distribution(struct soap*, const char*, ns1__Distribution **, const char*);

#ifndef SOAP_TYPE_PointerTons1__GroupMemberList
#define SOAP_TYPE_PointerTons1__GroupMemberList (345)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__GroupMemberList(struct soap*, ns1__GroupMemberList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__GroupMemberList(struct soap*, ns1__GroupMemberList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__GroupMemberList(struct soap*, ns1__GroupMemberList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__GroupMemberList(struct soap*, const char *, int, ns1__GroupMemberList *const*, const char *);
SOAP_FMAC3 ns1__GroupMemberList ** SOAP_FMAC4 soap_get_PointerTons1__GroupMemberList(struct soap*, ns1__GroupMemberList **, const char*, const char*);
SOAP_FMAC3 ns1__GroupMemberList ** SOAP_FMAC4 soap_in_PointerTons1__GroupMemberList(struct soap*, const char*, ns1__GroupMemberList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Status
#define SOAP_TYPE_PointerTons1__Status (344)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Status(struct soap*, ns1__Status *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Status(struct soap*, ns1__Status *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Status(struct soap*, ns1__Status *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Status(struct soap*, const char *, int, ns1__Status *const*, const char *);
SOAP_FMAC3 ns1__Status ** SOAP_FMAC4 soap_get_PointerTons1__Status(struct soap*, ns1__Status **, const char*, const char*);
SOAP_FMAC3 ns1__Status ** SOAP_FMAC4 soap_in_PointerTons1__Status(struct soap*, const char*, ns1__Status **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemRefList
#define SOAP_TYPE_PointerTons1__ItemRefList (343)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemRefList(struct soap*, ns1__ItemRefList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemRefList(struct soap*, ns1__ItemRefList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemRefList(struct soap*, ns1__ItemRefList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemRefList(struct soap*, const char *, int, ns1__ItemRefList *const*, const char *);
SOAP_FMAC3 ns1__ItemRefList ** SOAP_FMAC4 soap_get_PointerTons1__ItemRefList(struct soap*, ns1__ItemRefList **, const char*, const char*);
SOAP_FMAC3 ns1__ItemRefList ** SOAP_FMAC4 soap_in_PointerTons1__ItemRefList(struct soap*, const char*, ns1__ItemRefList **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Timezone
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Timezone (342)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__Timezone(struct soap*, const char *, int, std::vector<ns1__Timezone * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__Timezone * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Timezone * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__Timezone(struct soap*, const char*, std::vector<ns1__Timezone * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__Timezone
#define SOAP_TYPE_PointerTons1__Timezone (340)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Timezone(struct soap*, ns1__Timezone *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Timezone(struct soap*, ns1__Timezone *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Timezone(struct soap*, ns1__Timezone *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Timezone(struct soap*, const char *, int, ns1__Timezone *const*, const char *);
SOAP_FMAC3 ns1__Timezone ** SOAP_FMAC4 soap_get_PointerTons1__Timezone(struct soap*, ns1__Timezone **, const char*, const char*);
SOAP_FMAC3 ns1__Timezone ** SOAP_FMAC4 soap_in_PointerTons1__Timezone(struct soap*, const char*, ns1__Timezone **, const char*);

#ifndef SOAP_TYPE_PointerTons1__DayOfWeek
#define SOAP_TYPE_PointerTons1__DayOfWeek (339)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__DayOfWeek(struct soap*, ns1__DayOfWeek *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__DayOfWeek(struct soap*, ns1__DayOfWeek *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__DayOfWeek(struct soap*, ns1__DayOfWeek *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__DayOfWeek(struct soap*, const char *, int, ns1__DayOfWeek *const*, const char *);
SOAP_FMAC3 ns1__DayOfWeek ** SOAP_FMAC4 soap_get_PointerTons1__DayOfWeek(struct soap*, ns1__DayOfWeek **, const char*, const char*);
SOAP_FMAC3 ns1__DayOfWeek ** SOAP_FMAC4 soap_in_PointerTons1__DayOfWeek(struct soap*, const char*, ns1__DayOfWeek **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Day
#define SOAP_TYPE_PointerTons1__Day (338)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Day(struct soap*, unsigned char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Day(struct soap*, unsigned char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Day(struct soap*, unsigned char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Day(struct soap*, const char *, int, unsigned char *const*, const char *);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_get_PointerTons1__Day(struct soap*, unsigned char **, const char*, const char*);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_in_PointerTons1__Day(struct soap*, const char*, unsigned char **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Minute
#define SOAP_TYPE_PointerTons1__Minute (337)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Minute(struct soap*, unsigned char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Minute(struct soap*, unsigned char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Minute(struct soap*, unsigned char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Minute(struct soap*, const char *, int, unsigned char *const*, const char *);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_get_PointerTons1__Minute(struct soap*, unsigned char **, const char*, const char*);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_in_PointerTons1__Minute(struct soap*, const char*, unsigned char **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Hour
#define SOAP_TYPE_PointerTons1__Hour (336)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Hour(struct soap*, unsigned char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Hour(struct soap*, unsigned char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Hour(struct soap*, unsigned char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Hour(struct soap*, const char *, int, unsigned char *const*, const char *);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_get_PointerTons1__Hour(struct soap*, unsigned char **, const char*, const char*);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_in_PointerTons1__Hour(struct soap*, const char*, unsigned char **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Month
#define SOAP_TYPE_PointerTons1__Month (335)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Month(struct soap*, unsigned char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Month(struct soap*, unsigned char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Month(struct soap*, unsigned char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Month(struct soap*, const char *, int, unsigned char *const*, const char *);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_get_PointerTons1__Month(struct soap*, unsigned char **, const char*, const char*);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_in_PointerTons1__Month(struct soap*, const char*, unsigned char **, const char*);

#ifndef SOAP_TYPE_PointerTons1__TimezoneComponent
#define SOAP_TYPE_PointerTons1__TimezoneComponent (334)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__TimezoneComponent(struct soap*, ns1__TimezoneComponent *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__TimezoneComponent(struct soap*, ns1__TimezoneComponent *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__TimezoneComponent(struct soap*, ns1__TimezoneComponent *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__TimezoneComponent(struct soap*, const char *, int, ns1__TimezoneComponent *const*, const char *);
SOAP_FMAC3 ns1__TimezoneComponent ** SOAP_FMAC4 soap_get_PointerTons1__TimezoneComponent(struct soap*, ns1__TimezoneComponent **, const char*, const char*);
SOAP_FMAC3 ns1__TimezoneComponent ** SOAP_FMAC4 soap_in_PointerTons1__TimezoneComponent(struct soap*, const char*, ns1__TimezoneComponent **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ProblemList
#define SOAP_TYPE_PointerTons1__ProblemList (333)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ProblemList(struct soap*, ns1__ProblemList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ProblemList(struct soap*, ns1__ProblemList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ProblemList(struct soap*, ns1__ProblemList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ProblemList(struct soap*, const char *, int, ns1__ProblemList *const*, const char *);
SOAP_FMAC3 ns1__ProblemList ** SOAP_FMAC4 soap_get_PointerTons1__ProblemList(struct soap*, ns1__ProblemList **, const char*, const char*);
SOAP_FMAC3 ns1__ProblemList ** SOAP_FMAC4 soap_in_PointerTons1__ProblemList(struct soap*, const char*, ns1__ProblemList **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup (332)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, const char *, int, std::vector<ns1__SettingsGroup * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__SettingsGroup * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__SettingsGroup * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, const char*, std::vector<ns1__SettingsGroup * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__SettingsGroup
#define SOAP_TYPE_PointerTons1__SettingsGroup (330)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__SettingsGroup(struct soap*, ns1__SettingsGroup *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__SettingsGroup(struct soap*, ns1__SettingsGroup *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__SettingsGroup(struct soap*, ns1__SettingsGroup *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__SettingsGroup(struct soap*, const char *, int, ns1__SettingsGroup *const*, const char *);
SOAP_FMAC3 ns1__SettingsGroup ** SOAP_FMAC4 soap_get_PointerTons1__SettingsGroup(struct soap*, ns1__SettingsGroup **, const char*, const char*);
SOAP_FMAC3 ns1__SettingsGroup ** SOAP_FMAC4 soap_in_PointerTons1__SettingsGroup(struct soap*, const char*, ns1__SettingsGroup **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ReturnNotification
#define SOAP_TYPE_PointerTons1__ReturnNotification (329)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ReturnNotification(struct soap*, ns1__ReturnNotification *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ReturnNotification(struct soap*, ns1__ReturnNotification *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ReturnNotification(struct soap*, ns1__ReturnNotification *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ReturnNotification(struct soap*, const char *, int, ns1__ReturnNotification *const*, const char *);
SOAP_FMAC3 ns1__ReturnNotification ** SOAP_FMAC4 soap_get_PointerTons1__ReturnNotification(struct soap*, ns1__ReturnNotification **, const char*, const char*);
SOAP_FMAC3 ns1__ReturnNotification ** SOAP_FMAC4 soap_in_PointerTons1__ReturnNotification(struct soap*, const char*, ns1__ReturnNotification **, const char*);

#ifndef SOAP_TYPE_PointerTons1__StatusTracking
#define SOAP_TYPE_PointerTons1__StatusTracking (328)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__StatusTracking(struct soap*, ns1__StatusTracking *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__StatusTracking(struct soap*, ns1__StatusTracking *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__StatusTracking(struct soap*, ns1__StatusTracking *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__StatusTracking(struct soap*, const char *, int, ns1__StatusTracking *const*, const char *);
SOAP_FMAC3 ns1__StatusTracking ** SOAP_FMAC4 soap_get_PointerTons1__StatusTracking(struct soap*, ns1__StatusTracking **, const char*, const char*);
SOAP_FMAC3 ns1__StatusTracking ** SOAP_FMAC4 soap_in_PointerTons1__StatusTracking(struct soap*, const char*, ns1__StatusTracking **, const char*);

#ifndef SOAP_TYPE_PointerTons1__SendOptionsRequestReply
#define SOAP_TYPE_PointerTons1__SendOptionsRequestReply (327)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__SendOptionsRequestReply(struct soap*, ns1__SendOptionsRequestReply *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__SendOptionsRequestReply(struct soap*, ns1__SendOptionsRequestReply *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__SendOptionsRequestReply(struct soap*, ns1__SendOptionsRequestReply *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__SendOptionsRequestReply(struct soap*, const char *, int, ns1__SendOptionsRequestReply *const*, const char *);
SOAP_FMAC3 ns1__SendOptionsRequestReply ** SOAP_FMAC4 soap_get_PointerTons1__SendOptionsRequestReply(struct soap*, ns1__SendOptionsRequestReply **, const char*, const char*);
SOAP_FMAC3 ns1__SendOptionsRequestReply ** SOAP_FMAC4 soap_in_PointerTons1__SendOptionsRequestReply(struct soap*, const char*, ns1__SendOptionsRequestReply **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ReturnNotificationOptions
#define SOAP_TYPE_PointerTons1__ReturnNotificationOptions (326)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ReturnNotificationOptions(struct soap*, ns1__ReturnNotificationOptions *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ReturnNotificationOptions(struct soap*, ns1__ReturnNotificationOptions *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ReturnNotificationOptions(struct soap*, ns1__ReturnNotificationOptions *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ReturnNotificationOptions(struct soap*, const char *, int, ns1__ReturnNotificationOptions *const*, const char *);
SOAP_FMAC3 ns1__ReturnNotificationOptions ** SOAP_FMAC4 soap_get_PointerTons1__ReturnNotificationOptions(struct soap*, ns1__ReturnNotificationOptions **, const char*, const char*);
SOAP_FMAC3 ns1__ReturnNotificationOptions ** SOAP_FMAC4 soap_in_PointerTons1__ReturnNotificationOptions(struct soap*, const char*, ns1__ReturnNotificationOptions **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfxsd__date
#define SOAP_TYPE_PointerTostd__vectorTemplateOfxsd__date (325)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfxsd__date(struct soap*, const char *, int, std::vector<char * >*const*, const char *);
SOAP_FMAC3 std::vector<char * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >**, const char*, const char*);
SOAP_FMAC3 std::vector<char * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfxsd__date(struct soap*, const char*, std::vector<char * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__DelegateeStatus
#define SOAP_TYPE_PointerTons1__DelegateeStatus (323)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__DelegateeStatus(struct soap*, ns1__DelegateeStatus *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__DelegateeStatus(struct soap*, ns1__DelegateeStatus *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__DelegateeStatus(struct soap*, ns1__DelegateeStatus *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__DelegateeStatus(struct soap*, const char *, int, ns1__DelegateeStatus *const*, const char *);
SOAP_FMAC3 ns1__DelegateeStatus ** SOAP_FMAC4 soap_get_PointerTons1__DelegateeStatus(struct soap*, ns1__DelegateeStatus **, const char*, const char*);
SOAP_FMAC3 ns1__DelegateeStatus ** SOAP_FMAC4 soap_in_PointerTons1__DelegateeStatus(struct soap*, const char*, ns1__DelegateeStatus **, const char*);

#ifndef SOAP_TYPE_PointerTons1__DelegatedStatus
#define SOAP_TYPE_PointerTons1__DelegatedStatus (322)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__DelegatedStatus(struct soap*, ns1__DelegatedStatus *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__DelegatedStatus(struct soap*, ns1__DelegatedStatus *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__DelegatedStatus(struct soap*, ns1__DelegatedStatus *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__DelegatedStatus(struct soap*, const char *, int, ns1__DelegatedStatus *const*, const char *);
SOAP_FMAC3 ns1__DelegatedStatus ** SOAP_FMAC4 soap_get_PointerTons1__DelegatedStatus(struct soap*, ns1__DelegatedStatus **, const char*, const char*);
SOAP_FMAC3 ns1__DelegatedStatus ** SOAP_FMAC4 soap_in_PointerTons1__DelegatedStatus(struct soap*, const char*, ns1__DelegatedStatus **, const char*);

#ifndef SOAP_TYPE_PointerTons1__CommentStatus
#define SOAP_TYPE_PointerTons1__CommentStatus (321)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__CommentStatus(struct soap*, ns1__CommentStatus *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__CommentStatus(struct soap*, ns1__CommentStatus *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__CommentStatus(struct soap*, ns1__CommentStatus *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__CommentStatus(struct soap*, const char *, int, ns1__CommentStatus *const*, const char *);
SOAP_FMAC3 ns1__CommentStatus ** SOAP_FMAC4 soap_get_PointerTons1__CommentStatus(struct soap*, ns1__CommentStatus **, const char*, const char*);
SOAP_FMAC3 ns1__CommentStatus ** SOAP_FMAC4 soap_in_PointerTons1__CommentStatus(struct soap*, const char*, ns1__CommentStatus **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Recipient
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Recipient (320)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__Recipient(struct soap*, const char *, int, std::vector<ns1__Recipient * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__Recipient * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Recipient * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__Recipient(struct soap*, const char*, std::vector<ns1__Recipient * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__Recipient
#define SOAP_TYPE_PointerTons1__Recipient (318)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Recipient(struct soap*, ns1__Recipient *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Recipient(struct soap*, ns1__Recipient *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Recipient(struct soap*, ns1__Recipient *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Recipient(struct soap*, const char *, int, ns1__Recipient *const*, const char *);
SOAP_FMAC3 ns1__Recipient ** SOAP_FMAC4 soap_get_PointerTons1__Recipient(struct soap*, ns1__Recipient **, const char*, const char*);
SOAP_FMAC3 ns1__Recipient ** SOAP_FMAC4 soap_in_PointerTons1__Recipient(struct soap*, const char*, ns1__Recipient **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Filter
#define SOAP_TYPE_PointerTons1__Filter (317)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Filter(struct soap*, ns1__Filter *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Filter(struct soap*, ns1__Filter *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Filter(struct soap*, ns1__Filter *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Filter(struct soap*, const char *, int, ns1__Filter *const*, const char *);
SOAP_FMAC3 ns1__Filter ** SOAP_FMAC4 soap_get_PointerTons1__Filter(struct soap*, ns1__Filter **, const char*, const char*);
SOAP_FMAC3 ns1__Filter ** SOAP_FMAC4 soap_in_PointerTons1__Filter(struct soap*, const char*, ns1__Filter **, const char*);

#ifndef SOAP_TYPE_PointerTons1__QueryTarget
#define SOAP_TYPE_PointerTons1__QueryTarget (316)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__QueryTarget(struct soap*, ns1__QueryTarget *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__QueryTarget(struct soap*, ns1__QueryTarget *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__QueryTarget(struct soap*, ns1__QueryTarget *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__QueryTarget(struct soap*, const char *, int, ns1__QueryTarget *const*, const char *);
SOAP_FMAC3 ns1__QueryTarget ** SOAP_FMAC4 soap_get_PointerTons1__QueryTarget(struct soap*, ns1__QueryTarget **, const char*, const char*);
SOAP_FMAC3 ns1__QueryTarget ** SOAP_FMAC4 soap_in_PointerTons1__QueryTarget(struct soap*, const char*, ns1__QueryTarget **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail (315)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, const char *, int, std::vector<ns1__NameAndEmail * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__NameAndEmail * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__NameAndEmail * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, const char*, std::vector<ns1__NameAndEmail * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__NameAndEmail
#define SOAP_TYPE_PointerTons1__NameAndEmail (313)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__NameAndEmail(struct soap*, ns1__NameAndEmail *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__NameAndEmail(struct soap*, ns1__NameAndEmail *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__NameAndEmail(struct soap*, ns1__NameAndEmail *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__NameAndEmail(struct soap*, const char *, int, ns1__NameAndEmail *const*, const char *);
SOAP_FMAC3 ns1__NameAndEmail ** SOAP_FMAC4 soap_get_PointerTons1__NameAndEmail(struct soap*, ns1__NameAndEmail **, const char*, const char*);
SOAP_FMAC3 ns1__NameAndEmail ** SOAP_FMAC4 soap_in_PointerTons1__NameAndEmail(struct soap*, const char*, ns1__NameAndEmail **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry (312)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, const char *, int, std::vector<ns1__ProblemEntry * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__ProblemEntry * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__ProblemEntry * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, const char*, std::vector<ns1__ProblemEntry * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__ProblemEntry
#define SOAP_TYPE_PointerTons1__ProblemEntry (310)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ProblemEntry(struct soap*, ns1__ProblemEntry *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ProblemEntry(struct soap*, ns1__ProblemEntry *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ProblemEntry(struct soap*, ns1__ProblemEntry *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ProblemEntry(struct soap*, const char *, int, ns1__ProblemEntry *const*, const char *);
SOAP_FMAC3 ns1__ProblemEntry ** SOAP_FMAC4 soap_get_PointerTons1__ProblemEntry(struct soap*, ns1__ProblemEntry **, const char*, const char*);
SOAP_FMAC3 ns1__ProblemEntry ** SOAP_FMAC4 soap_in_PointerTons1__ProblemEntry(struct soap*, const char*, ns1__ProblemEntry **, const char*);

#ifndef SOAP_TYPE_PointerToint
#define SOAP_TYPE_PointerToint (309)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerToint(struct soap*, int *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerToint(struct soap*, int *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerToint(struct soap*, int *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerToint(struct soap*, const char *, int, int *const*, const char *);
SOAP_FMAC3 int ** SOAP_FMAC4 soap_get_PointerToint(struct soap*, int **, const char*, const char*);
SOAP_FMAC3 int ** SOAP_FMAC4 soap_in_PointerToint(struct soap*, const char*, int **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress (308)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, const char *, int, std::vector<ns1__PostalAddress * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__PostalAddress * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__PostalAddress * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, const char*, std::vector<ns1__PostalAddress * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__PostalAddress
#define SOAP_TYPE_PointerTons1__PostalAddress (306)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__PostalAddress(struct soap*, ns1__PostalAddress *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__PostalAddress(struct soap*, ns1__PostalAddress *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__PostalAddress(struct soap*, ns1__PostalAddress *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__PostalAddress(struct soap*, const char *, int, ns1__PostalAddress *const*, const char *);
SOAP_FMAC3 ns1__PostalAddress ** SOAP_FMAC4 soap_get_PointerTons1__PostalAddress(struct soap*, ns1__PostalAddress **, const char*, const char*);
SOAP_FMAC3 ns1__PostalAddress ** SOAP_FMAC4 soap_in_PointerTons1__PostalAddress(struct soap*, const char*, ns1__PostalAddress **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber (305)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, const char *, int, std::vector<ns1__PhoneNumber * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__PhoneNumber * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__PhoneNumber * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, const char*, std::vector<ns1__PhoneNumber * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__PhoneNumber
#define SOAP_TYPE_PointerTons1__PhoneNumber (303)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__PhoneNumber(struct soap*, ns1__PhoneNumber *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__PhoneNumber(struct soap*, ns1__PhoneNumber *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__PhoneNumber(struct soap*, ns1__PhoneNumber *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__PhoneNumber(struct soap*, const char *, int, ns1__PhoneNumber *const*, const char *);
SOAP_FMAC3 ns1__PhoneNumber ** SOAP_FMAC4 soap_get_PointerTons1__PhoneNumber(struct soap*, ns1__PhoneNumber **, const char*, const char*);
SOAP_FMAC3 ns1__PhoneNumber ** SOAP_FMAC4 soap_in_PointerTons1__PhoneNumber(struct soap*, const char*, ns1__PhoneNumber **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemRef
#define SOAP_TYPE_PointerTons1__ItemRef (302)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemRef(struct soap*, ns1__ItemRef *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemRef(struct soap*, ns1__ItemRef *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemRef(struct soap*, ns1__ItemRef *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemRef(struct soap*, const char *, int, ns1__ItemRef *const*, const char *);
SOAP_FMAC3 ns1__ItemRef ** SOAP_FMAC4 soap_get_PointerTons1__ItemRef(struct soap*, ns1__ItemRef **, const char*, const char*);
SOAP_FMAC3 ns1__ItemRef ** SOAP_FMAC4 soap_in_PointerTons1__ItemRef(struct soap*, const char*, ns1__ItemRef **, const char*);

#ifndef SOAP_TYPE_PointerTons1__UUID
#define SOAP_TYPE_PointerTons1__UUID (301)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__UUID(struct soap*, std::string *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__UUID(struct soap*, std::string *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__UUID(struct soap*, std::string *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__UUID(struct soap*, const char *, int, std::string *const*, const char *);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_get_PointerTons1__UUID(struct soap*, std::string **, const char*, const char*);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_in_PointerTons1__UUID(struct soap*, const char*, std::string **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__MessagePart
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__MessagePart (300)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__MessagePart(struct soap*, const char *, int, std::vector<ns1__MessagePart * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__MessagePart * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__MessagePart * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__MessagePart(struct soap*, const char*, std::vector<ns1__MessagePart * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__MessagePart
#define SOAP_TYPE_PointerTons1__MessagePart (298)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__MessagePart(struct soap*, ns1__MessagePart *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__MessagePart(struct soap*, ns1__MessagePart *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__MessagePart(struct soap*, ns1__MessagePart *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__MessagePart(struct soap*, const char *, int, ns1__MessagePart *const*, const char *);
SOAP_FMAC3 ns1__MessagePart ** SOAP_FMAC4 soap_get_PointerTons1__MessagePart(struct soap*, ns1__MessagePart **, const char*, const char*);
SOAP_FMAC3 ns1__MessagePart ** SOAP_FMAC4 soap_in_PointerTons1__MessagePart(struct soap*, const char*, ns1__MessagePart **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Item
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Item (297)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__Item(struct soap*, const char *, int, std::vector<ns1__Item * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__Item * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Item * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__Item(struct soap*, const char*, std::vector<ns1__Item * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__Item
#define SOAP_TYPE_PointerTons1__Item (295)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Item(struct soap*, ns1__Item *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Item(struct soap*, ns1__Item *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Item(struct soap*, ns1__Item *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Item(struct soap*, const char *, int, ns1__Item *const*, const char *);
SOAP_FMAC3 ns1__Item ** SOAP_FMAC4 soap_get_PointerTons1__Item(struct soap*, ns1__Item **, const char*, const char*);
SOAP_FMAC3 ns1__Item ** SOAP_FMAC4 soap_in_PointerTons1__Item(struct soap*, const char*, ns1__Item **, const char*);

#ifndef SOAP_TYPE_PointerTons1__ItemChanges
#define SOAP_TYPE_PointerTons1__ItemChanges (294)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ItemChanges(struct soap*, ns1__ItemChanges *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ItemChanges(struct soap*, ns1__ItemChanges *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ItemChanges(struct soap*, ns1__ItemChanges *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ItemChanges(struct soap*, const char *, int, ns1__ItemChanges *const*, const char *);
SOAP_FMAC3 ns1__ItemChanges ** SOAP_FMAC4 soap_get_PointerTons1__ItemChanges(struct soap*, ns1__ItemChanges **, const char*, const char*);
SOAP_FMAC3 ns1__ItemChanges ** SOAP_FMAC4 soap_in_PointerTons1__ItemChanges(struct soap*, const char*, ns1__ItemChanges **, const char*);

#ifndef SOAP_TYPE_PointerTounsignedLong
#define SOAP_TYPE_PointerTounsignedLong (293)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTounsignedLong(struct soap*, unsigned long *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTounsignedLong(struct soap*, unsigned long *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTounsignedLong(struct soap*, unsigned long *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTounsignedLong(struct soap*, const char *, int, unsigned long *const*, const char *);
SOAP_FMAC3 unsigned long ** SOAP_FMAC4 soap_get_PointerTounsignedLong(struct soap*, unsigned long **, const char*, const char*);
SOAP_FMAC3 unsigned long ** SOAP_FMAC4 soap_in_PointerTounsignedLong(struct soap*, const char*, unsigned long **, const char*);

#ifndef SOAP_TYPE_PointerTons1__uid
#define SOAP_TYPE_PointerTons1__uid (292)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__uid(struct soap*, std::string *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__uid(struct soap*, std::string *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__uid(struct soap*, std::string *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__uid(struct soap*, const char *, int, std::string *const*, const char *);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_get_PointerTons1__uid(struct soap*, std::string **, const char*, const char*);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_in_PointerTons1__uid(struct soap*, const char*, std::string **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__ImAddress
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__ImAddress (291)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__ImAddress(struct soap*, const char *, int, std::vector<ns1__ImAddress * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__ImAddress * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__ImAddress * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__ImAddress(struct soap*, const char*, std::vector<ns1__ImAddress * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__ImAddress
#define SOAP_TYPE_PointerTons1__ImAddress (289)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__ImAddress(struct soap*, ns1__ImAddress *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__ImAddress(struct soap*, ns1__ImAddress *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__ImAddress(struct soap*, ns1__ImAddress *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__ImAddress(struct soap*, const char *, int, ns1__ImAddress *const*, const char *);
SOAP_FMAC3 ns1__ImAddress ** SOAP_FMAC4 soap_get_PointerTons1__ImAddress(struct soap*, ns1__ImAddress **, const char*, const char*);
SOAP_FMAC3 ns1__ImAddress ** SOAP_FMAC4 soap_in_PointerTons1__ImAddress(struct soap*, const char*, ns1__ImAddress **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__GroupMember
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__GroupMember (288)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__GroupMember(struct soap*, const char *, int, std::vector<ns1__GroupMember * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__GroupMember * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__GroupMember * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__GroupMember(struct soap*, const char*, std::vector<ns1__GroupMember * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__GroupMember
#define SOAP_TYPE_PointerTons1__GroupMember (286)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__GroupMember(struct soap*, ns1__GroupMember *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__GroupMember(struct soap*, ns1__GroupMember *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__GroupMember(struct soap*, ns1__GroupMember *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__GroupMember(struct soap*, const char *, int, ns1__GroupMember *const*, const char *);
SOAP_FMAC3 ns1__GroupMember ** SOAP_FMAC4 soap_get_PointerTons1__GroupMember(struct soap*, ns1__GroupMember **, const char*, const char*);
SOAP_FMAC3 ns1__GroupMember ** SOAP_FMAC4 soap_in_PointerTons1__GroupMember(struct soap*, const char*, ns1__GroupMember **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo (285)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, const char *, int, std::vector<ns1__FreeBusyInfo * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__FreeBusyInfo * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyInfo * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, const char*, std::vector<ns1__FreeBusyInfo * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__FreeBusyInfo
#define SOAP_TYPE_PointerTons1__FreeBusyInfo (283)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FreeBusyInfo(struct soap*, ns1__FreeBusyInfo *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FreeBusyInfo(struct soap*, ns1__FreeBusyInfo *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FreeBusyInfo(struct soap*, ns1__FreeBusyInfo *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FreeBusyInfo(struct soap*, const char *, int, ns1__FreeBusyInfo *const*, const char *);
SOAP_FMAC3 ns1__FreeBusyInfo ** SOAP_FMAC4 soap_get_PointerTons1__FreeBusyInfo(struct soap*, ns1__FreeBusyInfo **, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyInfo ** SOAP_FMAC4 soap_in_PointerTons1__FreeBusyInfo(struct soap*, const char*, ns1__FreeBusyInfo **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser (282)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, const char *, int, std::vector<ns1__FreeBusyUser * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__FreeBusyUser * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyUser * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, const char*, std::vector<ns1__FreeBusyUser * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__FreeBusyUser
#define SOAP_TYPE_PointerTons1__FreeBusyUser (280)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FreeBusyUser(struct soap*, ns1__FreeBusyUser *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FreeBusyUser(struct soap*, ns1__FreeBusyUser *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FreeBusyUser(struct soap*, ns1__FreeBusyUser *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FreeBusyUser(struct soap*, const char *, int, ns1__FreeBusyUser *const*, const char *);
SOAP_FMAC3 ns1__FreeBusyUser ** SOAP_FMAC4 soap_get_PointerTons1__FreeBusyUser(struct soap*, ns1__FreeBusyUser **, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyUser ** SOAP_FMAC4 soap_in_PointerTons1__FreeBusyUser(struct soap*, const char*, ns1__FreeBusyUser **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock (279)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, const char *, int, std::vector<ns1__FreeBusyBlock * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__FreeBusyBlock * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyBlock * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, const char*, std::vector<ns1__FreeBusyBlock * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__FreeBusyBlock
#define SOAP_TYPE_PointerTons1__FreeBusyBlock (277)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FreeBusyBlock(struct soap*, ns1__FreeBusyBlock *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FreeBusyBlock(struct soap*, ns1__FreeBusyBlock *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FreeBusyBlock(struct soap*, ns1__FreeBusyBlock *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FreeBusyBlock(struct soap*, const char *, int, ns1__FreeBusyBlock *const*, const char *);
SOAP_FMAC3 ns1__FreeBusyBlock ** SOAP_FMAC4 soap_get_PointerTons1__FreeBusyBlock(struct soap*, ns1__FreeBusyBlock **, const char*, const char*);
SOAP_FMAC3 ns1__FreeBusyBlock ** SOAP_FMAC4 soap_in_PointerTons1__FreeBusyBlock(struct soap*, const char*, ns1__FreeBusyBlock **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Folder
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Folder (276)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__Folder(struct soap*, const char *, int, std::vector<ns1__Folder * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__Folder * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Folder * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__Folder(struct soap*, const char*, std::vector<ns1__Folder * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__Folder
#define SOAP_TYPE_PointerTons1__Folder (274)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Folder(struct soap*, ns1__Folder *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Folder(struct soap*, ns1__Folder *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Folder(struct soap*, ns1__Folder *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Folder(struct soap*, const char *, int, ns1__Folder *const*, const char *);
SOAP_FMAC3 ns1__Folder ** SOAP_FMAC4 soap_get_PointerTons1__Folder(struct soap*, ns1__Folder **, const char*, const char*);
SOAP_FMAC3 ns1__Folder ** SOAP_FMAC4 soap_in_PointerTons1__Folder(struct soap*, const char*, ns1__Folder **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry (273)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, const char *, int, std::vector<ns1__FolderACLEntry * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__FolderACLEntry * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FolderACLEntry * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, const char*, std::vector<ns1__FolderACLEntry * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__FolderACLEntry
#define SOAP_TYPE_PointerTons1__FolderACLEntry (271)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FolderACLEntry(struct soap*, ns1__FolderACLEntry *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FolderACLEntry(struct soap*, ns1__FolderACLEntry *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FolderACLEntry(struct soap*, ns1__FolderACLEntry *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FolderACLEntry(struct soap*, const char *, int, ns1__FolderACLEntry *const*, const char *);
SOAP_FMAC3 ns1__FolderACLEntry ** SOAP_FMAC4 soap_get_PointerTons1__FolderACLEntry(struct soap*, ns1__FolderACLEntry **, const char*, const char*);
SOAP_FMAC3 ns1__FolderACLEntry ** SOAP_FMAC4 soap_in_PointerTons1__FolderACLEntry(struct soap*, const char*, ns1__FolderACLEntry **, const char*);

#ifndef SOAP_TYPE_PointerTons1__FilterElement
#define SOAP_TYPE_PointerTons1__FilterElement (270)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__FilterElement(struct soap*, ns1__FilterElement *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__FilterElement(struct soap*, ns1__FilterElement *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__FilterElement(struct soap*, ns1__FilterElement *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__FilterElement(struct soap*, const char *, int, ns1__FilterElement *const*, const char *);
SOAP_FMAC3 ns1__FilterElement ** SOAP_FMAC4 soap_get_PointerTons1__FilterElement(struct soap*, ns1__FilterElement **, const char*, const char*);
SOAP_FMAC3 ns1__FilterElement ** SOAP_FMAC4 soap_in_PointerTons1__FilterElement(struct soap*, const char*, ns1__FilterElement **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfstd__string
#define SOAP_TYPE_PointerTostd__vectorTemplateOfstd__string (269)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfstd__string(struct soap*, const char *, int, std::vector<std::string >*const*, const char *);
SOAP_FMAC3 std::vector<std::string >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >**, const char*, const char*);
SOAP_FMAC3 std::vector<std::string >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfstd__string(struct soap*, const char*, std::vector<std::string >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__SendOptions
#define SOAP_TYPE_PointerTons1__SendOptions (267)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__SendOptions(struct soap*, ns1__SendOptions *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__SendOptions(struct soap*, ns1__SendOptions *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__SendOptions(struct soap*, ns1__SendOptions *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__SendOptions(struct soap*, const char *, int, ns1__SendOptions *const*, const char *);
SOAP_FMAC3 ns1__SendOptions ** SOAP_FMAC4 soap_get_PointerTons1__SendOptions(struct soap*, ns1__SendOptions **, const char*, const char*);
SOAP_FMAC3 ns1__SendOptions ** SOAP_FMAC4 soap_in_PointerTons1__SendOptions(struct soap*, const char*, ns1__SendOptions **, const char*);

#ifndef SOAP_TYPE_PointerTons1__RecipientList
#define SOAP_TYPE_PointerTons1__RecipientList (266)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__RecipientList(struct soap*, ns1__RecipientList *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__RecipientList(struct soap*, ns1__RecipientList *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__RecipientList(struct soap*, ns1__RecipientList *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__RecipientList(struct soap*, const char *, int, ns1__RecipientList *const*, const char *);
SOAP_FMAC3 ns1__RecipientList ** SOAP_FMAC4 soap_get_PointerTons1__RecipientList(struct soap*, ns1__RecipientList **, const char*, const char*);
SOAP_FMAC3 ns1__RecipientList ** SOAP_FMAC4 soap_in_PointerTons1__RecipientList(struct soap*, const char*, ns1__RecipientList **, const char*);

#ifndef SOAP_TYPE_PointerTons1__From
#define SOAP_TYPE_PointerTons1__From (265)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__From(struct soap*, ns1__From *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__From(struct soap*, ns1__From *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__From(struct soap*, ns1__From *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__From(struct soap*, const char *, int, ns1__From *const*, const char *);
SOAP_FMAC3 ns1__From ** SOAP_FMAC4 soap_get_PointerTons1__From(struct soap*, ns1__From **, const char*, const char*);
SOAP_FMAC3 ns1__From ** SOAP_FMAC4 soap_in_PointerTons1__From(struct soap*, const char*, ns1__From **, const char*);

#ifndef SOAP_TYPE_PointerTons1__Items
#define SOAP_TYPE_PointerTons1__Items (264)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Items(struct soap*, ns1__Items *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Items(struct soap*, ns1__Items *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Items(struct soap*, ns1__Items *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Items(struct soap*, const char *, int, ns1__Items *const*, const char *);
SOAP_FMAC3 ns1__Items ** SOAP_FMAC4 soap_get_PointerTons1__Items(struct soap*, ns1__Items **, const char*, const char*);
SOAP_FMAC3 ns1__Items ** SOAP_FMAC4 soap_in_PointerTons1__Items(struct soap*, const char*, ns1__Items **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Custom
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Custom (263)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__Custom(struct soap*, const char *, int, std::vector<ns1__Custom * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__Custom * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Custom * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__Custom(struct soap*, const char*, std::vector<ns1__Custom * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__Custom
#define SOAP_TYPE_PointerTons1__Custom (261)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Custom(struct soap*, ns1__Custom *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Custom(struct soap*, ns1__Custom *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Custom(struct soap*, ns1__Custom *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Custom(struct soap*, const char *, int, ns1__Custom *const*, const char *);
SOAP_FMAC3 ns1__Custom ** SOAP_FMAC4 soap_get_PointerTons1__Custom(struct soap*, ns1__Custom **, const char*, const char*);
SOAP_FMAC3 ns1__Custom ** SOAP_FMAC4 soap_in_PointerTons1__Custom(struct soap*, const char*, ns1__Custom **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfns1__uid
#define SOAP_TYPE_PointerTostd__vectorTemplateOfns1__uid (260)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfns1__uid(struct soap*, const char *, int, std::vector<std::string >*const*, const char *);
SOAP_FMAC3 std::vector<std::string >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >**, const char*, const char*);
SOAP_FMAC3 std::vector<std::string >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfns1__uid(struct soap*, const char*, std::vector<std::string >**, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Category
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__Category (258)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__Category(struct soap*, const char *, int, std::vector<ns1__Category * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__Category * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Category * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__Category(struct soap*, const char*, std::vector<ns1__Category * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__Category
#define SOAP_TYPE_PointerTons1__Category (256)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__Category(struct soap*, ns1__Category *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__Category(struct soap*, ns1__Category *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__Category(struct soap*, ns1__Category *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__Category(struct soap*, const char *, int, ns1__Category *const*, const char *);
SOAP_FMAC3 ns1__Category ** SOAP_FMAC4 soap_get_PointerTons1__Category(struct soap*, ns1__Category **, const char*, const char*);
SOAP_FMAC3 ns1__Category ** SOAP_FMAC4 soap_in_PointerTons1__Category(struct soap*, const char*, ns1__Category **, const char*);

#ifndef SOAP_TYPE_PointerToxsd__base64Binary
#define SOAP_TYPE_PointerToxsd__base64Binary (255)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerToxsd__base64Binary(struct soap*, xsd__base64Binary *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerToxsd__base64Binary(struct soap*, xsd__base64Binary *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerToxsd__base64Binary(struct soap*, xsd__base64Binary *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerToxsd__base64Binary(struct soap*, const char *, int, xsd__base64Binary *const*, const char *);
SOAP_FMAC3 xsd__base64Binary ** SOAP_FMAC4 soap_get_PointerToxsd__base64Binary(struct soap*, xsd__base64Binary **, const char*, const char*);
SOAP_FMAC3 xsd__base64Binary ** SOAP_FMAC4 soap_in_PointerToxsd__base64Binary(struct soap*, const char*, xsd__base64Binary **, const char*);

#ifndef SOAP_TYPE_PointerTostd__string
#define SOAP_TYPE_PointerTostd__string (252)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__string(struct soap*, std::string *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__string(struct soap*, std::string *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__string(struct soap*, std::string *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__string(struct soap*, const char *, int, std::string *const*, const char *);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_get_PointerTostd__string(struct soap*, std::string **, const char*, const char*);
SOAP_FMAC3 std::string ** SOAP_FMAC4 soap_in_PointerTostd__string(struct soap*, const char*, std::string **, const char*);

#ifndef SOAP_TYPE_PointerTons1__AttachmentID
#define SOAP_TYPE_PointerTons1__AttachmentID (251)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AttachmentID(struct soap*, ns1__AttachmentID *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AttachmentID(struct soap*, ns1__AttachmentID *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AttachmentID(struct soap*, ns1__AttachmentID *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AttachmentID(struct soap*, const char *, int, ns1__AttachmentID *const*, const char *);
SOAP_FMAC3 ns1__AttachmentID ** SOAP_FMAC4 soap_get_PointerTons1__AttachmentID(struct soap*, ns1__AttachmentID **, const char*, const char*);
SOAP_FMAC3 ns1__AttachmentID ** SOAP_FMAC4 soap_in_PointerTons1__AttachmentID(struct soap*, const char*, ns1__AttachmentID **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo (250)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, const char *, int, std::vector<ns1__AttachmentItemInfo * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__AttachmentItemInfo * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AttachmentItemInfo * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, const char*, std::vector<ns1__AttachmentItemInfo * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__AttachmentItemInfo
#define SOAP_TYPE_PointerTons1__AttachmentItemInfo (248)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AttachmentItemInfo(struct soap*, ns1__AttachmentItemInfo *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AttachmentItemInfo(struct soap*, ns1__AttachmentItemInfo *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AttachmentItemInfo(struct soap*, ns1__AttachmentItemInfo *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AttachmentItemInfo(struct soap*, const char *, int, ns1__AttachmentItemInfo *const*, const char *);
SOAP_FMAC3 ns1__AttachmentItemInfo ** SOAP_FMAC4 soap_get_PointerTons1__AttachmentItemInfo(struct soap*, ns1__AttachmentItemInfo **, const char*, const char*);
SOAP_FMAC3 ns1__AttachmentItemInfo ** SOAP_FMAC4 soap_in_PointerTons1__AttachmentItemInfo(struct soap*, const char*, ns1__AttachmentItemInfo **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AddressBook
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AddressBook (247)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__AddressBook(struct soap*, const char *, int, std::vector<ns1__AddressBook * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__AddressBook * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AddressBook * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__AddressBook(struct soap*, const char*, std::vector<ns1__AddressBook * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__AddressBook
#define SOAP_TYPE_PointerTons1__AddressBook (245)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AddressBook(struct soap*, ns1__AddressBook *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AddressBook(struct soap*, ns1__AddressBook *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AddressBook(struct soap*, ns1__AddressBook *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AddressBook(struct soap*, const char *, int, ns1__AddressBook *const*, const char *);
SOAP_FMAC3 ns1__AddressBook ** SOAP_FMAC4 soap_get_PointerTons1__AddressBook(struct soap*, ns1__AddressBook **, const char*, const char*);
SOAP_FMAC3 ns1__AddressBook ** SOAP_FMAC4 soap_in_PointerTons1__AddressBook(struct soap*, const char*, ns1__AddressBook **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry (244)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, const char *, int, std::vector<ns1__AccessRightEntry * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__AccessRightEntry * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AccessRightEntry * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, const char*, std::vector<ns1__AccessRightEntry * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__AccessRightEntry
#define SOAP_TYPE_PointerTons1__AccessRightEntry (242)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AccessRightEntry(struct soap*, ns1__AccessRightEntry *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AccessRightEntry(struct soap*, ns1__AccessRightEntry *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AccessRightEntry(struct soap*, ns1__AccessRightEntry *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AccessRightEntry(struct soap*, const char *, int, ns1__AccessRightEntry *const*, const char *);
SOAP_FMAC3 ns1__AccessRightEntry ** SOAP_FMAC4 soap_get_PointerTons1__AccessRightEntry(struct soap*, ns1__AccessRightEntry **, const char*, const char*);
SOAP_FMAC3 ns1__AccessRightEntry ** SOAP_FMAC4 soap_in_PointerTons1__AccessRightEntry(struct soap*, const char*, ns1__AccessRightEntry **, const char*);

#ifndef SOAP_TYPE_PointerTobool
#define SOAP_TYPE_PointerTobool (241)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTobool(struct soap*, bool *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTobool(struct soap*, bool *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTobool(struct soap*, bool *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTobool(struct soap*, const char *, int, bool *const*, const char *);
SOAP_FMAC3 bool ** SOAP_FMAC4 soap_get_PointerTobool(struct soap*, bool **, const char*, const char*);
SOAP_FMAC3 bool ** SOAP_FMAC4 soap_in_PointerTobool(struct soap*, const char*, bool **, const char*);

#ifndef SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry
#define SOAP_TYPE_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry (239)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >*const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >*const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >*const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, const char *, int, std::vector<ns1__AccessControlListEntry * >*const*, const char *);
SOAP_FMAC3 std::vector<ns1__AccessControlListEntry * >** SOAP_FMAC4 soap_get_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >**, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AccessControlListEntry * >** SOAP_FMAC4 soap_in_PointerTostd__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, const char*, std::vector<ns1__AccessControlListEntry * >**, const char*);

#ifndef SOAP_TYPE_PointerTons1__AccessControlListEntry
#define SOAP_TYPE_PointerTons1__AccessControlListEntry (237)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTons1__AccessControlListEntry(struct soap*, ns1__AccessControlListEntry *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTons1__AccessControlListEntry(struct soap*, ns1__AccessControlListEntry *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTons1__AccessControlListEntry(struct soap*, ns1__AccessControlListEntry *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTons1__AccessControlListEntry(struct soap*, const char *, int, ns1__AccessControlListEntry *const*, const char *);
SOAP_FMAC3 ns1__AccessControlListEntry ** SOAP_FMAC4 soap_get_PointerTons1__AccessControlListEntry(struct soap*, ns1__AccessControlListEntry **, const char*, const char*);
SOAP_FMAC3 ns1__AccessControlListEntry ** SOAP_FMAC4 soap_in_PointerTons1__AccessControlListEntry(struct soap*, const char*, ns1__AccessControlListEntry **, const char*);

#ifndef SOAP_TYPE_xsd__language
#define SOAP_TYPE_xsd__language (20)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_xsd__language(struct soap*, char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_xsd__language(struct soap*, char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_xsd__language(struct soap*, char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_xsd__language(struct soap*, const char*, int, char*const*, const char*);
SOAP_FMAC3 char ** SOAP_FMAC4 soap_get_xsd__language(struct soap*, char **, const char*, const char*);
SOAP_FMAC3 char * * SOAP_FMAC4 soap_in_xsd__language(struct soap*, const char*, char **, const char*);

#ifndef SOAP_TYPE_xsd__decimal
#define SOAP_TYPE_xsd__decimal (19)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_xsd__decimal(struct soap*, char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_xsd__decimal(struct soap*, char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_xsd__decimal(struct soap*, char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_xsd__decimal(struct soap*, const char*, int, char*const*, const char*);
SOAP_FMAC3 char ** SOAP_FMAC4 soap_get_xsd__decimal(struct soap*, char **, const char*, const char*);
SOAP_FMAC3 char * * SOAP_FMAC4 soap_in_xsd__decimal(struct soap*, const char*, char **, const char*);

#ifndef SOAP_TYPE_xsd__date
#define SOAP_TYPE_xsd__date (18)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_xsd__date(struct soap*, char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_xsd__date(struct soap*, char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_xsd__date(struct soap*, char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_xsd__date(struct soap*, const char*, int, char*const*, const char*);
SOAP_FMAC3 char ** SOAP_FMAC4 soap_get_xsd__date(struct soap*, char **, const char*, const char*);
SOAP_FMAC3 char * * SOAP_FMAC4 soap_in_xsd__date(struct soap*, const char*, char **, const char*);

#ifndef SOAP_TYPE_xsd__bool
#define SOAP_TYPE_xsd__bool (17)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_xsd__bool(struct soap*, char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_xsd__bool(struct soap*, char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_xsd__bool(struct soap*, char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_xsd__bool(struct soap*, const char*, int, char*const*, const char*);
SOAP_FMAC3 char ** SOAP_FMAC4 soap_get_xsd__bool(struct soap*, char **, const char*, const char*);
SOAP_FMAC3 char * * SOAP_FMAC4 soap_in_xsd__bool(struct soap*, const char*, char **, const char*);

#ifndef SOAP_TYPE_PointerTounsignedByte
#define SOAP_TYPE_PointerTounsignedByte (14)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_PointerTounsignedByte(struct soap*, unsigned char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_PointerTounsignedByte(struct soap*, unsigned char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_PointerTounsignedByte(struct soap*, unsigned char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_PointerTounsignedByte(struct soap*, const char *, int, unsigned char *const*, const char *);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_get_PointerTounsignedByte(struct soap*, unsigned char **, const char*, const char*);
SOAP_FMAC3 unsigned char ** SOAP_FMAC4 soap_in_PointerTounsignedByte(struct soap*, const char*, unsigned char **, const char*);

#ifndef SOAP_TYPE_xsd__anyURI
#define SOAP_TYPE_xsd__anyURI (10)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_xsd__anyURI(struct soap*, char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_xsd__anyURI(struct soap*, char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_xsd__anyURI(struct soap*, char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_xsd__anyURI(struct soap*, const char*, int, char*const*, const char*);
SOAP_FMAC3 char ** SOAP_FMAC4 soap_get_xsd__anyURI(struct soap*, char **, const char*, const char*);
SOAP_FMAC3 char * * SOAP_FMAC4 soap_in_xsd__anyURI(struct soap*, const char*, char **, const char*);

#ifndef SOAP_TYPE__QName
#define SOAP_TYPE__QName (5)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize__QName(struct soap*, char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark__QName(struct soap*, char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put__QName(struct soap*, char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out__QName(struct soap*, const char*, int, char*const*, const char*);
SOAP_FMAC3 char ** SOAP_FMAC4 soap_get__QName(struct soap*, char **, const char*, const char*);
SOAP_FMAC3 char * * SOAP_FMAC4 soap_in__QName(struct soap*, const char*, char **, const char*);

#ifndef SOAP_TYPE_string
#define SOAP_TYPE_string (3)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_string(struct soap*, char *const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_string(struct soap*, char *const*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_string(struct soap*, char *const*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_string(struct soap*, const char*, int, char*const*, const char*);
SOAP_FMAC3 char ** SOAP_FMAC4 soap_get_string(struct soap*, char **, const char*, const char*);
SOAP_FMAC3 char * * SOAP_FMAC4 soap_in_string(struct soap*, const char*, char **, const char*);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__FilterElement
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__FilterElement (373)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, const std::vector<ns1__FilterElement * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, const std::vector<ns1__FilterElement * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, const char*, int, const std::vector<ns1__FilterElement * >*, const char*);
SOAP_FMAC3 std::vector<ns1__FilterElement * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FilterElement * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, const char*, std::vector<ns1__FilterElement * >*, const char*);

SOAP_FMAC5 std::vector<ns1__FilterElement * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, std::vector<ns1__FilterElement * >*);
SOAP_FMAC5 std::vector<ns1__FilterElement * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__FilterElement(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__ContainerRef
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__ContainerRef (370)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, const std::vector<ns1__ContainerRef * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, const std::vector<ns1__ContainerRef * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, const char*, int, const std::vector<ns1__ContainerRef * >*, const char*);
SOAP_FMAC3 std::vector<ns1__ContainerRef * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__ContainerRef * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, const char*, std::vector<ns1__ContainerRef * >*, const char*);

SOAP_FMAC5 std::vector<ns1__ContainerRef * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, std::vector<ns1__ContainerRef * >*);
SOAP_FMAC5 std::vector<ns1__ContainerRef * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__ContainerRef(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__Timezone
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__Timezone (341)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, const std::vector<ns1__Timezone * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, const std::vector<ns1__Timezone * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, const char*, int, const std::vector<ns1__Timezone * >*, const char*);
SOAP_FMAC3 std::vector<ns1__Timezone * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Timezone * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, const char*, std::vector<ns1__Timezone * >*, const char*);

SOAP_FMAC5 std::vector<ns1__Timezone * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, std::vector<ns1__Timezone * >*);
SOAP_FMAC5 std::vector<ns1__Timezone * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__Timezone(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__SettingsGroup
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__SettingsGroup (331)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, const std::vector<ns1__SettingsGroup * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, const std::vector<ns1__SettingsGroup * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, const char*, int, const std::vector<ns1__SettingsGroup * >*, const char*);
SOAP_FMAC3 std::vector<ns1__SettingsGroup * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__SettingsGroup * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, const char*, std::vector<ns1__SettingsGroup * >*, const char*);

SOAP_FMAC5 std::vector<ns1__SettingsGroup * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, std::vector<ns1__SettingsGroup * >*);
SOAP_FMAC5 std::vector<ns1__SettingsGroup * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__SettingsGroup(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfxsd__date
#define SOAP_TYPE_std__vectorTemplateOfxsd__date (324)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfxsd__date(struct soap*, const std::vector<char * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfxsd__date(struct soap*, const std::vector<char * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfxsd__date(struct soap*, const char*, int, const std::vector<char * >*, const char*);
SOAP_FMAC3 std::vector<char * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >*, const char*, const char*);
SOAP_FMAC3 std::vector<char * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfxsd__date(struct soap*, const char*, std::vector<char * >*, const char*);

SOAP_FMAC5 std::vector<char * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfxsd__date(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfxsd__date(struct soap*, std::vector<char * >*);
SOAP_FMAC5 std::vector<char * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfxsd__date(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfxsd__date(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__Recipient
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__Recipient (319)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, const std::vector<ns1__Recipient * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, const std::vector<ns1__Recipient * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, const char*, int, const std::vector<ns1__Recipient * >*, const char*);
SOAP_FMAC3 std::vector<ns1__Recipient * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Recipient * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, const char*, std::vector<ns1__Recipient * >*, const char*);

SOAP_FMAC5 std::vector<ns1__Recipient * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, std::vector<ns1__Recipient * >*);
SOAP_FMAC5 std::vector<ns1__Recipient * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__Recipient(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__NameAndEmail
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__NameAndEmail (314)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, const std::vector<ns1__NameAndEmail * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, const std::vector<ns1__NameAndEmail * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, const char*, int, const std::vector<ns1__NameAndEmail * >*, const char*);
SOAP_FMAC3 std::vector<ns1__NameAndEmail * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__NameAndEmail * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, const char*, std::vector<ns1__NameAndEmail * >*, const char*);

SOAP_FMAC5 std::vector<ns1__NameAndEmail * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, std::vector<ns1__NameAndEmail * >*);
SOAP_FMAC5 std::vector<ns1__NameAndEmail * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__NameAndEmail(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__ProblemEntry
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__ProblemEntry (311)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, const std::vector<ns1__ProblemEntry * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, const std::vector<ns1__ProblemEntry * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, const char*, int, const std::vector<ns1__ProblemEntry * >*, const char*);
SOAP_FMAC3 std::vector<ns1__ProblemEntry * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__ProblemEntry * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, const char*, std::vector<ns1__ProblemEntry * >*, const char*);

SOAP_FMAC5 std::vector<ns1__ProblemEntry * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, std::vector<ns1__ProblemEntry * >*);
SOAP_FMAC5 std::vector<ns1__ProblemEntry * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__ProblemEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__PostalAddress
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__PostalAddress (307)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, const std::vector<ns1__PostalAddress * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, const std::vector<ns1__PostalAddress * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, const char*, int, const std::vector<ns1__PostalAddress * >*, const char*);
SOAP_FMAC3 std::vector<ns1__PostalAddress * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__PostalAddress * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, const char*, std::vector<ns1__PostalAddress * >*, const char*);

SOAP_FMAC5 std::vector<ns1__PostalAddress * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, std::vector<ns1__PostalAddress * >*);
SOAP_FMAC5 std::vector<ns1__PostalAddress * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__PostalAddress(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__PhoneNumber
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__PhoneNumber (304)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, const std::vector<ns1__PhoneNumber * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, const std::vector<ns1__PhoneNumber * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, const char*, int, const std::vector<ns1__PhoneNumber * >*, const char*);
SOAP_FMAC3 std::vector<ns1__PhoneNumber * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__PhoneNumber * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, const char*, std::vector<ns1__PhoneNumber * >*, const char*);

SOAP_FMAC5 std::vector<ns1__PhoneNumber * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, std::vector<ns1__PhoneNumber * >*);
SOAP_FMAC5 std::vector<ns1__PhoneNumber * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__PhoneNumber(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__MessagePart
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__MessagePart (299)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, const std::vector<ns1__MessagePart * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, const std::vector<ns1__MessagePart * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, const char*, int, const std::vector<ns1__MessagePart * >*, const char*);
SOAP_FMAC3 std::vector<ns1__MessagePart * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__MessagePart * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, const char*, std::vector<ns1__MessagePart * >*, const char*);

SOAP_FMAC5 std::vector<ns1__MessagePart * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, std::vector<ns1__MessagePart * >*);
SOAP_FMAC5 std::vector<ns1__MessagePart * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__MessagePart(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__Item
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__Item (296)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__Item(struct soap*, const std::vector<ns1__Item * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__Item(struct soap*, const std::vector<ns1__Item * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__Item(struct soap*, const char*, int, const std::vector<ns1__Item * >*, const char*);
SOAP_FMAC3 std::vector<ns1__Item * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Item * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__Item(struct soap*, const char*, std::vector<ns1__Item * >*, const char*);

SOAP_FMAC5 std::vector<ns1__Item * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__Item(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__Item(struct soap*, std::vector<ns1__Item * >*);
SOAP_FMAC5 std::vector<ns1__Item * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__Item(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__Item(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__ImAddress
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__ImAddress (290)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, const std::vector<ns1__ImAddress * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, const std::vector<ns1__ImAddress * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, const char*, int, const std::vector<ns1__ImAddress * >*, const char*);
SOAP_FMAC3 std::vector<ns1__ImAddress * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__ImAddress * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, const char*, std::vector<ns1__ImAddress * >*, const char*);

SOAP_FMAC5 std::vector<ns1__ImAddress * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, std::vector<ns1__ImAddress * >*);
SOAP_FMAC5 std::vector<ns1__ImAddress * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__ImAddress(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__GroupMember
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__GroupMember (287)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, const std::vector<ns1__GroupMember * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, const std::vector<ns1__GroupMember * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, const char*, int, const std::vector<ns1__GroupMember * >*, const char*);
SOAP_FMAC3 std::vector<ns1__GroupMember * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__GroupMember * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, const char*, std::vector<ns1__GroupMember * >*, const char*);

SOAP_FMAC5 std::vector<ns1__GroupMember * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, std::vector<ns1__GroupMember * >*);
SOAP_FMAC5 std::vector<ns1__GroupMember * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__GroupMember(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__FreeBusyInfo
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__FreeBusyInfo (284)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, const std::vector<ns1__FreeBusyInfo * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, const std::vector<ns1__FreeBusyInfo * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, const char*, int, const std::vector<ns1__FreeBusyInfo * >*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyInfo * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyInfo * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, const char*, std::vector<ns1__FreeBusyInfo * >*, const char*);

SOAP_FMAC5 std::vector<ns1__FreeBusyInfo * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, std::vector<ns1__FreeBusyInfo * >*);
SOAP_FMAC5 std::vector<ns1__FreeBusyInfo * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__FreeBusyInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__FreeBusyUser
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__FreeBusyUser (281)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, const std::vector<ns1__FreeBusyUser * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, const std::vector<ns1__FreeBusyUser * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, const char*, int, const std::vector<ns1__FreeBusyUser * >*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyUser * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyUser * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, const char*, std::vector<ns1__FreeBusyUser * >*, const char*);

SOAP_FMAC5 std::vector<ns1__FreeBusyUser * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, std::vector<ns1__FreeBusyUser * >*);
SOAP_FMAC5 std::vector<ns1__FreeBusyUser * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__FreeBusyUser(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__FreeBusyBlock
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__FreeBusyBlock (278)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, const std::vector<ns1__FreeBusyBlock * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, const std::vector<ns1__FreeBusyBlock * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, const char*, int, const std::vector<ns1__FreeBusyBlock * >*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyBlock * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FreeBusyBlock * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, const char*, std::vector<ns1__FreeBusyBlock * >*, const char*);

SOAP_FMAC5 std::vector<ns1__FreeBusyBlock * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, std::vector<ns1__FreeBusyBlock * >*);
SOAP_FMAC5 std::vector<ns1__FreeBusyBlock * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__FreeBusyBlock(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__Folder
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__Folder (275)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__Folder(struct soap*, const std::vector<ns1__Folder * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__Folder(struct soap*, const std::vector<ns1__Folder * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__Folder(struct soap*, const char*, int, const std::vector<ns1__Folder * >*, const char*);
SOAP_FMAC3 std::vector<ns1__Folder * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Folder * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__Folder(struct soap*, const char*, std::vector<ns1__Folder * >*, const char*);

SOAP_FMAC5 std::vector<ns1__Folder * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__Folder(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__Folder(struct soap*, std::vector<ns1__Folder * >*);
SOAP_FMAC5 std::vector<ns1__Folder * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__Folder(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__Folder(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__FolderACLEntry
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__FolderACLEntry (272)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, const std::vector<ns1__FolderACLEntry * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, const std::vector<ns1__FolderACLEntry * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, const char*, int, const std::vector<ns1__FolderACLEntry * >*, const char*);
SOAP_FMAC3 std::vector<ns1__FolderACLEntry * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__FolderACLEntry * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, const char*, std::vector<ns1__FolderACLEntry * >*, const char*);

SOAP_FMAC5 std::vector<ns1__FolderACLEntry * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, std::vector<ns1__FolderACLEntry * >*);
SOAP_FMAC5 std::vector<ns1__FolderACLEntry * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__FolderACLEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfstd__string
#define SOAP_TYPE_std__vectorTemplateOfstd__string (268)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfstd__string(struct soap*, const std::vector<std::string >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfstd__string(struct soap*, const std::vector<std::string >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfstd__string(struct soap*, const char*, int, const std::vector<std::string >*, const char*);
SOAP_FMAC3 std::vector<std::string >* SOAP_FMAC4 soap_get_std__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >*, const char*, const char*);
SOAP_FMAC3 std::vector<std::string >* SOAP_FMAC4 soap_in_std__vectorTemplateOfstd__string(struct soap*, const char*, std::vector<std::string >*, const char*);

SOAP_FMAC5 std::vector<std::string > * SOAP_FMAC6 soap_new_std__vectorTemplateOfstd__string(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfstd__string(struct soap*, std::vector<std::string >*);
SOAP_FMAC5 std::vector<std::string > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfstd__string(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfstd__string(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__Custom
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__Custom (262)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__Custom(struct soap*, const std::vector<ns1__Custom * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__Custom(struct soap*, const std::vector<ns1__Custom * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__Custom(struct soap*, const char*, int, const std::vector<ns1__Custom * >*, const char*);
SOAP_FMAC3 std::vector<ns1__Custom * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Custom * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__Custom(struct soap*, const char*, std::vector<ns1__Custom * >*, const char*);

SOAP_FMAC5 std::vector<ns1__Custom * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__Custom(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__Custom(struct soap*, std::vector<ns1__Custom * >*);
SOAP_FMAC5 std::vector<ns1__Custom * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__Custom(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__Custom(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfns1__uid
#define SOAP_TYPE_std__vectorTemplateOfns1__uid (259)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfns1__uid(struct soap*, const std::vector<std::string >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfns1__uid(struct soap*, const std::vector<std::string >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfns1__uid(struct soap*, const char*, int, const std::vector<std::string >*, const char*);
SOAP_FMAC3 std::vector<std::string >* SOAP_FMAC4 soap_get_std__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >*, const char*, const char*);
SOAP_FMAC3 std::vector<std::string >* SOAP_FMAC4 soap_in_std__vectorTemplateOfns1__uid(struct soap*, const char*, std::vector<std::string >*, const char*);

SOAP_FMAC5 std::vector<std::string > * SOAP_FMAC6 soap_new_std__vectorTemplateOfns1__uid(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfns1__uid(struct soap*, std::vector<std::string >*);
SOAP_FMAC5 std::vector<std::string > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfns1__uid(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfns1__uid(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__Category
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__Category (257)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__Category(struct soap*, const std::vector<ns1__Category * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__Category(struct soap*, const std::vector<ns1__Category * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__Category(struct soap*, const char*, int, const std::vector<ns1__Category * >*, const char*);
SOAP_FMAC3 std::vector<ns1__Category * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__Category * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__Category(struct soap*, const char*, std::vector<ns1__Category * >*, const char*);

SOAP_FMAC5 std::vector<ns1__Category * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__Category(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__Category(struct soap*, std::vector<ns1__Category * >*);
SOAP_FMAC5 std::vector<ns1__Category * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__Category(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__Category(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__AttachmentItemInfo
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__AttachmentItemInfo (249)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, const std::vector<ns1__AttachmentItemInfo * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, const std::vector<ns1__AttachmentItemInfo * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, const char*, int, const std::vector<ns1__AttachmentItemInfo * >*, const char*);
SOAP_FMAC3 std::vector<ns1__AttachmentItemInfo * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AttachmentItemInfo * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, const char*, std::vector<ns1__AttachmentItemInfo * >*, const char*);

SOAP_FMAC5 std::vector<ns1__AttachmentItemInfo * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, std::vector<ns1__AttachmentItemInfo * >*);
SOAP_FMAC5 std::vector<ns1__AttachmentItemInfo * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__AttachmentItemInfo(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__AddressBook
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__AddressBook (246)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, const std::vector<ns1__AddressBook * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, const std::vector<ns1__AddressBook * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, const char*, int, const std::vector<ns1__AddressBook * >*, const char*);
SOAP_FMAC3 std::vector<ns1__AddressBook * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AddressBook * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, const char*, std::vector<ns1__AddressBook * >*, const char*);

SOAP_FMAC5 std::vector<ns1__AddressBook * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, std::vector<ns1__AddressBook * >*);
SOAP_FMAC5 std::vector<ns1__AddressBook * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__AddressBook(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__AccessRightEntry
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__AccessRightEntry (243)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, const std::vector<ns1__AccessRightEntry * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, const std::vector<ns1__AccessRightEntry * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, const char*, int, const std::vector<ns1__AccessRightEntry * >*, const char*);
SOAP_FMAC3 std::vector<ns1__AccessRightEntry * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AccessRightEntry * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, const char*, std::vector<ns1__AccessRightEntry * >*, const char*);

SOAP_FMAC5 std::vector<ns1__AccessRightEntry * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, std::vector<ns1__AccessRightEntry * >*);
SOAP_FMAC5 std::vector<ns1__AccessRightEntry * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__AccessRightEntry(struct soap*, int, int, void*, const void*, size_t);

#ifndef SOAP_TYPE_std__vectorTemplateOfPointerTons1__AccessControlListEntry
#define SOAP_TYPE_std__vectorTemplateOfPointerTons1__AccessControlListEntry (238)
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_serialize_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >const*);
SOAP_FMAC3 void SOAP_FMAC4 soap_mark_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, const std::vector<ns1__AccessControlListEntry * >*);
SOAP_FMAC3 void SOAP_FMAC4 soap_default_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >*);
SOAP_FMAC3 int SOAP_FMAC4 soap_put_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, const std::vector<ns1__AccessControlListEntry * >*, const char*, const char*);
SOAP_FMAC3 int SOAP_FMAC4 soap_out_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, const char*, int, const std::vector<ns1__AccessControlListEntry * >*, const char*);
SOAP_FMAC3 std::vector<ns1__AccessControlListEntry * >* SOAP_FMAC4 soap_get_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >*, const char*, const char*);
SOAP_FMAC3 std::vector<ns1__AccessControlListEntry * >* SOAP_FMAC4 soap_in_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, const char*, std::vector<ns1__AccessControlListEntry * >*, const char*);

SOAP_FMAC5 std::vector<ns1__AccessControlListEntry * > * SOAP_FMAC6 soap_new_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, int);

SOAP_FMAC5 void SOAP_FMAC6 soap_delete_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, std::vector<ns1__AccessControlListEntry * >*);
SOAP_FMAC5 std::vector<ns1__AccessControlListEntry * > * SOAP_FMAC6 soap_instantiate_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, int, const char*, const char*, size_t*);
SOAP_FMAC5 void SOAP_FMAC6 soap_copy_std__vectorTemplateOfPointerTons1__AccessControlListEntry(struct soap*, int, int, void*, const void*, size_t);

SOAP_END_NAMESPACE(soap)

#endif

/* end of soapH.h */
